<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use PDO;
use PDOException;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;

class PlaceOrderController extends Controller
{
    public function place_order(Request $request){
        $outputData = [];
        $post = $request->post();
        extract($post);
        $cartData = $post['Order_Details']; // if submit order is directly call as an api
        //$cartData = $post; if submit order is called from place_order as an api

        // $logger = new Logger('placeorder');
        // $logger->pushHandler(new StreamHandler($_ENV['LOG_FILE_PATH'], $_ENV['LOG_MODE']));
        $this->log->addNotice('Payload of place_order API : ', ['post' => $post]);
        try {

            $this->pdo->beginTransaction();
            if(!empty($cartData)){

                ### Check customer address is valid before place order
                $buyerAddress = [];
                $msg = [];
                if(isset($cartData[0]) && !empty($cartData[0])){
                    $cd = $cartData[0];
                    $odCustNo = $cd['cust_no'];
                    $odCustNoAdd = $cd['address'];
                    $odCustNoBAdd = $cd['billaddress'];
                    $checkBuyerAddress = $this->pdo->prepare("SELECT Address_ID, Address_Type FROM `BBC_Addresses` WHERE User_No = ? AND Login_Type = ? AND (Address_ID = ? OR Address_ID = ? ) ");
                    $checkBuyerAddress->execute([$odCustNo, 'Buyer', $odCustNoAdd, $odCustNoBAdd]);
                    $buyerAddress = $checkBuyerAddress->fetchAll(PDO::FETCH_ASSOC);
                    if(!empty($buyerAddress)){
                        $buyerAddress = array_column($buyerAddress, 'Address_Type','Address_ID');
                        if(!array_key_exists($odCustNoAdd, $buyerAddress)){
                            $msg[] = 'Delivery';
                        }
                        if(!array_key_exists($odCustNoBAdd, $buyerAddress)){
                            $msg[] = 'Billing';
                        }
                    } else {
                        $msg = ['Delivery', 'Billing'];
                    }
                }
                if(!empty($msg)){
                    $this->log->addError('Response Error : ', ['msg' => "Invalid ".implode(', ', $msg)." address.",'status' => false,'data' => []]);
                    return  get_response("Invalid ".implode(', ', $msg)." address.", false, []);
                }

                ### Place order
                //$output[] = placeOrder1($this->pdo,$cartData);
                $parentorderNos=array();
                $firstStepOrderDetails = placeOrder1($this->pdo,$cartData,'N',$parentorderNos, $response); //$parentorderNos added by Meenal for oreder no linkage
                ## If items array is not present then return error
                if(empty($firstStepOrderDetails)){
                    $this->log->addError('Response Error : ', ['msg' => "Please provide valid articles",'status' => false,'data' => []]);
                    return  get_response("Please provide valid articles", false, []);
                }
                $output[] = $firstStepOrderDetails;

                list($custNo1, $storeNo1, $orderNo1, $prefDelDate1, $storeTotalAmount1, $storeTotalItems1, $custType1, $prefDelTimeFrom1, $prefDelTimeTo1) = $_SESSION['msgPostData'];
                $_SESSION['msgPostData'] = '';

                $orderDetails1 = getAllOrderDetails($firstStepOrderDetails);
                $orderDetailsArr1 = explode("__SK__", $orderDetails1);
                $orderNos = $orderDetailsArr1['0'];
                $parentorderNos = explode(',', $orderNos);
                $orderAmt = $orderDetailsArr1['1'];
                $newOrderAmt = $orderDetailsArr1['2'];

                $newOrderData = prepareOrderData1($this->pdo, $cartData);
                //print "<pre>"; print_r($newOrderData);

                if($newOrderData == false){
                    // no split order
                } else if(!empty($newOrderData)) {
                    //$output[] = placeOrder1($this->pdo,$newOrderData,'Y');
                    $secondStepOrderDetails = placeOrder1($this->pdo,$newOrderData,'Y',$parentorderNos, $response); //$parentorderNos added by Meenal for oreder no linkage

                    $output[] = $secondStepOrderDetails;

                    list($custNo2, $storeNo2, $orderNo2, $prefDelDate2, $storeTotalAmount2, $storeTotalItems2, $custType2, $prefDelTimeFrom2, $prefDelTimeTo2) = $_SESSION['msgPostData'];
                    $_SESSION['msgPostData'] = '';

                    /*$orderDetails2 = getAllOrderDetails($secondStepOrderDetails);
                    $orderDetailsArr2 = explode("__SK__", $orderDetails2);
                    $orderNos .= ",".$orderDetailsArr2['0']; // concat
                    $orderAmt += $orderDetailsArr2['1']; // sum
                    $newOrderAmt += $orderDetailsArr2['2']; //sum*/
                }
                $outputData['Order_No'] = $orderNos;
                $outputData['Order_Tot_Amt'] = $orderAmt;
                $outputData['Order_Net_Tot_Amt'] = $newOrderAmt;
                //$outputData['output'] = $output;


                $this->pdo->commit();

                ### SEND SMS AND EMAIL #############

                ### SEND SMS and EMAIL LEG 1

                if($custNo1 > 0 &&  $storeNo1 > 0){
                    createLogPO([ '1st leg order = ', $custNo1, $storeNo1 ]);
                    $msgPosted = postSmsTweet1($this->pdo, $custNo1, $storeNo1, $orderNo1, $prefDelDate1, $storeTotalAmount1, $storeTotalItems1, $custType1, $prefDelTimeFrom1, $prefDelTimeTo1);
                }
                if($custNo2 > 0 &&  $storeNo2 > 0){
                    createLogPO([ '2nd leg order = ', $custNo1, $storeNo1 ]);
                    $msgPosted = postSmsTweet1($this->pdo, $custNo2, $storeNo2, $orderNo2, $prefDelDate2, $storeTotalAmount2, $storeTotalItems2, $custType2, $prefDelTimeFrom2, $prefDelTimeTo2);
                }

                if(!empty($cartData)){
                    $custArr = [];
                    foreach ($cartData as $key => $cart) {
                        $custArr[] = $cart['cust_no'];
                        # code...
                    }
                    $custArr = array_unique($custArr);
                    send_api_notification($this->pdo,'place_order', implode(',', $custArr), 'Self');
                }

                createLogPO([ 'Order Placed Successfully------------------' ]);
                $this->log->addNotice('Response of place_order API: ', ['msg' => "Order Placed Successfully",'status' => true,'data' => $outputData]);
                return get_response("Order Placed Successfully", true, $outputData);
            }

        } catch (PDOException $e) {

            createLogPO([ ' place order = Something went wrong ']);
            $this->pdo->rollback();
            $data = $e->getMessage();

            if(!empty($cartData)){
                $custArr = [];
                foreach ($cartData as $key => $cart) {
                    $custArr[] = $cart['cust_no'];
                    # code...
                }
                $custArr = array_unique($custArr);
                send_api_notification($this->pdo,'place_order', implode(',', $custArr), 'Self');
            }
            $this->log->addError('Response Error : ', ['msg' => $data,'status' => false,'data' => []]);
            return  get_response($data, false, []);
        }
    }

    private function getAllOrderDetails($placedOrderDetails){
        Log::notice('Request Data for place_order => getAllOrderDetails', ['request' => $placedOrderDetails]);
        $orderNoArr = [];
        $orderAmtArr = [];
        $netOrderAmtArr = [];
        if(!empty($placedOrderDetails)){
            foreach ($placedOrderDetails as $key => $details) {
                $orderNoArr[$key] = $details['Order_No'];
                $orderAmtArr[$key] = $details['Order_Amount'];
                $netOrderAmtArr[$key] = $details['Net_Amount'];
            }
        }
        $orderNos = implode(",", $orderNoArr);
        //echo $orderNos,$orderNoArr;
        $totalOrderAmount = array_sum($orderAmtArr);
        $totalNewOrderAmount = array_sum($netOrderAmtArr);
        Log::notice('Response Data for place_order => getAllOrderDetails', ['response' => $orderNos."__SK__".$totalOrderAmount."__SK__".$totalNewOrderAmount]);
        return $orderNos."__SK__".$totalOrderAmount."__SK__".$totalNewOrderAmount;
    }

    private function prepareOrderData1($pdo, $cartData){
        Log::notice('Request Data for place_order => prepareOrderData1', ['request' => $cartData]);

        $newOrderData = [];

        foreach ($cartData as $key => $orderData) {

            $storeNo = $orderData['store_no'];

            $stmt = $pdo->prepare("SELECT
        SM.`Cust_No`, SM.`Split_Order` FROM `Store_Master` as SM WHERE SM.`Status` = ? AND SM.`Store_No` = ?
        ORDER BY SM.`Store_Name`");
            $stmt->execute(['A', $storeNo]);
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            if(!empty($row)){
                $custNo = $row['Cust_No'];
                $splitOrder = $row['Split_Order'];
            }

            if($splitOrder == 'N') return false;

            $newOrderData[$key]['cust_no'] = $custNo;
            $newOrderData[$key]['address'] = $orderData['address'];
            $newOrderData[$key]['billaddress'] = $orderData['billaddress'];
            $newOrderData[$key]['Business_ID'] = $orderData['Business_ID']; //mmb-android-pricing

            //Added by for capturing latitude and longitude
            $newOrderData[$key]['latitude']  = $orderData['latitude'] ?? "";
            $newOrderData[$key]['longitude'] = $orderData['longitude'] ?? "";

            $newOrderData[$key]['default_store'] = $orderData['default_store'];
            $newOrderData[$key]['payment_type'] = $orderData['payment_type'];
            if(isset($orderData['set_product_favourite'])){
                $newOrderData[$key]['set_product_favourite'] = $orderData['set_product_favourite'];
            } else {
                $newOrderData[$key]['set_product_favourite'] = 0;
            }
            $newOrderData[$key]['delivery_type'] = $orderData['delivery_type'];

            ### Get store name with using brand name from Brand_Store_Master

            $newStoreArr = [];
            $newOrderData[$key]['stores']['0']['delivery_charge'] = $orderData['stores']['0']['delivery_charge'];
            $newOrderData[$key]['stores']['0']['promo_code'] = $orderData['stores']['0']['promo_code'];
            $newOrderData[$key]['stores']['0']['discount_amount'] = $orderData['stores']['0']['discount_amount'];
            $newOrderData[$key]['stores']['0']['custType'] = $orderData['stores']['0']['custType'];



            $items = $orderData['stores']['0']['items'];

            if(!empty($items)){
                $newItemArr = [];
                foreach ($items as $k => $item) {
                    # code...
                    $articleNo = $item['articleId'];
                    $quantity = $item['quantity'];
                    //$brand_name = $item['brand_name'];
                    $newStoreNo=$item['brand_name'];
                    //Commented by Meenal Barve as Brand Name now contains split_Store No
                    ### Get 2nd steps order Store No's
                    /*$stmtB = $pdo->prepare("SELECT
                    BSM.`Store_No` FROM `Brand_Store_Master` as BSM Join Brand_Master As BM on BM.`Brand_ID`=BSM.`Brand_ID` WHERE BM.`Brand_Desc` = ? ");
                    $stmtB->execute([$brand_name]);
                    $rowB = $stmtB->fetch(PDO::FETCH_ASSOC);
                    $newStoreNo = $rowB['Store_No'];
                    $newOrderData[$key]['store_no'] = $newStoreNo;*/
                    $newOrderData[$key]['stores']['0']['store_no'] = $newStoreNo;

                    //Added by Meenal Barve for Self ordering in case articles are not listed properly
                    //sss-sendcollection

                    ## If newStoreNo/ brand_name is zero or empty then derive it from Store_city_article table
                    ## Else assign storeNo
                    if ($newStoreNo==0 || $newStoreNo=='') {
                        $stmtSplitStoreNo = $pdo->prepare("SELECT
			        Split_Store_No FROM `Store_City_Article` WHERE Article_No = ? AND Store_No = ? ");
                        $stmtSplitStoreNo->execute([$articleNo, $storeNo]);
                        $splitStoreNo = $stmtSplitStoreNo->fetch(PDO::FETCH_ASSOC);

                        if(!empty($splitStoreNo)){
                            $newStoreNo	= $splitStoreNo['Split_Store_No'];
                            $newOrderData[$key]['stores']['0']['store_no'] = $newStoreNo;
                        } else {
                            $newStoreNo=$storeNo;
                            $newOrderData[$key]['stores']['0']['store_no'] = $storeNo;
                        }

                        $newOrderData[$key]['stores']['0']['add_legacy_order_no'] = 1;
                    }


                    ### Get prices from store listing from storeNo2 and custNo
                    //mmb-android-pricing price to be fetched from Article Margins table
                    /*$stmtA = $pdo->prepare("select Offer_Price, Unit_Price from Store_Listing where Store_No=? and Article_No=?");
                    $stmtA->execute([$newStoreNo, $articleNo]);
                    $rowA = $stmtA->fetch(PDO::FETCH_ASSOC);




                    if(!empty($rowA)) {
                        $newOfferPrice = $rowA['Offer_Price'];
                        $newUnitPrice = $rowA['Unit_Price'];
                    } else {
                        //$stmtA->debugDumpParams(); exit;
                        throw new PDOException($articleNo." Article/s are not listed properly!");
                    } */

                    //$newStoreArr[$k]['offer_price'] = $newOfferPrice;  //mmb-android-pricing
                    $newStoreArr[$k]['offer_price'] =$item['offer_price'];
                    $newStoreArr[$k]['articleId'] = $articleNo;
                    $newStoreArr[$k]['quantity'] = $quantity;
                    //$newStoreArr[$k]['unit_price'] = $newUnitPrice;    //mmb-android-pricing
                    $newStoreArr[$k]['unit_price'] = $item['unit_price'];
                    $newStoreArr[$k]['brand_name'] = $brand_name ?? "";
                }
                $newOrderData[$key]['stores']['0']['items'] = $newStoreArr;
            }
        }
        Log::notice('Response Data for place_order => prepareOrderData1', ['response' => $newOrderData]);

        return $newOrderData;
    }

    private function placeOrder1($pdo, $cartData, $split, $parentorderNos, $response){
        Log::notice('Request for place_order => placeOrder1 ', ['cartData' => $cartData, 'split' => $split, 'parentorderNos' => $parentorderNos, 'response' => $response]);
        $output = [];
        $Store_No_Leg2 = '';

        $src = $_REQUEST['src'] ?? "";
        if(!empty($src)){
            $orderSource="wl_android";
        }else{
            $orderSource="android";
        }
        //echo "*****";
        //echo $parentorderNos;
        $p=0;

        foreach ($cartData as $key => $cart_data) {

            $kk = 0;
            $time_slot = '';
            $Pref_Del_Date = '';
            $Pref_Del_Time_From = ''; $Pref_Del_Time_To = '';
            $removed_array = array();
            //mmb_android-pricingp2 start
            $businessno = $cart_data['Business_ID'] ?? "";
            //echo "****".$businessno;
            $stmt = $pdo->prepare("Select Config_Value from Business_Config_Master
				where Business_Id = ?
				and   Config_Attribute='PRICING'
				and   Status='A'");
            $stmt->execute([$businessno]);
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            if(!empty($row)){
                $pricingMode = $row['Config_Value'];
            }
            else $pricingMode ='';

            //mmb_android_pricingp2 End

            if(!empty($cart_data['stores']))
                for ($i = 0; $i < sizeof($cart_data['stores']); $i++) {
                    $custType   = $cart_data['stores'][$i]['custType'] ?? "";
                    $fsoNo      = $cart_data['stores'][$i]['fsoNo'] ?? "0";
                    $store_no = $cart_data['stores'][$i]['store_no'];
                    $delivery_charge = $cart_data['stores'][$i]['delivery_charge'];
                    $promo_code = $cart_data['stores'][$i]['promo_code'];
                    $discount_amount = $cart_data['stores'][$i]['discount_amount'];
                    //Added to capture latitude , longitude
                    $latitude  = $cart_data['stores'][$i]['latitude'] ?? "";
                    $longitude = $cart_data['stores'][$i]['longitude'] ?? "";
                    //$txnId = $cart_data['txnid'];

                    $payment_type = strtolower($cart_data['payment_type']);
                    if ($payment_type == 'billdesk') {
                        $payment_type = "online_" . $payment_type;
                        //$payment_type = $payment_type;
                    }

                    $delivery_type = $cart_data['delivery_type'];
                    if ($delivery_type == "home_delivery") {
                        $delivery_type = 1;
                    } else {
                        $delivery_type = 2;
                    }

                    if(isset($cart_data['stores'][$i]['time_slot']) ){
                        $time_slot = explode('-', $cart_data['stores'][$i]['time_slot']);
                        $Pref_Del_Time_From = $time_slot[0];
                        $Pref_Del_Time_To = $time_slot[1];
                    }

                    $parent_order_no=""; //null for 1st leg
                    ### If Order is split then update data for 2nd leg order
                    if($split == 'Y'){
                        $payment_type = 'NA';
                        $delivery_type = 2;
                        $delivery_charge = 0;
                        $promo_code = "";
                        $discount_amount = 0;
                        $orderSource = 'Auto';
                        $custType = 'Self';
                        $parent_order_no=$parentorderNos[$p++]; //get Order No of first leg at index i

                    }


                    $items = array();
                    $favourite_cart = array();
                    $favourite = array();

                    $custNo = $cart_data['cust_no'];

                    if(isset($cart_data['stores'][$i]['items']) && (empty($cart_data['stores'][$i]['items']))){
                        return [];
                    }

                    for ($j = 0; $j < sizeof($cart_data['stores'][$i]['items']); $j++) {

                        $offer_price = $cart_data['stores'][$i]['items'][$j]['offer_price'];
                        $article_no = $cart_data['stores'][$i]['items'][$j]['articleId'];
                        $quantity = $cart_data['stores'][$i]['items'][$j]['quantity'];
                        $unit_price = $cart_data['stores'][$i]['items'][$j]['unit_price'];
                        $scheme_id = $cart_data['stores'][$i]['items'][$j]['scheme_id'] ?? ""; //Added by Meenal B. for customer based pricing
                        ###Get 2nd Leg Store No for Finding Shipping Address of 1st leg ###
                        try{
                            //Commented by Meenal Barve as Brand Name now contains split_Store_No
                            $stmt = $pdo->prepare("SELECT
					 SM.`Split_Order` ,SM.`Store_Name` FROM `Store_Master` as SM WHERE SM.`Status` = ? AND SM.`Store_No` = ?
					ORDER BY SM.`Store_Name`");
                            $stmt->execute(['A', $store_no]);
                            $row = $stmt->fetch(PDO::FETCH_ASSOC);
                            if(!empty($row)){
                                $splitOrder = $row['Split_Order'];
                                $StoreName  = $row['Store_Name']; //For MFG
                            }
                            else  {
                                $splitOrder ='';
                                $StoreName  = ''; //For MFG
                            }
                            if ($splitOrder =='Y')  //dev-17 dervie address bug
                                //if($split == 'Y') {
                                $Store_No_Leg2 = $cart_data['stores'][$i]['items'][$j]['brand_name']; //brand name contains store no.
                            else //Added by Meenal B. for family mart
                                $Store_No_Leg2 = '';
                        } catch (PDOException $e) {
                            $Store_No_Leg2='';
                        }
                        ###Get 2nd Leg Store No for Finding Shipping Address of 1st leg ###


                        ### If Order is split then Get Slab Pricing if available
                        if($split == 'Y'){

                            // sph-offlinepilot-01
                            $queryArticleSchemes = $pdo->prepare("SELECT Store_No, Business_No, Cust_No, Cust_Type, Offer_Price, Offer_Rate, Unit_Price, Unit_Rate, Scheme_Price, Scheme_ID, Cust_Margin
                    	FROM Store_Cust_Article_Price SCAP
                    	WHERE SCAP.Article_No = '$article_no'
                    	AND SCAP.Store_No = '$store_no'
                    	AND ( SCAP.Cust_No = '$custNo'
                    	OR (SCAP.Cust_No = 0
                    	AND SCAP.Cust_Type = (SELECT Get_Cust_Type('$custNo', '$store_no') as Cust_Type)))
                    	ORDER BY SCAP.Cust_No DESC");

                            $queryArticleSchemes->execute();
                            $resultArticleSchemes  = $queryArticleSchemes->fetchAll(PDO::FETCH_ASSOC);
                            $queryArticleSchemes->closeCursor();

                            //$dataArticleSchemes = $resultArticleSchemes[0];
                            $unit_price  = $resultArticleSchemes[0]['Unit_Price']; //mmb-android-pricing
                            if ($resultArticleSchemes[0]['Scheme_ID']=='0')
                                $offer_price = $resultArticleSchemes[0]['Offer_Price'];
                            else
                                $offer_price = $resultArticleSchemes[0]['Scheme_Price'];
                            //$row['offer_rate']	= $dataArticleSchemes['Offer_Rate'];
                            //$row['scheme_price']= $dataArticleSchemes['Scheme_Price'];
                            $scheme_id   = $resultArticleSchemes[0]['Scheme_ID'];
                            //$row['scheme_name'] = $dataArticleSchemes['Scheme_Name'];


                        }
                        if($offer_price === 0 && $split == 'Y'){
                            $offer_price = $cart_data['stores'][$i]['items'][$j]['offer_price'];
                            $unit_price = $cart_data['stores'][$i]['items'][$j]['unit_price']; //mmb-android-pricing
                        }

                        $tempFavourite = array();
                        $favourite_cart['Article_No'] = $article_no;
                        $favourite_cart['Store_No']   = $store_no;

                        #### Check article is listed
                        //$stmt = $pdo->prepare("select Offer_Price from Store_Listing where Store_No=? and Article_No=?");
                        //Offer price no longer stored in StoreListing after pricing module implementation
                        $stmt = $pdo->prepare("select Article_No from Store_Listing where Store_No=? and Article_No=?");
                        $stmt->execute([$store_no, $article_no]);
                        $result = $stmt->fetch(PDO::FETCH_ASSOC);
                        //$stmt->debugDumpParams();

                        if(empty($result))
                        {
                            $data = [];
                            //Commented by Meenal B. sss-sendcollections to avoid 2 responses
                            //return  getResponse($response, $article_no." Article is not available in this store", false, $data);
                            $GLOBALS['containers']->log->addError('Response Error for place_order => placeOrder1 : ', ['result' => $article_no." Article is not available in this store"]);

                            throw new PDOException($article_no." Article is not available in this store");
                        }


                        $items[$j]['Article_No'] = $article_no;
                        //echo $offer_price;echo $unit_price;
                        //echo "****";

                        $items[$j]['Offer_Price'] = $offer_price;
                        $items[$j]['Unit_Price'] = $unit_price;
                        $items[$j]['Quantity'] = $quantity;
                        $items[$j]['Scheme_ID'] = $scheme_id; //Added by Meenal B. for customer based pricing


                        ##### Calculate approx weight
                        $stmt1 = $pdo->prepare("SELECT Article_No, Approx_Weight FROM `Article_Master` WHERE Article_No = ? ");
                        $stmt1->execute([$article_no]);
                        $result1 = $stmt1->fetchAll(PDO::FETCH_ASSOC);
                        //$stmt1->debugDumpParams();

                        $Approx_Weight = [];
                        $row1=$result1;
                        for ($a=0; $a <count($row1) ; $a++) {
                            $Approx_Weight = $row1[$a]['Approx_Weight'] * $quantity;
                        }
                        $items[$j]['Approx_Weight'] = $Approx_Weight;

                        #### Create array of remove articles
                        $removed_array[$kk]['Article_No'] = $article_no;
                        $removed_array[$kk]['Store_No'] = $store_no;

                        $kk = $kk + 1;
                    }

                    ##### Add security check

                    #### Total order amount should not be grater than 1 lakh
                    /*if(calculateOrderAmount1($items) >= 100000){
                        $return = array();
                        $return['message'] = "Order amount should not be exceeding 1 lakh";
                        $return['flag'] = 0;
                        return $return;
                    }*/


                    #### INSERT/UPDATE favourite items
                    if(isset($cart_data['set_product_favourite']) && $cart_data['set_product_favourite'] == 1){
                        $this->updateFavouriteProducts($pdo, $custNo, $store_no, $favourite_cart, 'cart_data');
                    }


                    #### Prepare Order Code
                    ## Get Indian Finincial year
                    //$current_year = ( date('m') > 6) ? date('Y') + 1 : date('Y');
                    $current_year = ( date('m') < 4) ? date('Y') - 1 : date('Y') ;
                    $stmt2 = $pdo->prepare("select * from Order_Hdr_Year where Year=?");
                    $stmt2->execute([$current_year]);
                    $year_data = $stmt2->fetch(PDO::FETCH_ASSOC);
                    $Current_ID = $year_data['Current_ID'] + 1;
                    $OrderCode = $year_data['Year_ID'] . $year_data['Start_ID'] .$Current_ID;
                    $Reference_ID = str_pad(mt_rand(0, 999999), 6, '0', STR_PAD_LEFT);
                    $Order_No = $OrderCode;
                    $Order_Code = $OrderCode;
                    $Cust_No = $cart_data['cust_no'];
                    $Store_No = $store_no;
                    $Order_Date = date('Y-m-d');
                    $Order_Time = date('H:i:s');
                    $Address_ID = $cart_data['address'];
                    $BillAddress_ID = $cart_data['billaddress'];
                    ########Added to get the default Supplier Shipping and Billing Address
                    $stmt2 = $pdo->prepare("select Get_Shipping_Address(?) as Address_ID");
                    $stmt2->execute([$Store_No]);
                    $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                    $ShippingAddress_ID = $srow['Address_ID'];
                    $stmt2 = $pdo->prepare("select Get_Billing_Address(?,?) as Address_ID");
                    $stmt2->execute([$Store_No,$ShippingAddress_ID]);
                    $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                    $ShippingBillAddress_ID=$srow['Address_ID'];
                    $stmt2 = $pdo->prepare("select Order_Processing from Store_Master where Store_No = ?");
                    $stmt2->execute([$Store_No]);
                    $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                    $Order_Processing=$srow['Order_Processing'];
                    #######
                    //$ShippingAddress_ID='';
                    //$ShippingBillAddress_ID='';
                    if ($Order_Processing =='Local') {
                        ############If Local only then calculate the Supplier warehouse address
                        ############If Central then get Default Shipping and Billing Address
                        ############If 1st leg Order then calculate Shipping Address###########
                        if ($orderSource != 'Auto'){
                            #Get Shipping Address ID for Leg 2
                            //if ($split=='Y')  { //If Split='Y' then calculate Shipping Address id for leg 2 using brand store
                            if ($splitOrder=='Y')  { //dev-17 derive address bug
                                //$ShippingAddress_ID = get_Derived_Address($pdo,$Store_No_Leg2,$Address_ID,'Shipping'); //sph-b2c-bugfix
                                $stmt2 = $pdo->prepare("select Get_Derived_Address(?,?,?) as address");
                                $stmt2->execute([$Store_No_Leg2,$Address_ID,'Shipping']);
                                $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                                $ShippingAddress_ID = $srow['address'];
                            }
                            else {
                                $ShippingAddress_ID = $Address_ID; //if Split is N, then derive shipping address for Customer's delivery address
                            }
                            #Get Shipping Address ID for Leg 1,Store No is AutoD
                            //$ShippingAddress_ID = get_Derived_Address($pdo,$Store_No,$ShippingAddress_ID,'Shipping'); //sph-b2c-bugfix
                            $stmt2 = $pdo->prepare("select Get_Derived_Address(?,?,?) as address");
                            $stmt2->execute([$Store_No,$ShippingAddress_ID,'Shipping']);
                            $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                            $ShippingAddress_ID = $srow['address'];
                            #Get Billing Address corresponding to Shipping Address of AutoD
                            //$ShippingBillAddress_ID = get_Billing_Address($pdo,$Store_No,$ShippingAddress_ID); //sph-b2c-bugfix
                            $stmt2 = $pdo->prepare("select Get_Billing_Address(?,?) as address");
                            $stmt2->execute([$Store_No,$ShippingAddress_ID]);
                            $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                            $ShippingBillAddress_ID = $srow['address'];
                        }
                        else {
                            ###Get Shipping Address ID for Store with Customer Address ID in Leg 1
                            //$ShippingAddress_ID = get_Derived_Address($pdo,$Store_No,$Address_ID,'Shipping'); //sph-b2c-bugfix
                            $stmt2 = $pdo->prepare("select Get_Derived_Address(?,?,?) as address");
                            $stmt2->execute([$Store_No,$Address_ID,'Shipping']);
                            $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                            $ShippingAddress_ID = $srow['address'];
                            ###Get Delivery Address ID for Customer(AutoD) from Shipping Address ID
                            //$Address_ID = get_Derived_Address($pdo,$Cust_No,$ShippingAddress_ID,'Delivery');  //sph-b2c-bugfix
                            $stmt2 = $pdo->prepare("select Get_Derived_Address(?,?,?) as address");
                            $stmt2->execute([$Cust_No,$ShippingAddress_ID,'Delivery']);
                            $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                            $Address_ID = $srow['address'];
                            ###Set Billing Address ID for Customer(AutoD) same as Delivery Address
                            $BillAddress_ID = $Address_ID;
                            ###Get Billing Address ID for Customer(AutoD) from
                            ##$ShippingBillAddress_ID = get_Billing_Address($pdo,$Store_No_Leg2,$ShippingAddress_ID);
                            #Above code commented as for Leg2 $Store_No_Leg2 is null ,so use $store_no
                            //$ShippingBillAddress_ID = get_Billing_Address($pdo,$store_no,$ShippingAddress_ID); //sph-b2c-bugfix
                            $stmt2 = $pdo->prepare("select Get_Billing_Address(?,?) as address");
                            $stmt2->execute([$store_no,$ShippingAddress_ID]);
                            $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                            $ShippingBillAddress_ID = $srow['address'];
                            #echo "Shipping Address ID".$ShippingAddress_ID."Store".$Store_No_Leg2."\n";
                            #echo "Billing Address ID".$ShippingBillAddress_ID;
                        }
                    }


                    ######### Calculate pref_del_date ####START#############
                    ### if pref_date is coming from input data
                    if(isset($cart_data['stores'][$i]['pref_date'])) {
                        $datea = $cart_data['stores'][$i]['pref_date'];
                        $dateaa = new DateTime($datea);
                        $Pref_Del_Date = $dateaa->format('Y-m-d');
                    }
                    //echo "Step1".$Pref_Del_Date;
                    ### if pref_date is not coming from input data, then calculate it
                    if($Pref_Del_Date == ''){

                        $Pref_Del_Date = $this->calculatePrefDeliveryDate1($pdo, $store_no, $Cust_No, $Address_ID);
                        //echo "Step2".$Pref_Del_Date;
                    }

                    ######### Calculate pref_del_date ####END#############

                    if($fsoNo > 0){
                        $Order_Status = 'Accepted';
                    }else{
                        $Order_Status = 'Open';
                    }

                    $payment_status = 'Pending';

                    #Check payment type is online or billdesk and change payment_type to pending#
                    /*if (isset($payment_type) && $payment_type == 'online_billdesk')
                    {
                        $payment_status = 'Pending';
                    } */

                    ##### Insert into Order_Hdr
                    $query = "INSERT INTO Order_Hdr( Del_Type,
				                                 Order_No,
												 Cust_No,
												 Order_Code,
												 Reference_ID,
												 Address_ID,
												 Bill_Address_ID,
												 Supplier_Address_ID,
												 Supplier_Bill_Address_ID,
												 Store_No,
												 Order_Date,
												 Order_Time,
												 Pref_Del_Date,
												 Order_Status,
												 Payment_Type,
												 Source,
												 Cust_Type,
												 FSO_No,
												 Creation_Date,
												 Creation_Time,
												 Created_By,
												 Parent_Order_No,
												 Latitude,
												 Longitude ";
                    if($payment_status != '') {
                        $query.= ", Payment_Status ";
                    }
                    if($time_slot != ''){
                        $query.= ", Pref_Del_Time_From, Pref_Del_Time_To ";
                    }

                    $query .= " ) values( ?, ?, ? ,? , ? , ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? ,? , ? , ? , ?";

                    if($payment_status != '') {
                        $query .= ", ? ";
                    }
                    if($time_slot != ''){
                        $query .= ", ? , ? ";
                    }
                    $query .= " ) ";

                    $stmt = $pdo->prepare($query);

                    $inputOrderData = array($delivery_type, $Order_No, $Cust_No, $Order_Code, $Reference_ID, $Address_ID, $BillAddress_ID,$ShippingAddress_ID,$ShippingBillAddress_ID,$Store_No, $Order_Date, $Order_Time, $Pref_Del_Date, $Order_Status, $payment_type, $orderSource, $custType, $fsoNo, $Order_Date, $Order_Time, $Cust_No,$parent_order_no,$latitude,$longitude);
                    if($payment_status != '') {
                        array_push($inputOrderData,$payment_status);
                    }
                    if($time_slot != ''){
                        array_push($inputOrderData, $Pref_Del_Time_From, $Pref_Del_Time_To);
                    }

                    $stmt->execute($inputOrderData);

                    /*print "<br>";
                    $stmt->debugDumpParams();*/

                    #####INSERT into Order_Addresses to store snapshot address
                    //For Seller , dev-14
                    $queryAdd ="INSERT INTO Order_Addresses
				(`Order_No`,
				 `Address_ID`,
				 `Address_Type`,
				 `Ref_Address_ID`,
				 `User_No`,
				 `Login_Type`,
				 `GSTIN`,
				 `Name`,
				 `Email_ID`,
				 `Address`,
				 `Flat_No`,
				 `Locality`,
				 `Pincode`,
				 `City_Code`,
				 `State_Code`,
				 `Latitude`,
				 `Longitude`,
				 `Contact_No`,
				 `Contact_No_Type`,
				 `Status`,
				 `Default_Flag`,
				 `Creation_Date`,
				 `Created_By`,
				 `Changed_Date`,
				 `Changed_By`,
				 `Landmark`)
					SELECT
					? ,
					`Address_ID`,
					`Address_Type`,
					`Ref_Address_ID`,
					`User_No`,
					`Login_Type`,
					`GSTIN`,
					`Name`,
					`Email_ID`,
					`Address`,
					`Flat_No`,
					`Locality`,
					`Pincode`,
					`City_Code`,
					`State_Code`,
					`Latitude`,
					`Longitude`,
					`Contact_No`,
					`Contact_No_Type`,
					`Status`,
					`Default_Flag`,
					`Creation_Date`,
					`Created_By`,
					`Changed_Date`,
					`Changed_By`,
					`Landmark` FROM `BBC_Addresses` where Address_ID IN (?, ?)";
                    $stmtOrderAdd = $pdo->prepare($queryAdd);
                    $stmtOrderAdd->execute([$Order_No,$ShippingAddress_ID,$ShippingBillAddress_ID]);
                    //For Buyer - Delivery Address
                    $queryAdd ="INSERT INTO Order_Addresses
				(`Order_No`,
				 `Address_ID`,
				 `Address_Type`,
				 `Ref_Address_ID`,
				 `User_No`,
				 `Login_Type`,
				 `GSTIN`,
				 `Name`,
				 `Email_ID`,
				 `Address`,
				 `Flat_No`,
				 `Locality`,
				 `Pincode`,
				 `City_Code`,
				 `State_Code`,
				 `Latitude`,
				 `Longitude`,
				 `Contact_No`,
				 `Contact_No_Type`,
				 `Status`,
				 `Default_Flag`,
				 `Creation_Date`,
				 `Created_By`,
				 `Changed_Date`,
				 `Changed_By`,
				 `Landmark`)
					SELECT
					? ,
					`Address_ID`,
					'Delivery',
					`Ref_Address_ID`,
					`User_No`,
					`Login_Type`,
					`GSTIN`,
					`Name`,
					`Email_ID`,
					`Address`,
					`Flat_No`,
					`Locality`,
					`Pincode`,
					`City_Code`,
					`State_Code`,
					`Latitude`,
					`Longitude`,
					`Contact_No`,
					`Contact_No_Type`,
					`Status`,
					`Default_Flag`,
					`Creation_Date`,
					`Created_By`,
					`Changed_Date`,
					`Changed_By`,
					`Landmark` FROM `BBC_Addresses` where Address_ID IN (?)";
                    $stmtOrderAdd = $pdo->prepare($queryAdd);
                    $stmtOrderAdd->execute([$Order_No,$Address_ID]);

                    //For Buyer - Billing Address
                    $queryAdd ="INSERT INTO Order_Addresses
				(`Order_No`,
				 `Address_ID`,
				 `Address_Type`,
				 `Ref_Address_ID`,
				 `User_No`,
				 `Login_Type`,
				 `GSTIN`,
				 `Name`,
				 `Email_ID`,
				 `Address`,
				 `Flat_No`,
				 `Locality`,
				 `Pincode`,
				 `City_Code`,
				 `State_Code`,
				 `Latitude`,
				 `Longitude`,
				 `Contact_No`,
				 `Contact_No_Type`,
				 `Status`,
				 `Default_Flag`,
				 `Creation_Date`,
				 `Created_By`,
				 `Changed_Date`,
				 `Changed_By`,
				 `Landmark`)
					SELECT
					? ,
					`Address_ID`,
					'Billing',
					`Ref_Address_ID`,
					`User_No`,
					`Login_Type`,
					`GSTIN`,
					`Name`,
					`Email_ID`,
					`Address`,
					`Flat_No`,
					`Locality`,
					`Pincode`,
					`City_Code`,
					`State_Code`,
					`Latitude`,
					`Longitude`,
					`Contact_No`,
					`Contact_No_Type`,
					`Status`,
					`Default_Flag`,
					`Creation_Date`,
					`Created_By`,
					`Changed_Date`,
					`Changed_By`,
					`Landmark` FROM `BBC_Addresses` where Address_ID IN (?)";
                    $stmtOrderAdd = $pdo->prepare($queryAdd);
                    $stmtOrderAdd->execute([$Order_No,$BillAddress_ID]);
                    ######################

                    $weight = 0;
                    $total_weight = 0;
                    $total_amount = 0;
                    $total_saving_amount = 0;
                    $store_total_items = 0;
                    $store_total_amount = 0;

                    for ($k = 0; $k < sizeof($items); $k++) {

                        $Order_No = $Order_No;
                        $Article_No = $items[$k]['Article_No'];
                        $Order_Quantity = $items[$k]['Quantity'];
                        $Order_Amount = $items[$k]['Quantity'] * $items[$k]['Offer_Price'];
                        $saving = $items[$k]['Unit_Price'] - $items[$k]['Offer_Price'];
                        $Saving_Amount = $items[$k]['Quantity'] * $saving;
                        $Order_Offer_Price = $items[$k]['Offer_Price'];
                        $Order_Mrp = $items[$k]['Unit_Price'];
                        $Item_Weight = $items[$k]['Approx_Weight'];
                        $OrderScheme_ID = $items[$k]['Scheme_ID'];
                        ##### Insert into Order_Itm
                        $stmtItem = $pdo->prepare("insert into Order_Itm(Order_No,Article_No,Order_Quantity,Order_Amount,Saving_Amount,Order_Offer_Price,Order_Mrp,Item_Weight,Scheme_ID,Creation_Date,Creation_Time,Created_By)
						values(?,?,?,?,?,?,?,?,?,?,?,?)");
                        $stmtItem->execute([$Order_No, $Article_No, $Order_Quantity, $Order_Amount, $Saving_Amount, $Order_Offer_Price, $Order_Mrp, $Item_Weight, $OrderScheme_ID,$Order_Date, $Order_Time,$Cust_No]);

                        ### Get total weight and amount
                        $total_weight = $total_weight + $Item_Weight;
                        $total_amount = $total_amount + $Order_Amount;
                        $total_saving_amount = $total_saving_amount + $Saving_Amount;
                        $store_total_items = $store_total_items + 1;
                        $store_total_amount = $store_total_amount + $Order_Amount;
                    }


                    #### If promo code applied
                    if ($promo_code != "") {
                        $Delivery_Charges = $delivery_charge;
                        $order_amount = $total_amount;
                        $total_amount = $total_amount + $delivery_charge;
                        $Order_Amount = $total_amount;
                        $Promo_Code = $promo_code;
                        $Header_Discount = $discount_amount;
                        $Net_Amount = $total_amount - $discount_amount;
                        $total_saving_amount = $total_saving_amount+$discount_amount;
                    } else {

                        $Delivery_Charges = $delivery_charge;
                        $total_amount = $total_amount + $delivery_charge;
                        $Order_Amount = $total_amount;
                        $Promo_Code = '';
                        $Header_Discount = '0';
                        $Net_Amount = $total_amount;
                    }

                    $legacyOrderNo = null;
                    if($cart_data['stores'][$i]['add_legacy_order_no'] == 1){
                        $legacyOrderNo = $Order_No;
                    }

                    //Order Type
                    $orderType = '';
                    //Logic for MFG Start
                    if ($StoreName =='CBM (Bhilwadi)' ||
                        $StoreName =='B G Chitale' ||
                        $StoreName =='Chitale Foods' ||
                        $StoreName =='Chitale Agro Industries P Ltd' ||
                        $StoreName =='CBM Bhilwadi ( Super SS)' ) {

                        $orderType = 'MFG';
                    }
                    //End
                    $stmtUOH = $pdo->prepare("update Order_Hdr set Order_Amount=?,Promo_Code=? ,Header_Discount=?,Net_Amount=?, Delivery_Charges=?,Saving_Amount=?, Total_Weight=?, Change_Date=?, Change_Time=?, Changed_By=?, Legacy_Order_No=?, Order_Type=? where Order_No=?");
                    $stmtUOH->execute([$Order_Amount, $Promo_Code, $Header_Discount, $Net_Amount, $Delivery_Charges, $total_saving_amount, $total_weight, $Order_Date, $Order_Time, $Cust_No, $legacyOrderNo, $orderType, $Order_No]);
                    //$stmtUOH->debugDumpParams();


                    /*$stmtU = $pdo->prepare("update Promo_Hdr set Total_Redeemption=Total_Redeemption+1 where Promo_Code=? and Store_No=?");
                    $stmtU->execute([$promo_code, $Store_No]);*/

                    ###### B2C Market Place Changes Start  //sph-b2c-bugfix
                    //1. check cust store master entry , if not present create

                    $stmt2 = $pdo->prepare("select Is_Store_Mapped(?,?) as Flag");
                    $stmt2->execute([$Cust_No,$Store_No]);
                    $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                    $isMapped = $srow['Flag'];
                    $mcreation_date  = date('Y-m-d H:i:s');
                    $mcreated_by     = $Cust_No;
                    if ($isMapped ==0) //insert mapping
                    {
                        $stmtinsert =$pdo->prepare("INSERT INTO Cust_Store_Master
					    	         (Cust_No,
					    	          Store_No,
                                      Supplier_Retailer_No,
                                      Credit_Limit,
                                      Credit_Days,
                                      Creation_Date,
                                      Created_By,
                                      Changed_Date,
                                      Changed_By,
                                      Status
					    	         )
								     VALUES (?,
								             ?,
								             (select Get_Cust_Legacyno(?,?,?) ),
								             '200000.00',
								             '0',
								             ?,
								             ?,
								             ?,
								             ?,
								             'A'
								             )");
                        $stmtinsert->execute([$Cust_No,$Store_No,$Cust_No,$Store_No,$businessno,$mcreation_date,$mcreated_by,$mcreation_date,$mcreated_by]);

                    }


                    //2. get beatroute id from Area , if not already present then create

                    //first get area associated with delivery address
                    $stmt2 = $pdo->prepare("select Get_Geo_Area(?,?) as areacode");
                    $stmt2->execute([$Address_ID,'Area']);
                    $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                    $areaCD = $srow['areacode'];

                    do  //loop thru all eligile beatroutes
                    {
                        //get possible beatroute for Area
                        $stmt2 = $pdo->prepare("select Get_Derived_Beatroute(?,?,?) as beatroute");
                        $stmt2->execute([$Cust_No,$Store_No,$areaCD]);
                        $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                        $brouteID = $srow['beatroute'];
                        if ($brouteID >0) {
                            //check if that beatroute is mapped to customer
                            $stmt2 = $pdo->prepare("select Is_Beatroute_Mapped(?,?,?) as flag");
                            $stmt2->execute([$Cust_No,$Store_No,$brouteID]);
                            $srow = $stmt2->fetch(PDO::FETCH_ASSOC);
                            $brflag = $srow['flag'];
                            if ($brflag==0) {
                                //map
                                $stmtinsert =$pdo->prepare("INSERT INTO BBC_Org_Group_Users (Org_Group_ID,
										        Store_No,
										        User_No,
										        Login_Type,
										        Status,
										        Creation_Date,
										        Created_By,
										        Changed_Date,
										        Changed_By)
										        VALUES (?,
										                ?,
										                ?,
										             'Buyer',
										             'A',
										             ?,
										             ?,
										             ?,
										             ?)");
                                $stmtinsert->execute([$brouteID,$Store_No,$Cust_No,$mcreation_date,$mcreated_by,$mcreation_date,$mcreated_by]);
                            }
                            else if ($brflag==2) {

                                //mapped but delete or inactive so update status
                                #### Update Order_Hdr_Year current ID
                                $stmtupdate =$pdo->prepare("update BBC_Org_Group_Users set Status='A',Changed_Date=?,Changed_By=? where Org_Group_ID=? and Login_Type='Buyer' and Store_No=? and User_No=?");
                                $stmtupdate->execute([$mcreation_date,$mcreated_by,$brouteID, $Store_No ,$Cust_No]);
                            }
                            else if ($brflag==1){
                                //already mapped do nothing

                            }
                        } //brrouteid >0
                    }
                    while ($brouteID >0);  //end when no more beatroutes

                    ###### B2C Market Place Changes End //sph-b2c-bugfix

                    ###### POS ENTRIES START ##############
                    if(isset($cart_data['src'])){
                        if(strtolower($cart_data['src']) == 'pos'){

                            $data = array(
                                'Store_No' => $Store_No,
                                'Store_Session_ID' => $cart_data['Store_Session_ID'],
                                'Mobile_No' => $cart_data['Mobile_No'],
                                'Order_No' => $Order_No,
                                'Session_Amount' => $Net_Amount,
                                'Created_By' => $Cust_No
                            );

                            $this->posEntries1($pdo,$data);
                        }
                    }
                    ###### POS ENTRIES END ##############


                    #### Prepare Order_History data
                    $this->insertOrderHistory1($pdo, $Cust_No, $Order_No, $fsoNo, $custType);

                    #### Update Order_Hdr_Year current ID
                    $stmtOHY =$pdo->prepare("update Order_Hdr_Year set Current_ID=? where Year=?");
                    $stmtOHY->execute([$Current_ID, $current_year]);


                    $return = array();

                    /* update default store */
                    //insertUpdateDefaultStore1($pdo, $store_no, $Cust_No, $cart_data['default_store']);
                    /* update default store */

                    /* get details of the default store */
                    if ($cart_data['default_store'] != "") {
                        if($split == 'Y'){
                            $cart_data['default_store'] = $store_no;
                        }
                        $return['default_store'] = $this->getDefaultStoreDetails1($pdo, $cart_data['default_store']);
                    }
                    /* get details of default store */

                    $return['Order_No'] = $Order_No;
                    $return['Order_Amount'] = $Order_Amount;
                    $return['Net_Amount'] = $Net_Amount;

                    ### Insert withdraw entry in Store_Transaction table
                    $data = array(
                        'Store_No' => $Store_No,
                        'Cust_No' => $Cust_No,
                        'FSO_No' => $fsoNo,
                        'Order_No' => $Order_No,
                        'Withdraw_Amt' => $store_total_amount,
                        'Creation_Date' => $Order_Date,
                        'Created_By' => $Cust_No,
                    );

                    if(isset($cart_data['src']) && strtolower($cart_data['src']) == 'pos'){
                        ### nothing
                    } else 	{
                        $storeTrnId = $this->withdrawAmount1($pdo,$data);
                    }

                    ### Update Cart Start ####
                    $this->insertOrUpdateCart1($pdo, $Cust_No, $removed_array);
                    ### Update Cart END ####


                    ### Get Cart_data array with the Article details ####
                    $cart_array['stores'] = $this->getCartItemDetails1($pdo, $Cust_No);

                    #### Add cart data to response array i.e. return
                    if (!empty($cart_array['stores']) ) {
                        $return['cart_data'] = $cart_array['stores'];
                    } else {
                        $return['cart_data'] = "";
                    }


                    /*****SMS AND TWEET functionality ****************************************/
                    //if($payment_type != 'online_billdesk')
                    {
                        #### DEFINE SMS related data in SESSION ARRAY, TO send SMS and EMAIL after successful transaction completion

                        $_SESSION['msgPostData'] = [$Cust_No, $store_no, $Order_No, $Pref_Del_Date, $store_total_amount, $store_total_items, $custType, $Pref_Del_Time_From, $Pref_Del_Time_To];

                        /*$msgPosted = postSmsTweet1($pdo, $Cust_No, $store_no, $Order_No, $Pref_Del_Date, $store_total_amount, $store_total_items, $custType, $Pref_Del_Time_From, $Pref_Del_Time_To);
                        $return['msgPosted'] = $msgPosted;*/

                    }
                    /*****SMS AND TWEET functionality****************************************/


                    /*****Update Customer_Promo_Applied & Promo Hdr  **************/
                    $this->updatePromoHdr1($pdo,$promo_code, $Store_No, $Cust_No);

                    /* Update Customer_Promo_Applied & Promo Hdr */

                    array_push($output, $return);
                    //$response[] = $return;

                    //updateTxnIdWithOrderId($txnId, $Order_No, 'android');

                }

        }
        Log::notice('Response for place_order => placeOrder1 ', ['response' => $output]);
        return $output;

    }
#####Added by Meenal Barve for Billing Addresss
    private function get_Derived_Address($pdo,$uno,$add_id,$add_type){
        try{
            $ret_add_id = '';
            $city='';
            $state='';
            $pin='';
            $locality='';
            $stmt = $pdo->prepare("SELECT City_Code,State_Code,Pincode,Locality
		FROM   BBC_Addresses
		WHERE  Address_ID = ?");
            $stmt->execute([$add_id]);
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            if(!empty($row)){
                $city  = $row['City_Code'];
                $state = $row['State_Code'];
                $pin   = $row['Pincode'];
                $locality = $row['Locality'];
            }

            if ($add_type =='Delivery') {

                $stmt = $pdo->prepare("SELECT Address_ID
			FROM BBC_Addresses
			WHERE User_No = ?
			AND   `Login_Type`='Buyer'
			AND   ((City_Code = '$city' AND State_Code = '$state' AND Pincode   = '$pin') OR
				(((Pincode = '$pin' AND City_Code = '$city') OR (Pincode ='$pin' AND State_Code= '$state')) OR
				((City_Code = '$city' AND State_Code = '$state') OR (City_Code = '$city' AND Pincode = '$pin')) OR
				((State_Code= '$state' AND City_Code = '$city') OR (State_Code = '$state' AND Pincode = '$pin'))) OR
				(State_Code='$state') OR (City_Code='$city') OR (Pincode = '$pin'))
			AND   Address_Type='Delivery'
			AND   Default_Flag='A'");
                $stmt->execute([$uno]);
                $row= $stmt->fetch(PDO::FETCH_ASSOC);
                if(!empty($row)) {
                    $ret_add_id = $row['Address_ID'];
                }

                if ($ret_add_id=='') { //if nothing matches return default Delivery
                    $stmt = $pdo->prepare("SELECT Address_ID
				FROM BBC_Addresses
				WHERE User_No = ?
				AND   `Login_Type`='Buyer'
				AND   Address_Type='Delivery'
				AND   `Status`='A'");
                    $stmt->execute([$uno]);
                    $row= $stmt->fetch(PDO::FETCH_ASSOC);
                    if(!empty($row)) {
                        $ret_add_id = $row['Address_ID'];
                    }
                }
            }

            elseif ($add_type == 'Shipping') {
                //Check if any addresses matches in exception table
                $stmt = $pdo->prepare("SELECT Address_ID
			FROM   BBC_Org_Serving_Areas
			WHERE  Store_No = ?
			AND   ((City_Code = '$city' AND State_Code = '$state' AND Pincode   = '$pin') OR
				(((Pincode = '$pin' AND City_Code = '$city') OR (Pincode ='$pin' AND State_Code= '$state')) OR
				((City_Code = '$city' AND State_Code = '$state') OR (City_Code = '$city' AND Pincode = '$pin')) OR
				((State_Code= '$state' AND City_Code = '$city') OR (State_Code = '$state' AND Pincode = '$pin'))) OR
				(State_Code='$state') OR (City_Code='$city') OR (Pincode = '$pin'))
			AND   `Status`='A'");
                $stmt->execute([$uno]);
                $row= $stmt->fetch(PDO::FETCH_ASSOC);
                if(!empty($row)) {
                    $ret_add_id = $row['Address_ID'];
                }
                if ($ret_add_id=='') { //Added for MFM , if city,statte,pincode does not match in exception table then check if Area matches
                    $stmt = $pdo->prepare("SELECT Address_ID
				FROM BBC_Org_Serving_Areas
				WHERE  Store_No = ?
				AND    Area_Code= '$locality'
				AND   `Status`='A'");
                    $stmt->execute([$uno]);
                    $row= $stmt->fetch(PDO::FETCH_ASSOC);
                    if(!empty($row)) {
                        $ret_add_id = $row['Address_ID'];
                    }
                    if ($ret_add_id=='') { //if nothing matches in exception table, match city, state, pin in address table ,do not check default flag
                        $stmt = $pdo->prepare("SELECT Address_ID
				FROM BBC_Addresses
				WHERE User_No =?
				AND   `Login_Type` ='Seller'
				AND   ((City_Code = '$city' AND State_Code = '$state' AND Pincode   = '$pin') OR
				(((Pincode = '$pin' AND City_Code = '$city') OR (Pincode ='$pin' AND State_Code= '$state')) OR
				((City_Code = '$city' AND State_Code = '$state') OR (City_Code = '$city' AND Pincode = '$pin')) OR
				((State_Code= '$state' AND City_Code = '$city') OR (State_Code = '$state' AND Pincode = '$pin'))) OR
				(State_Code='$state') OR (City_Code='$city') OR (Pincode = '$pin'))
				AND   Address_Type='Shipping'
				AND   `Status`='A'");
                        $stmt->execute([$uno]);
                        $row= $stmt->fetch(PDO::FETCH_ASSOC);
                        if(!empty($row)) {
                            $ret_add_id = $row['Address_ID'];
                        }

                        if ($ret_add_id=='') { //if nothing matches return default shipping
                            $stmt = $pdo->prepare("SELECT Address_ID
					FROM BBC_Addresses
					WHERE User_No =?
					AND   `Login_Type` ='Seller'
					AND   Address_Type='Shipping'
					AND   Default_Flag='A'
					AND   `Status`='A'");
                            $stmt->execute([$uno]);
                            $row= $stmt->fetch(PDO::FETCH_ASSOC);
                            if(!empty($row)) {
                                $ret_add_id = $row['Address_ID'];
                            }
                            else {  //if nothing matches return registered address
                                $stmt = $pdo->prepare("SELECT Address_ID
							FROM BBC_Addresses
							WHERE User_No = ?
							AND   `Login_Type`='Seller'
							AND   Address_Type='Registered'
							AND   `Status`='A'");
                                $stmt->execute([$uno]);
                                $row= $stmt->fetch(PDO::FETCH_ASSOC);
                                if(!empty($row)) {
                                    $ret_add_id = $row['Address_ID'];
                                }
                            }
                        }
                    }
                }

            }
        } catch (PDOException $e) {

            //$this->pdo->rollback();
            //$data = $e->getMessage();
            $ret_add_id ='';
        }

        return $ret_add_id;

    }

    private function get_Billing_Address($pdo,$uno,$add_id){
        $ret_add_id='';
        try{
            $stmt = $pdo->prepare("SELECT Ref_Address_ID
	FROM BBC_Addresses
	WHERE User_No = ?
	AND   `Login_Type`= 'Seller'
	AND   Address_ID = ?
	AND   Address_Type='Shipping'");
            $stmt->execute([$uno, $add_id]);
            $row  = $stmt->fetch(PDO::FETCH_ASSOC);
            if(!empty($row)) {
                $ret_add_id = $row['Ref_Address_ID'];
            }
            else { //if add_id is registered address then there will no Billing address hence return the same address
                $ret_add_id = $add_id;
            }
        }
        catch (PDOException $e) {
            $ret_add_id='';
        }
        return $ret_add_id;
    }
##### Added by Meenal for Billing Address

    private function getSlabsPrice1($pdo, $articleNo, $quantity, $storeNo){
        $stmt = $pdo->prepare("select SK_Offer_Price from Article_Offers where Supplier_No=? and Article_No=? and ? >= Min_Qty AND ? <= Max_Qty");
        $stmt->execute([$storeNo, $articleNo, $quantity, $quantity]);
        //$stmt->debugDumpParams();
        $price = $stmt->fetch(PDO::FETCH_ASSOC);
        $offerPrice = 0;
        if(!empty($price)){
            $offerPrice = $price['SK_Offer_Price'];
        }
        return $offerPrice;
    }

    private function insertUpdateDefaultStore1($pdo, $storeNo, $custNo, $defaultStore){

        $creationDate = date('Y-m-d');
        $creationTime = date('H:i:s');

        $stmt4 = $pdo->prepare("select * from User_Store_Listing where Store_No=? and Cust_No=? ");
        $stmt4->execute([$storeNo, $custNo]);
        $result = $stmt4->fetch(PDO::FETCH_ASSOC);

        if (!empty($result)) {

            ### Update if default store
            if ($defaultStore != "") {
                ### Update existing listed store Default_Flag to I
                $stmt = $pdo->prepare("update User_Store_Listing set Default_Flag=?, Change_Date = ?,Change_Time = ?,Changed_By = ? where Cust_No=? ");
                $stmt->execute(['I', $creationDate, $creationTime, $custNo, $custNo]);

                ### Update new listed store Default_Flag to A
                $stmt = $pdo->prepare("update User_Store_Listing set Default_Flag=?, Change_Date = ?,Change_Time = ?,Changed_By = ? where Cust_No=? and Store_No=? ");
                $stmt->execute(['A', $creationDate, $creationTime, $custNo, $custNo, $defaultStore]);
            }
        } else {
            $stmt = $pdo->prepare("insert into User_Store_Listing (Cust_No, Store_No, Status, Default_Flag, Creation_Date, Creation_Time, Created_By )
				values(?, ? , ? ,?, ? , ? , ?)");
            $stmt->execute([$custNo, $storeNo, 'A', 'I', $creationDate, $creationTime, $custNo]);
        }

    }


    private function getDefaultStoreDetails1($pdo, $defaultStore) {
        Log::notice('Request Data for place_order => getDefaultStoreDetails1', ['defaultStore' => $defaultStore]);
        $json_response = array();
        $stmt = $pdo->prepare("SELECT
			SM.`Store_Name`,
			SM.`Store_No`,
			SM.`City_Code`,
			CM.`City_Name`,
			SM.`Area_Code`,
			SM.`Delivery_Limit`,
			SM.`Min_Order_Amt`,
			SM.`Min_Order_Amt_Without_Chrg`,
			SM.`Delivery_Charge`,
			SM.`Ratings`
			FROM `Store_Master` as SM Join City_Master As CM on SM.City_Code=CM.City_Code
			WHERE SM.`Status` = ? AND SM.`Store_No` = ?
			ORDER BY SM.`Store_Name`");
        $stmt->execute(['A', $defaultStore]);
        $row = $stmt->fetchAll(PDO::FETCH_ASSOC);

        for ($is=0; $is <count($row) ; $is++) {
            $store = array();
            $store['no'] 			= $row[$is]['Store_No'];
            $store['name'] 			= $row[$is]['Store_Name'];
            $store['area_code'] 	= $row[$is]['Area_Code'];
            $store['city_code'] 	= $row[$is]['City_Code'];
            $store['city_name'] 	= $row[$is]['City_Name'];
            $store['limit'] 		= $row[$is]['Delivery_Limit'];
            $store['min'] 			= $row[$is]['Min_Order_Amt'];
            $store['min_free'] 		= $row[$is]['Min_Order_Amt_Without_Chrg'];
            $store['charge'] 		= $row[$is]['Delivery_Charge'];
            $store['rating'] 		= $row[$is]['Ratings'];
            $store['areas'] 		= array();

            $store_no = $row[$is]['Store_No'];

            $stmt5 = $pdo->prepare("SELECT SA.`Area_Code`,
					AM.`Area_Name` FROM `Store_Area` AS `SA`
					INNER JOIN `Area_Master` AS `AM`
					ON SA.`Area_Code` = AM.`Area_Code`
					WHERE SA.`Store_No` = ? ");
            $stmt5->execute([$defaultStore]);
            $row2 = $stmt5->fetchAll(PDO::FETCH_ASSOC);


            for ($si=0; $si <count($row2) ; $si++) {
                $served_area = array();
                $served_area['code'] = $row2[$si]['Area_Code'];
                $served_area['name'] = $row2[$si]['Area_Name'];
                array_push($store['areas'], $served_area);
            }
            for ($i = 0; $i < sizeof($store['areas']); $i++) {
                if ($store['areas'][$i]['code'] == $row[$is]['Area_Code']) {
                    $store['area_name'] = $store['areas'][$i]['name'];
                    break;
                }
            }
            array_push($json_response, $store);
        }
        Log::notice('Response Data for place_order => getDefaultStoreDetails1', ['response' => $json_response]);
        if(!empty($json_response)) return $json_response[0];
        else $json_response;
    }

    private function insertOrderHistory1($pdo, $custNo, $orderNo, $fsoNo){
        Log::notice('Request Data for place_order => insertOrderHistory1', ['custNo' => $custNo, 'orderNo' => $orderNo, 'fsoNo' =>$fsoNo]);
        $status = 'A';
        $creationDate = date('Y-m-d');
        $creationTime = date('H:i:s');
        $Login_Type="Self";
        $User_No=$custNo;
        ### If Order Placed by FSO
        if($fsoNo > 0){
            $Login_Type="FSO";
            $User_No   =$fsoNo;
        }
        $stmt = $pdo->prepare("INSERT INTO `Order_History`(`Order_ID`, `Memo`, `History_Date`, `History_Time`, `Status`, `Creation_Date`, `Creation_Time`, `Created_By`, `Change_Date`, `Change_Time`, `Changed_By` ,`Login_Type`)
			VALUES ( ? , ?, ? , ? , ? , ? , ? , ? , ? , ? , ? ,?) ");
        $stmt->execute([$orderNo, 'Order Placed', $creationDate, $creationTime, $status, $creationDate, $creationTime, $User_No, $creationDate, $creationTime, $User_No, $Login_Type]);
        if ($fsoNo >0) {
            $stmt1 = $pdo->prepare("INSERT INTO `Order_History`(`Order_ID`, `Memo`, `History_Date`, `History_Time`, `Status`, `Creation_Date`, `Creation_Time`, `Created_By`, `Change_Date`, `Change_Time`, `Changed_By`,`Login_Type`)
			VALUES ( ? , ?, ? , ? , ? , ? , ? , ? , ? , ? , ? ,?) ");
            $stmt1->execute([$orderNo, 'Order Accepted', $creationDate, $creationTime, $status, $creationDate, $creationTime, $User_No, $creationDate, $creationTime, $User_No, $Login_Type]);
        }

    }

    private function getCartItemDetails1($pdo, $Cust_No){
        Log::notice('Request Data for place_order => getCartItemDetails1', ['Cust_No' => $Cust_No]);
        $stmt7 = $pdo->prepare("select * from User_Cart where Cust_No=?");
        $stmt7->execute([$Cust_No]);
        $userCart = $stmt7->fetch(PDO::FETCH_ASSOC);

        $cartData = [];
        if(!empty($userCart)){
            $cartData = json_decode($userCart['Cart_Data'],true);
        }

        $i = 0;
        $store = array();
        $cart_array = array();
        $cart_array['stores'] = array();

        ### Get customers all store no from cart_data
        if(!empty($cartData))
            foreach ($cartData as $value) {
                foreach ($value as $newa) {
                    $store[$i] = $newa['Store_No'];
                    $i = $i + 1;
                }
            }

        ### Get unique store no from cart
        $store = array_unique($store);

        ### Prepare cart data : with article details
        $mystore = array();

        foreach ($store as $StoreNo) {
            $mystore[$StoreNo]['store_no'] = $StoreNo;

            foreach ($cartData as $cart) {

                foreach ($cart as $new) {

                    if ($new['Store_No'] == $StoreNo) {

                        $articleNo = $new['Article_No'];
                        $quantity = $new['quantity'];

                        /*$stmt8 = $pdo->prepare("SELECT
                                AM.`Article_No` as no,
                                AM.`Valume` as volume,
                                AM.`Unit` as unit,
                                AM.`Merch_Cat` as cat,
                                AM.`Image_Url` as image,
                                AM.`Article_Desc` as descp,
                                AM.`Brand_ID` as brand,
                                SM.`Store_Name` as store_name,
                                SL.`Unit_Price` as unit_price,
                                SL.`Offer_Price`  as offer_price,
                                SL.`Out_Of_Stock` as outofstock
                                FROM `Article_Master` As `AM`
                                INNER JOIN `Store_Listing` AS `SL` ON AM.`Article_No` = SL.`Article_No`
                                INNER JOIN `Store_Master` AS `SM` ON SL.`Store_No` = SM.`Store_No`
                                WHERE AM.`Article_No` = ? AND SL.`Store_No` = ? AND AM.`Status` = 'A'");*/
                        //removed offer price and unit price mmb-android-pricing
                        $stmt8 = $pdo->prepare("SELECT
							AM.`Article_No` as no,
							AM.`Valume` as volume,
							AM.`Unit` as unit,
							AM.`Merch_Cat` as cat,
							AM.`Image_Url` as image,
							AM.`Article_Desc` as descp,
							AM.`Brand_ID` as brand,
							SM.`Store_Name` as store_name,
							SL.`Out_Of_Stock` as outofstock
							FROM `Article_Master` As `AM`
							INNER JOIN `Store_Listing` AS `SL` ON AM.`Article_No` = SL.`Article_No`
							INNER JOIN `Store_Master` AS `SM` ON SL.`Store_No` = SM.`Store_No`
							WHERE AM.`Article_No` = ? AND SL.`Store_No` = ? AND AM.`Status` = 'A'");
                        //mmb-android-pricing
                        $stmt8->execute([$articleNo, $StoreNo]);
                        $row = $stmt8->fetch(PDO::FETCH_ASSOC);
                        //$stmt8->debugDumpParams();

                        if(!empty($row)) {

                            $mystore[$StoreNo]['store_name'] = $row['store_name'];
                            $row['out_of_stock'] = intval($row['outofstock']);
                            $row['quantity'] = $quantity;
                            unset($row['outofstock']);
                            unset($row['store_name']);
                            $mystore[$StoreNo]['cart_items'][$articleNo] = $row;

                        }

                    }
                }
            }


        }
        Log::notice('Response Data for place_order => getCartItemDetails1', ['response' => $mystore]);
        return $mystore;

    }

    private function insertOrUpdateCart1($pdo, $Cust_No, $removed_array){
        Log::notice('Request Data for place_order => insertOrUpdateCart1', ['Cust_No' => $Cust_No, 'removed_array' => $removed_array]);
        $stmt6 = $pdo->prepare("select * from User_Cart where Cust_No=?");
        $stmt6->execute([$Cust_No]);
        $row = $stmt6->fetch(PDO::FETCH_ASSOC);
        $date_time = date('Y-m-d H:i:s');

        $cartData = [];
        //if(!empty($row) && isset($row['Cart_Data']))
        //{
        $cartData = json_decode($row['Cart_Data'], true);
        //}
        //else {
        $newCartData = array();
        $tt = 0;
        for ($ll = 0; $ll < sizeof($cartData); $ll++) {
            $flag = 0;
            for ($pp = 0; $pp < sizeof($removed_array); $pp++) {
                if ($cartData[$ll][0]['Article_No'] == $removed_array[$pp]['Article_No'] and $cartData[$ll][0]['Store_No'] == $removed_array[$pp]['Store_No']) {
                    $flag = 1;
                    break;
                }
            }
            if ($flag != 1) {
                $newCartData[$tt] = $cartData[$ll];
                $tt = $tt + 1;
            }
        }
        $newCart = json_encode($newCartData);

        ### Delete existing cart
        /*$stmtD = $pdo->prepare("delete from User_Cart where Cust_No=?");
        $stmtD->execute([$Cust_No]);

        ### And INsert new cart
        $stmtI = $pdo->prepare("insert into User_Cart(Cust_No, Cart_Data, Creation_Date, Created_By, Changed_By, Changed_Date) values(?,?,?,?,?,?)");
        $stmtI->execute([$Cust_No, $newCart, $date_time, $Cust_No, $Cust_No, $date_time]);*/
        Log::notice('Response Data for place_order => insertOrUpdateCart1', ['response' => $newCart]);

        $stmtU = $pdo->prepare("update User_Cart set Cart_Data =?,Changed_Date=?,Changed_By=?
		                    where Cust_No =?");
        $stmtU->execute([$newCart, $date_time, $Cust_No, $Cust_No]);
//}
    }

    private function postSmsTweet1($pdo, $Cust_No, $store_no, $Order_No, $Pref_Del_Date,$store_total_amount, $store_total_items, $custType, $Pref_Del_Time_From = '', $Pref_Del_Time_To = ''){
        Log::notice('Request Data for place_order => postSmsTweet1', ['request' => $Cust_No, $store_no, $Order_No, $Pref_Del_Date,$store_total_amount, $store_total_items, $custType, $Pref_Del_Time_From = '', $Pref_Del_Time_To = '']);

        $stmt9 = $pdo->prepare("select Mobile_No from Cust_Master where Cust_Master.Cust_No=? ");
        $stmt9->execute([$Cust_No]);
        $sms_result_customer = $stmt9->fetch(PDO::FETCH_ASSOC);
        $phone_no = $sms_result_customer['Mobile_No'];

        $stmt10 = $pdo->prepare("select Store_Master.Store_Name,Store_Master.Mobile_No_1,Store_Master.Mobile_No_2,City_Master.City_Name from Store_Master inner join City_Master on City_Master.City_Code=Store_Master.City_Code where Store_Master.Store_No=?");
        $stmt10->execute([$store_no]);
        $sms_array = $stmt10->fetch(PDO::FETCH_ASSOC);

        $store_name_sms = $sms_array['Store_Name'];
        $store_city = $sms_array['City_Name'];
        $store_mobile_no_1 = $sms_array["Mobile_No_1"];
        $store_mobile_no_2 = $sms_array["Mobile_No_2"];

        ### Send SMS to USER
        /*$message_sms = generate_sms(ORDER_PLACED, array($store_name_sms,$Order_No));
        Submit(SMS_HOST,SMS_PORT,SMS_USERNAME,SMS_PASSWORD,SMS_SENDER,$message_sms,$phone_no,SMS_MSG_TYPE,SMS_DLR);*/

        $smsLevel = '';
        if($custType == 'FSO' && MY_WEBSITE == 'Chitale') {
            $smsLevel = '3';
        } else if($custType == 'Self' && MY_WEBSITE == 'Chitale') {
            $smsLevel = '2';
        }

        $res1 = SubmitSMS($phone_no, 'ORDER_PLACED', array($store_name_sms, $Order_No), $smsLevel);

        #### Order Placed SMS For Store Owner ####
        $store_owner_mobile = [];
        if ($store_mobile_no_1 != ''){
            $this->createLogPO(['store_mobile_no_1', $store_mobile_no_1]);
            $store_owner_mobile[] = $store_mobile_no_1;
        }
        if ($store_mobile_no_2 != ''){
            $this->createLogPO(['store_mobile_no_2', $store_mobile_no_2]);
            $store_owner_mobile[] = $store_mobile_no_2;
        }
        $store_owner_mobile = implode(",", $store_owner_mobile);

        $this->createLogPO([$store_owner_mobile]);

        $New_Pref_Del_Date = date("d-M-Y", strtotime($Pref_Del_Date));

        ### Send SMS to store owner
        $smsLevel = '';
        if($custType == 'FSO' && MY_WEBSITE == 'Chitale') {
            $smsLevel = '3';
        } else if($custType == 'Self' && MY_WEBSITE == 'Chitale') {
            $smsLevel = '2';
        }
        $res2 = SubmitSMS($store_owner_mobile, 'NEW_ORDER_SUPPLIER', array($store_name_sms,$Order_No,$store_total_amount,$store_total_items,$New_Pref_Del_Date,$Pref_Del_Time_From,$Pref_Del_Time_To) , $smsLevel);

        $res3 = '';
        if(MY_WEBSITE == 'AutoD'){
            $res3 = $this->sendEmailOrderPlaced($pdo, $store_no, $Order_No);
        }

        /* snapkirana twitter post code */
        if(SEND_TWEET == 'Y')
        {
            $connection = $this->getConnectionWithAccessToken1(TWEET_API_KEY, TWEET_API_SECRET, TWEET_ACCESS_TOKEN, TWEET_ACCESS_TOKEN_SECRET);

            //$say = "Order placed on snapkirana.com to store ".$store_name." is ".$sms_code." at ".$Creation_Time;
            $news_name = explode(' ', $store_name_sms);
            $newstore_name = join(' ', array_slice($news_name, 0, 2));
            $twtDate = date('d-m-y');
            $twttime = date('H:i');
            $say = $Order_No . " to " . $newstore_name . ", " . $store_city . " on " . $twtDate . " at " . $twttime;
            $parameters = array('status' => $say);
            $status_twitter = $connection->post('statuses/update', $parameters);
        }
        /* snapkirana twitter post code */
        Log::notice('Response Data for place_order => postSmsTweet1', ['response' => $res3]);

        return $res3;

    }

    private function sendEmailOrderPlaced($pdo, $storeNo, $orderNo){
        Log::notice('Request Data for place_order => sendEmailOrderPlaced', ['storeNo' => $storeNo, 'orderNo' => $orderNo]);
        $stmt9 = $pdo->prepare("select OA.Name, CM.City_Name, SM.State_Name
			FROM Order_Addresses OA
		    LEFT JOIN `City_Master` AS `CM` ON OA.`City_Code` = CM.`City_Code`
            LEFT JOIN `State_Master` AS `SM` ON OA.`State_Code` = SM.`State_Code`
            WHERE OA.Order_No=? AND Address_Type = ?
            ");

        $stmt9->execute([$orderNo, 'Delivery']);
        $buyerInfo = $stmt9->fetch(PDO::FETCH_ASSOC);
        //$stmt9->debugDumpParams();


        $stmt = $pdo->prepare("SELECT * FROM `Subscriber_Group` as SG WHERE SG.`Mode` = ? AND SG.`Store_No` = ? AND SG.`Topic` = ? AND SG.`Status` = ? ");
        $stmt->execute(['EMAIL', $storeNo, 'ORDER_PLACED', 'A']);
        $row = $stmt->fetchAll(PDO::FETCH_ASSOC);
        //$stmt->debugDumpParams();

        $res = '';
        $senderInfo = [];
        $recepientInfo = [];
        if(!empty($row)){
            foreach ($row as $key => $value) {
                if($value['User_Type'] == 'SENDER') {
                    $senderInfo['usernameSmtp'] = $value['User_ID'];
                    $senderInfo['passwordSmtp'] = $value['Password'];
                    $senderInfo['username'] = $value['User_Name'];
                } else if($value['User_Type'] == 'RECIPIENT') {
                    $recepientInfo[] = $value['User_ID'];
                }
            }
        }

        // Replace recipient@example.com with a "To" address. If your account
        // is still in the sandbox, this address must be verified.
        $recepient = 'daya4608@gmail.com';

        // The plain-text body of the email
        $bodyText =  "Order Received";

        $subject =  "Order Received- $orderNo";

        // The HTML-formatted body of the email
        $bodyHtml = '
		<pre>Dear Sir/Madam,

		We have received an order as per following details

		Order Number : '.$orderNo.'
		Buyer Name: '.$buyerInfo['Name'].'
		City: '.$buyerInfo['City_Name'].'
		State:  '.$buyerInfo['State_Name'].'
		Order Date & Time: '.date('Y-m-d H:i:s').'

		Please process this further.

		Thanks and Regards,

		'.MY_WEBSITE.' Ordering System </pre>';

        $mail = new PHPMailer(true);

        if(!empty($senderInfo) && !empty($recepientInfo)){
            try {
                // Specify the SMTP settings.
                $mail->isSMTP();
                $mail->setFrom($senderInfo['usernameSmtp'], $senderInfo['username']);
                $mail->Username   = $senderInfo['usernameSmtp'];
                $mail->Password   = $senderInfo['passwordSmtp'];
                $mail->Host       = 'smtp-mail.outlook.com';
                $mail->Port       = 587;
                $mail->SMTPAuth   = true;
                $mail->SMTPSecure = 'tls';
                //$mail->addCustomHeader('X-SES-CONFIGURATION-SET', $configurationSet);

                // Specify the message recipients.
                if(!empty($recepientInfo)){
                    foreach ($recepientInfo as $recepient) {
                        $mail->addAddress($recepient);
                    }
                }

                // You can also add CC, BCC, and additional To recipients here.
                //$mail->addCC("ccaddress@ccdomain.com", "Some CC Name");
                //$mail->addBCC("bccaddress@ccdomain.com", "Some BCC Name");


                // Specify the content of the message.
                $mail->isHTML(true);
                $mail->Subject    = $subject;
                $mail->Body       = $bodyHtml;
                $mail->AltBody    = $bodyText;
                $mail->SMTPDebug = 2;
                $mail->Send();
                $res = "Email sent!";
            } catch (phpmailerException $e) {
                $res = "An error occurred. {$e->errorMessage()}"; //Catch errors from PHPMailer.
            } catch (Exception $e) {
                $res = "Email not sent. {$mail->ErrorInfo}"; //Catch errors from Amazon SES.
            }
            Log::notice('Response Data for place_order => sendEmailOrderPlaced', ['response' => $res]);
            return $res;
        }

    }


    private function updatePromoHdr1($pdo, $promo_code, $Store_No, $Cust_No){
        Log::notice('Request Data for place_order => updatePromoHdr1', ['promo_code' => $promo_code, 'Store_No' => $Store_No, 'Cust_No' => $Cust_No]);

        $stmt1 = $pdo->prepare("select * from Promo_Hdr where Promo_Code=? AND Store_No=?");
        $stmt1->execute([$promo_code, $Store_No]);
        $resultPromo = $stmt1->fetchAll(PDO::FETCH_ASSOC);

        if (!empty($resultPromo) && count($resultPromo) > 0) {

            ### Update total redeemption of promo code
            $stmtU = $pdo->prepare("update Promo_Hdr set Total_Redeemption=Total_Redeemption+1 where Promo_Code=? and Store_No=?");
            $stmtU->execute([$promo_code, $Store_No]);

            $stmt2 = $pdo->prepare("select * from Customer_Promo_Applied where Promo_Code=? AND Cust_No=? AND Store_No=? ");
            $stmt2->execute([$promo_code, $Cust_No, $Store_No]);
            $resultAppliedPromo = $stmt2->fetch(PDO::FETCH_ASSOC);

            ### Insert/Update in Customer_Promo_Applied
            if (!empty($resultAppliedPromo) && count($resultAppliedPromo) > 0) {
                $UpdatedTimes = $resultAppliedPromo['Times'] + 1;
                $stmtTU = $pdo->prepare("UPDATE Customer_Promo_Applied SET `Times`=? WHERE Promo_Code=? AND Cust_No=? AND Store_No=?");
                $stmtTU->execute([$UpdatedTimes, $promo_code, $Cust_No, $Store_No]);
            } else {
                $stmtTI = $pdo->prepare("INSERT INTO `Customer_Promo_Applied`(`Cust_No`, `Promo_Code`, `Store_No`, `Times`) VALUES (?,?,?,?)");
                $stmtTI->execute([$Cust_No, $promo_code, $Store_No, '1']);
            }

        }/* else {

		#### I dont know why this condition is available :(
		$stmtU = $pdo->prepare("update Promo_Hdr set Total_Redeemption=Total_Redeemption+1 where Promo_Code=? and Store_No=?");
		$stmtU->execute([$promo_code, '0']);

		$stmt2 = $pdo->prepare("select * from Promo_Hdr where Promo_Code=? AND Store_No=? ");
		$stmt2->execute([$promo_code, '0' ]);
		$resultPromo = $stmt2->fetchAll(PDO::FETCH_ASSOC);

		if (count($resultPromo) > 0) {

			$stmt3 = $pdo->prepare("select * from Customer_Promo_Applied where Promo_Code=? AND Cust_No=? AND Store_No=?");
			$stmt3->execute([$promo_code, $Cust_No, $Store_No]);
			$resultAppliedPromo = $stmt3->fetch(PDO::FETCH_ASSOC);

			if (count($resultAppliedPromo) > 0) {
				$UpdatedTimes = $resultAppliedPromo['Times'] + 1;
				$stmtTU = $pdo->prepare("UPDATE Customer_Promo_Applied SET `Times`=? WHERE Promo_Code=? AND Cust_No=? AND Store_No=?");
				$stmtTU->execute([$UpdatedTimes, $promo_code, $Cust_No, $Store_No]);

			} else {
				$stmtTI = $pdo->prepare("INSERT INTO `Customer_Promo_Applied`(`Cust_No`, `Promo_Code`, `Store_No`, `Times`) VALUES (?,?,?,?)");
				$stmtTI->execute([$Cust_No, $promo_code, $Store_No, '1']);
			}
		}
	}*/
        Log::notice('Response Data for place_order => updatePromoHdr1', ['resultPromo' => $resultPromo]);
    }

    private function calculateOrderAmount1($items){

        $total_amount = 0;
        if(!empty($items)) {
            for ($k = 0; $k < sizeof($items); $k++) {
                $Order_Amount = $items[$k]['Quantity'] * $items[$k]['Offer_Price'];
                $total_amount = $total_amount + $Order_Amount;
            }
        }
        return $total_amount;

    }
    private function withdrawAmount1($pdo, $data){

        ### GET Balance_Amount from Store_Transaction
        ## Get last inserted id balance amount of store for that customer
        #SELECT `Transaction_ID`, `Balance_Amt`, `Creation_Date`  FROM `store_transaction` WHERE Store_No = 2 AND Cust_No = 3 ORDER BY Transaction_ID DESC Limit 1
        Log::notice('Request Data for place_order => withdrawAmount1', ['request' => $data]);
        $Creation_Date = date('Y-m-d H:i:s');

        $stmt = $pdo->prepare("select Transaction_ID, Balance_Amt, Outstanding_Balance_Amt from Store_Transaction where `Store_No` = ? AND Cust_No = ? ORDER BY Transaction_ID DESC Limit 1");
        $stmt->execute([$data['Store_No'], $data['Cust_No']]);
        $row = $stmt->fetch(PDO::FETCH_ASSOC);

        if(empty($row))
        {
            $balanceAmt = 0;
            $data['Outstanding_Balance_Amt'] = 0;
        } else {
            $balanceAmt = $row['Balance_Amt'];
            $data['Outstanding_Balance_Amt'] = $row['Outstanding_Balance_Amt'];
        }

        ## Calculate balance amount  balance amt = existing balance amt - withdraw amt
        $withAmt = $data['Withdraw_Amt'];
        $balanceAmt = $balanceAmt - $withAmt;
        $balanceAmt = round($balanceAmt, 2);

        $data['Transaction_Type'] = '2';
        $data['Balance_Amt'] = $balanceAmt;
        $data['Outstanding_Amt'] = '0';
        $data['Remark'] = 'Amount credited';

        Log::notice('Response Data for place_order => withdrawAmount1', ['response' => $data]);

        $stmtI = $pdo->prepare('INSERT INTO `Store_Transaction`(`Transaction_Type`, `Store_No`, `Cust_No`, `FSO_No`, `Order_No`, `Deposit_Amt`, `Withdraw_Amt`, `Balance_Amt`, `Outstanding_Amt`, `Outstanding_Balance_Amt`, `Transaction_Date`, `Creation_Date`, `Created_By`, `Changed_Date`, `Changed_By`,`Remark`) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)');

        $stmtI->execute([ $data['Transaction_Type'], $data['Store_No'], $data['Cust_No'], $data['FSO_No'], $data['Order_No'], "0", $data['Withdraw_Amt'], $data['Balance_Amt'], $data['Outstanding_Amt'], $data['Outstanding_Balance_Amt'], $Creation_Date, $Creation_Date, $data['Created_By'], $Creation_Date, $data['Created_By'], $data['Remark'] ]);

        //return mysqli_insert_id($con);

    }

    public function getConnectionWithAccessToken1($cons_key, $cons_secret, $oauth_token, $oauth_token_secret) {
        Log::notice('Request Data for place_order => getConnectionWithAccessToken1', ['cons_key' => $cons_key, 'cons_secret' => $cons_secret, 'oauth_token' => $oauth_token, 'oauth_token_secret' => $oauth_token_secret]);
        $connection = new TwitterOAuth($cons_key, $cons_secret, $oauth_token, $oauth_token_secret);
        Log::notice('Response Data for place_order => getConnectionWithAccessToken1', ['response' => $connection]);
        return $connection;
    }

    /***********
    If src is pos then add entries in pos tables
    posEntries1
    @param: (array) $data
     ************/
    private function posEntries1($pdo, $data){

        require 'db_connect.php';

        $Creation_Date = date('Y-m-d H:i:s');
        Log::notice('Request Data for place_order => posEntries1', ['request' => $data]);
        ### Get payment type
        /*$query1 = "select Store_Payment_Types_Id from Store_Payment_Types where Store_No='".$data['Store_No']."'";
        $row1 = $pdo->query($query1)->fetch(PDO::FETCH_ASSOC);*/

        $stmt=$pdo->prepare("select Store_Payment_Types_Id from Store_Payment_Types where Store_No=?");
        $stmt->execute([$data['Store_No']]);
        $row1 = $stmt->fetch(PDO::FETCH_ASSOC);

        $storePaymentType = '';
        if(!empty($row1)){
            $storePaymentType = $row1['Store_Payment_Types_Id'];
        }

        ### Insert into POS_Store_Session_Orders
        /*$sql1 = "INSERT INTO `POS_Store_Session_Orders` (`Store_Session_ID`, `Store_No`, `Order_No`, `Mobile_No`, `Status`, `Creation_Date`, `Created_By`, `Changed_Date`, `Changed_By`) VALUES ( '".$data['Store_Session_ID']."', '".$data['Store_No']."', '".$data['Order_No']."', '".$data['Mobile_No']."', 'Y', '".$Creation_Date."', '".$data['Created_By']."', '".$Creation_Date."', '".$data['Created_By']."')";
        $pdo->query($sql1);*/

        $stmtI=$pdo->prepare("INSERT INTO `POS_Store_Session_Orders` (`Store_Session_ID`, `Store_No`, `Order_No`, `Mobile_No`, `Status`, `Creation_Date`, `Created_By`, `Changed_Date`, `Changed_By`) VALUES ( ?,?,?,?,?,?,?,?,?) ");
        $stmtI->execute([
            $data['Store_Session_ID'], $data['Store_No'], $data['Order_No'], $data['Mobile_No'], 'Y', $Creation_Date, $data['Created_By'], $Creation_Date, $data['Created_By']
        ]);


        ### Get data from POS_Store_Session_Summary
        /*$query2 = "select Session_Amount from POS_Store_Session_Summary where Store_Session_ID='".$data['Store_Session_ID']."'";
        $row2 = $pdo->query($query2)->fetchAll(PDO::FETCH_ASSOC);*/

        $stmtS = $this->pdo->prepare("select Session_Amount from POS_Store_Session_Summary where Store_Session_ID=?");
        $stmtS->execute([$data['Store_Session_ID']]);
        $row2 = $stmt->fetchAll(PDO::FETCH_ASSOC);

        //$result2 = mysqli_query($con, $query2);

        if(!empty($row2)) {
            $sessionAmt = $row2['0']['Session_Amount'];
            $amt = $sessionAmt + $data['Session_Amount'];
            ### Update into POS_Store_Session_Summary new amount
            /*$sql2 = "UPDATE `POS_Store_Session_Summary` SET `Session_Amount` = '".$amt."',`Changed_Date` = '".$Creation_Date."', `Changed_By` = '".$data['Created_By']."'  WHERE `POS_Store_Session_Summary`.`Store_Session_ID` = ".$data['Store_Session_ID'];
            $pdo->query($sql2);*/

            $stmtPSSS = $this->pdo->prepare("UPDATE `POS_Store_Session_Summary` SET `Session_Amount` = ?,`Changed_Date` = ?, `Changed_By` = ? WHERE `POS_Store_Session_Summary`.`Store_Session_ID` = ?");
            $stmtPSSS->execute([$amt, $Creation_Date, $data['Created_By'], $data['Store_Session_ID']]);

        } else {
            ### Insert into POS_Store_Session_Summary
            /*$sql2  = "INSERT INTO `POS_Store_Session_Summary` (`Store_Session_ID`, `Payment_Type_ID`, `Session_Amount`, `Creation_Date`, `Created_By`, `Changed_Date`, `Changed_By`)
            VALUES ('".$data['Store_Session_ID']."', '".$storePaymentType."', '".$data['Session_Amount']."', '".$Creation_Date."', '".$data['Created_By']."', '".$Creation_Date."', '".$data['Created_By']."')";
            $pdo->query($sql2);*/

            $stmtPSSS = $this->pdo->prepare("INSERT INTO `POS_Store_Session_Summary` (`Store_Session_ID`, `Payment_Type_ID`, `Session_Amount`, `Creation_Date`, `Created_By`, `Changed_Date`, `Changed_By`)
        VALUES (?, ?, ?, ?, ?, ?, ?)");
            $stmtPSSS->execute([$data['Store_Session_ID'], $storePaymentType, $data['Session_Amount'], $Creation_Date, $data['Created_By']]);
        }


    }

    private function createLogPO($data){
        Log::notice('Request Data for place_order => createLogPO', ['request' => $data]);

        $dataNew = [];
        $logMe = 1;
        if($logMe == 1){
            $logdate = date("d-m-Y H:i:s");
            $log = __DIR__.'/../logs/po_log.txt';
            $dataNew[$logdate] = $data;
            Log::notice('Response Data for place_order => createLogPO', ['response' => $dataNew]);

            //file_put_contents($log, print_r($dataNew, true), FILE_APPEND | LOCK_EX );
            Storage::disk('local')->put($log,print_r($dataNew, true));
        }
    }
}
