<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Http\Controllers\ApiValidation;
use PDO;
use PDOException;

class UpsertArticleController extends Controller
{
    /*
    Request
    {
       "page": 1, //Mandatory, NotNull, non zero Integer (2) current page (hardcoded value For chitale 1)
       "total_pages": 10, // Mandatory, NotNull, non zero Integer (2) , total No. of pages that would be sent today before 23:59:59 hours (hardcoded value For chitale 1)
       "articles" : //entire article information is for case_size = 1
       [
        {
            "article_no"         : 14, //Optional, NotNull and Integer(max 10), sk article no
            "legacy_article_no"  : "1234", //Mandatory, NotNull and String(max 30), legacy article no
            "article_desc"       : "", //Mandatory, String(500), article name
            "article_short_desc" : "", // Optional String(500), article small description
            "gst"                : 2, //Mandatory, Integer(max 3) ==> less than or equel to 100
            "brand_id"           : 100, //Mandatory, NotNull and Integer(max 10)
            "volume"             : "", //Mandatory, decimal(6,2)
            "unit"               : "", //Mandatory, string 100, valid values ('ML','KG','GM','PACK','BOX','PC','PKT','LTR','PCS','NOS','PAIR')
            "EAN"                : "", // Optional, string(13)
            "status"             : "A", // Mandatory, string(1) default A valid values A/I/D
            "dietary_code"       : "", // Optional, Integer(max 3), default 0, dietary code for chitale is 1 (1 means for applicable for food coupons like SODEXO, 0 means not applicable)
            "image_url"          : "", //Optional, String(100), image url
            "more_images"        : 	[ //Optional
                {
                    "legacy_image_url" : "", //Mandatory, String(100), image url
                    "status"           : "A" // Mandatory, string 1 default A valid values A/I/D
                }
            ], // Optional, should give large image file
            "approx_weight"      : "", //Optional, decimal(20,2) in KG
            "article_tag"        : "", //Optional, string(200), tag based searching
            "HSN"                : "", // Mandatory, string(8)
            "schedule"           : "", //Mandatory,int(3), should be 1/2/3/4
            "case_sizes"         : [ //Mandatory, array atleast 1 element case_size = 1 is Mandatory
                {
                    "case_size" : 1,  //Mandatory, integer(max 11) default 1  ===> Case_Size  1 no change more than one case size one record as well as another record
                    "status"    : "A" // Mandatory, string(1) default "A", valid values A/I/D
                }
            ],
            "tech_spec"          : "", //Optional, string(5000), Technical Specification
            "product_pdf"        : "", //Optional, string(200),  PDF file url of product/article
            "composition"        : "", //Optional, string(200), Product composition details
            "V_Mfg"              : null, //Optional, string(100), Default null
            "V_Mfg_Month_Year"   : null, //Optional, Default null
            "V_Model"            : null, //Optional, string(250), Default null,
            "article_languages"  : 	[ //Optional
                {
                    "lang_code"  		: "EN", // Mandatory, default "EN", not null, valid values EN/HI/MR
                    "article_desc_lang" : "", // Mandatory, default empty, not null
                    "status"           	: "A" // Mandatory, string(1) default "A" valid values A/I/D
                }
            ],
            "product_type"	    "", //Mandatory, String(100), ProductType
            "product"	    "", //Mandatory, String(100), Product
            "sub_product"	    ""  //Mandatory, String(100), subProduct
        }
       ]
    }
    */

    /**** This API will do insert or upsert Article_Master
     * Upserted Table: Article_Master
     * Select Tables:
     */
    public function upsert_articles(Request $request){
        $data = array();
        $status = false;
        $post = $request->post();
        extract($post);
        $articles = $post['articles'];
        // print_r($articles);die;
        // date_default_timezone_set("Asia/Kolkata");
        $headBusinessId = $request->header('BusinessId');
        if(!isset($headBusinessId) || $headBusinessId == '' || $headBusinessId == 0){
            $headBusinessId = getDBBusinessId();
        }

        if(!isset($headBusinessId) || $headBusinessId == '' || $headBusinessId == 0){
            $status = false;
            fwrite($myfile, PHP_EOL);
            fwrite($myfile, date("d-m-Y h:i:sa"));
            fwrite($myfile, "   upsert_articles   Response Data : ");
            $restxt = json_encode($response);
            fwrite($myfile, $restxt);
            $restxt = json_encode("BusinessId missing in header");
            fwrite($myfile, $restxt);
            $restxt = json_encode($status);
            fwrite($myfile, $restxt);
            fclose($myfile);
            return get_response("BusinessId missing in header", $status);
        }

        ## Get Pricing mode (Config_Value) from Business_Config_Master which config attribute is PRICING and Business Id is current business Id. Default pricing mode is DECENTRALIZED
        $queryPricing = $this->pdo->prepare("Select Config_Value from Business_Config_Master
					where Business_Id = ?
					and   Config_Attribute='PRICING'
					and   Status='A'");
        $queryPricing->execute([$headBusinessId]);
        $resultPricing = $queryPricing->fetch(PDO::FETCH_ASSOC);
        $pricingMode ='DECENTRALIZED'; //CENTRALIZED
        if($resultPricing){
            $pricingMode   = $resultPricing['Config_Value'];
        }

        if($pricingMode == 'DECENTRALIZED'){
            $status = false;
            fwrite($myfile, PHP_EOL);
            fwrite($myfile, date("d-m-Y h:i:sa"));
            fwrite($myfile, "   upsert_articles   Response Data : ");
            $restxt = json_encode($response);
            fwrite($myfile, $restxt);
            $restxt = json_encode("This API is not meant for this decentralize business.");
            fwrite($myfile, $restxt);
            $restxt = json_encode($status);
            fwrite($myfile, $restxt);
            fclose($myfile);
            return get_response("This API is not meant for this decentralize business.", $status);
        }

        ## legacy article no number numeric validation is is depend on pricing mode,
        # if it is CENTRALIZED then legacy article number is mandatory and
        # if it is DECENTRALIZED then legacy article number is not mandatory
        $legacy_article_no_validations = 'alnum|10||[>]0';
        if($pricingMode == 'CENTRALIZED'){
            $legacy_article_no_validations = "mandatory-alnum|10||[>]0";
        }

        ## Get brand ids from brand master
        $brandQuery = $this->pdo->prepare("SELECT Brand_ID FROM `Brand_Master` WHERE Status = 'Y' ");
        $brandQuery->execute();
        $brands = $brandQuery->fetchAll(PDO::FETCH_ASSOC);
        $brandIds = '';
        if(!empty($brands)){
            $brandIds = array_column($brands, 'Brand_ID');
            $brandIds = implode('/',$brandIds);
        }

        ## Get Units from Unit Master
        $unitQuery = $this->pdo->prepare("SELECT Unit_Name FROM `Unit_Master`");
        $unitQuery->execute();
        $units = $unitQuery->fetchAll(PDO::FETCH_ASSOC);
        $unitIds = '';
        if(!empty($units)){
            $unitIds = array_column($units, 'Unit_Name');
            $unitIds = implode('/',$unitIds);
        }

        ## Decide request parametes required validations
        $dataValidations =
            [
                "article_no" => "numeric|10||[>=]0",
                "legacy_article_no" => $legacy_article_no_validations,
                "article_desc" => "mandatory-text|500",
                "article_short_desc" => "mandatory-text|500",
                "gst" => "mandatory-numeric|2",
                "brand_id" => "mandatory-numeric|10|$brandIds",
                "volume" => "mandatory-decimal|[>]|0.00",
                "unit" => "mandatory-alpha|100|$unitIds",
                "EAN" => "alnum|13",
                "status" => "mandatory-alpha|1|A/I/D",
                "dietary_code" => "numeric|3|0/1",
                //"image_url" => "mandatory-file",
                "more_images" => "array",
                "approx_weight" => "decimal||0.00",
                "article_tag" => "alnum|200",
                "HSN" => "mandatory-numeric|8",
                "schedule" => "mandatory-numeric|3|1/2/3/4",
                "case_sizes" => "mandatory-array",
                "tech_spec"          => "text|5000",
                //"product_pdf"        => "file",
                "composition"        => "text|200",
                "V_Mfg"              => "text|100|null",
                "V_Mfg_Month_Year"   => "text|10|null",
                "V_Model"            => "text|250|null",
                "article_languages" => "array",
                "brand_shop_name" => "mandatory-text|100",
                "product_type" => "mandatory-text|100",
                "product" => "text|100",
                "sub_product" => "text|100"
            ];

        $moreImagesValidations = [
            //"legacy_image_url" => "mandatory-alnumslash",
            "status" => "mandatory-alpha|1|A/I/D"
        ];

        $caseSizesValidations = [
            "case_size" => "mandatory-numeric|11|1|[>]0",
            "status" => "mandatory-alpha|1|A/I/D"
        ];

        $languageValidations = [
            "lang_code" => "mandatory-alpha|3|EN/HI/MR",
            "article_desc_lang" => "mandatory-utf16text",
            "status" => "mandatory-alpha|1|A/I/D"
        ];

        $msg = [];
        try {

            $responseData = [];
            $successRows = [];
            $failedRows = [];
            $sk_article_no = '';
            $moreImagesErrors = [];
            $caseSizesErrors = [];
            $amResponses = [];
            $responseCS = [];
            $filename = __DIR__.'/app_data.log';
            $myfile = fopen($filename, "a");
            $reqtxt = json_encode($articles);
            fwrite($myfile, PHP_EOL);
            fwrite($myfile, date("d-m-Y h:i:sa"));
            fwrite($myfile, "   upsert_articles   Request Data : ");
            fwrite($myfile, $reqtxt);
            if(!empty($articles)){

                foreach($articles as $articleIndex=>$articleDetails){
                    $insertArticleDetails = [];
                    $responseData = [];
                    $amResponses = [];
                    $legacy_article_no = $articleDetails['legacy_article_no'];
                    $Display_Seq_No = $articleDetails['display_Seq_No'];
                    $Display_Seq_Text = $articleDetails['display_Seq_Text'];

                    $processed = [];
                    $article_no         = $articleDetails['article_no'] ?? null;
                    $legacy_article_no  = $articleDetails['legacy_article_no'] ?? null;
                    $case_sizes         = $articleDetails['case_sizes'] ?? [];

                    // print_r($case_sizes);die;
                    $articleDetails['business_id']  = $headBusinessId;

                    ##validation start############
                    #1. check API validation first
                    $errors = ApiValidation::validateAPIData($dataValidations, $articleDetails);
                    //exit;
                    $moreImgsErrors = [];
                    if(isset($articleDetails['more_images'])){
                        $moreImgsDetails = $articleDetails['more_images'];
                        if(!empty($moreImgsDetails)){
                            foreach($moreImgsDetails as $i => $moreImgsDetail){
                                $miErrors = ApiValidation::validateAPIData($moreImagesValidations, $moreImgsDetail);
                                if(!empty($miErrors)) {
                                    if(isset($moreImgsDetail['large_image_url'])){
                                        $moreImgsErrors[$i]['legacy_image_url'] = $moreImgsDetail['large_image_url'];
                                    }
                                    $moreImgsErrors[$i]['reason'] = $miErrors;
                                    $moreImgsErrors[$i]['processed'] = false;
                                }
                            }
                        }
                    }

                    $caseSizesErrors = [];
                    $caseSizesDetails = $case_sizes;
                    if(!empty($caseSizesDetails)){
                        foreach($caseSizesDetails as $i => $caseSizesDetail){
                            $czErrors = ApiValidation::validateAPIData($caseSizesValidations, $caseSizesDetail);
                            if(!empty($czErrors)) {
                                if(isset($caseSizesDetail['case_size'])) {
                                    $caseSizesErrors[$i]['case_size'] = $caseSizesDetail['case_size'];
                                }
                                $caseSizesErrors[$i]['reason'] = $czErrors;
                                $caseSizesErrors[$i]['processed'] = false;
                            }
                        }
                    }

                    $articleLangErrors = [];
                    if(isset($articleDetails['article_languages'])){
                        $articleLanguagesDetails = $articleDetails['article_languages'];
                        if(!empty($articleLanguagesDetails)){
                            foreach($articleLanguagesDetails as $i => $articleLanguagesDetail){
                                $csErrors = ApiValidation::validateAPIData($languageValidations, $articleLanguagesDetail);
                                if(!empty($csErrors)) {
                                    if(isset($articleLanguagesDetail['lang_code'])){
                                        $articleLangErrors[$i]['lang_code'] = $articleLanguagesDetail['lang_code'];
                                    }
                                    if(isset($articleLanguagesDetail['article_desc_lang'])){
                                        $articleLangErrors[$i]['article_desc_lang'] = $articleLanguagesDetail['article_desc_lang'];
                                    }
                                    $articleLangErrors[$i]['reason'] = $csErrors;
                                    $articleLangErrors[$i]['processed'] = false;
                                }
                            }
                        }
                    }

                    ##Article insert or update
                    # if no validation error
                    $case1ArticleNo = 0;
                    if(empty($errors) && empty($caseSizesErrors) && empty($moreImgsErrors)){

                        $where = '';
                        $article_no = $articleDetails['article_no'] ?? 0;
                        $Display_Seq_No = $articleDetails['display_Seq_No'];
                        $Display_Seq_Text = $articleDetails['display_Seq_Text'];

                        $legacy_article_no = $articleDetails['legacy_article_no'];
                        $rowStatus = $articleDetails['status'];

                        # Validate legacy article no
                        if($legacy_article_no > 0){
                            $where .= 'ASPL.Supplier_Article_No = '.$legacy_article_no.' AND Supplier_No = 0';
                        }
                        /* if($article_no > 0){
                            $where .= ' AND AM.Article_No = '.$article_no;
                        } */
                        $validArticleSQL = "SELECT AM.Article_No, AM.Case_Size FROM `Article_Master` AM
                        LEFT JOIN Article_Supplier_Product_Lookup ASPL ON ASPL.Article_No = AM.Article_No
                        WHERE $where ";
                        $queryArticle = $this->pdo->prepare($validArticleSQL);
                        $queryArticle->execute();
                        /****
                         * commented by sheetal
                         * Reason : as per discussion by daya this line is same as print_r to sql statement
                         * Commented at 2 feb 2022
                         */
                        // $queryArticle->debugDumpParams();
                        $getMappedArticles = $queryArticle->fetchAll(PDO::FETCH_ASSOC);

                        $actionOnArticle = 'insert';
                        $isValidArticle = false;
                        $dbCaseSizes = [];
                        if(!empty($getMappedArticles)){
                            $dbCaseSizes = array_column($getMappedArticles,'Case_Size', 'Article_No');
                        }

                        if(!empty($dbCaseSizes)){
                            ## Get new case size and existing case sizes array differently
                            $csWithStatus = (array_column($articleDetails['case_sizes'], 'status', 'case_size'));
                            $requestedCaseSizes = (array_column($articleDetails['case_sizes'], 'case_size'));
                            $newCaseSizes = array_diff($requestedCaseSizes, $dbCaseSizes);
                            $updateCaseSizes = array_diff($requestedCaseSizes, $newCaseSizes);

                            #prepare insert array with new case size
                            if(!empty($newCaseSizes)){
                                $articleDetails['case_sizes'] = [];
                                foreach($newCaseSizes as $i => $newCS){
                                    $articleDetails['case_sizes'][$i]['case_size'] = $newCS;
                                    $articleDetails['case_sizes'][$i]['status'] = $csWithStatus[$newCS];
                                }
                                $insertArticleDetails = $articleDetails;
                            }

                            # prepare array of case size which are present in Article Master
                            # $updateCaseSizes this array will be pass if only requested case size to update
                            if(!empty($updateCaseSizes)){
                                $articleDetails['case_sizes'] = [];
                                foreach($updateCaseSizes as $i => $updCS){
                                    $articleDetails['case_sizes'][$i]['case_size'] = $updCS;
                                    $articleDetails['case_sizes'][$i]['status'] = $csWithStatus[$updCS];
                                }
                                $updateArticleDetails = $articleDetails;
                            }

                            #Preapare array with all db case sizes, because all existing case sizes data need to update ont only updateArticleDetails (requested case sizes data)
                            $updateArticleDetailsWithAllCaseSizes = '';
                            if(!empty($dbCaseSizes)){
                                $articleDetails['case_sizes'] = [];
                                foreach($dbCaseSizes as $i => $dbCS){
                                    $articleDetails['case_sizes'][$i]['case_size'] = $dbCS;
                                    if(isset($csWithStatus[$dbCS])) $articleDetails['case_sizes'][$i]['status'] = $csWithStatus[$dbCS];
                                }
                                $updateArticleDetailsWithAllCaseSizes = $articleDetails;
                            }

                            # Response will show case size 1 article no
                            if(in_array(1, $dbCaseSizes)){
                                $case1ArticleNo = array_search(1, $dbCaseSizes);
                            }

                        } else {
                            $insertArticleDetails = $articleDetails;
                        }

                        //print "<br>actionOnArticle==".$actionOnArticle;
                        $responseCS1 = []; $responseCS2 = [];
                        if(!empty($updateArticleDetailsWithAllCaseSizes)){

                            $responseData['article_no'] = $case1ArticleNo;
                            $responseData['legacy_article_no'] = (int)$legacy_article_no;

                            /* if($isValidArticle == false){
                                $failedRows[] = $sk_article_no;
                                $responseData['reason'] = 'Article number not found';
                                $responseData['margins'] = [];
                                $data['articles'][] = $responseData;
                            } else { */
                            $articleUpsertResponses = $this->upsertArticleData($this->pdo, 'update', $updateArticleDetailsWithAllCaseSizes, $dbCaseSizes, $articleIndex);
                            $amResponses = $articleUpsertResponses['successRecords'];
                            $responseCS1 = $articleUpsertResponses['responseCS'];
                            $responseMI = $articleUpsertResponses['responseMI'];
                            $responseAL = $articleUpsertResponses['responseAL'];

                            # send notification to all cust of the business
                            //send_api_notification($this->pdo,'upsert_articles');
                            //}
                        }

                        if(!empty($insertArticleDetails)){
                            # insert
                            $articleUpsertResponses = $this->upsertArticleData($this->pdo, 'insert', $insertArticleDetails, $dbCaseSizes, $articleIndex);
                            $amResponses = $articleUpsertResponses['successRecords'];

                            $responseCS2 = $articleUpsertResponses['responseCS'];
                            $responseMI = $articleUpsertResponses['responseMI'];
                            $responseAL = $articleUpsertResponses['responseAL'];

                            # get case size 1 article no
                            if(isset($amResponses[$legacy_article_no]) && !empty($amResponses[$legacy_article_no])){
                                $case1ArticleNo = array_search(1, $amResponses[$legacy_article_no]);
                                $responseData['article_no'] = $case1ArticleNo;
                            }
                            $responseData['legacy_article_no'] = (int)$legacy_article_no;

                            # send notification to all cust of the business
                            //send_api_notification($this->pdo,'upsert_articles');
                        }

                        $responseCS = array_merge($responseCS1, $responseCS2);

                        if(!empty($amResponses)){
                            $responseData['processed'] = true;
                            $responseData['reason'] = 'Success';
                        } else {
                            $responseData['processed'] = false;
                            $responseData['reason'] = 'Failed';
                        }

                        if(!empty($responseCS)){

                            $responseData['case_sizes'] = $responseCS;
                            $reasons = array_column($responseData['case_sizes'], 'reason');
                            $processed = array_column($responseData['case_sizes'], 'processed');
                        }

                        if(!empty($responseMI)){
                            $responseData['more_images'] = $responseMI;
                        }

                        if(!empty($responseAL)){
                            $responseData['article_languages'] = $responseAL;
                        }
                        $data['articles'][] = $responseData;

                    } else {
                        $status = false;
                        $responseData['article_no'] = $articleDetails['article_no'];
                        $responseData['legacy_article_no'] = $articleDetails['legacy_article_no'];
                        if(!empty($errors)){
                            $responseData['reason'] = $errors;
                        } else {
                            $responseData['reason'] = "Failed";
                        }

                        if(!empty($caseSizesErrors)){
                            $responseData['case_size'] = array_values($caseSizesErrors);
                        }
                        if(!empty($moreImgsErrors)){
                            $responseData['more_images'] = array_values($moreImgsErrors);
                        }

                        if(!empty($articleLangErrors)){
                            $responseData['article_languages'] = array_values($articleLangErrors);
                        }
                        $data['articles'][] = $responseData;
                        $failedRows[] = $sk_article_no;
                        // continue;
                    }

                    if(!empty($amResponses)){
                        foreach($amResponses as $legArtNo => $amResponse){
                            $case_sizes = array_values($amResponse);
                            $successRows[] = "$legArtNo with case size(s) (".implode(", ",array_unique($case_sizes)).")";
                        }
                    }
                }// end foreach $articles

                $allErrors = [];

                if(!empty($successRows)){
                    if(in_array(false, $processed)){
                        $msg[] = implode(',', $reasons);
                        $status = false;
                    } else {
                        $msg[] = 'Article data updated successfully for above legacy article number '.implode(", ",$successRows)." ";
                        $status = true;
                    }
                } else {
                    $msg[] = ' Failed: Article data not inserted or updated ';
                    $status = false;
                }

                $msg = implode(",", $msg);
                fwrite($myfile, PHP_EOL);
                fwrite($myfile, date("d-m-Y h:i:sa"));
                fwrite($myfile, "   upsert_articles   Response Data : ");
                $restxt = json_encode($response);
                fwrite($myfile, $restxt);
                $restxt = json_encode($msg);
                fwrite($myfile, $restxt);
                $restxt = json_encode($status);
                fwrite($myfile, $restxt);
                $restxt = json_encode($data);
                fwrite($myfile, $restxt);
                fclose($myfile);
                return get_response($msg, $status, $data);

            }
            fwrite($myfile, PHP_EOL);
            fwrite($myfile, date("d-m-Y h:i:sa"));
            fwrite($myfile, "   upsert_articles   Response Data : ");
            $restxt = json_encode($response);
            fwrite($myfile, $restxt);
            $restxt = json_encode($msg);
            fwrite($myfile, $restxt);
            $restxt = json_encode($status);
            fwrite($myfile, $restxt);
            $restxt = json_encode($data);
            fwrite($myfile, $restxt);
            fclose($myfile);
            return get_response($msg, $status, $data);
        }
        catch (PDOException $e)
        {
            $status = false;
            $data = $e->getMessage();
            fwrite($myfile, PHP_EOL);
            fwrite($myfile, date("d-m-Y h:i:sa"));
            fwrite($myfile, "   upsert_articles   Response Data : ");
            $restxt = json_encode($data);
            fwrite($myfile, $restxt);
            $restxt = json_encode($response);
            fwrite($myfile, $restxt);
            $restxt = json_encode("Something went wrong");
            fwrite($myfile, $restxt);
            $restxt = json_encode($status);
            fwrite($myfile, $restxt);
            fclose($myfile);
            return get_response("Something went wrong", $status);
        }
    }

    public function upsert_article_margin_pricing(Request $request){
        define ("RETAILER","Retailer");
        define ("DISTRIBUTOR","Distributor");
        define ("SUPERSTOCKIEST","Superstockiest");
        define ("SSSTOCKIEST","SSstockiest");
        define ("COCO","COCO");
        define ("COFO","COFO");
        define ("FOFO","FOFO");
        define ("FOCO","FOCO");
        define ("MARGIN_RETAILER","Margin_Retailer");
        define ("MARGIN_DISTRIBUTOR","Margin_Distributor");
        define ("MARGIN_SS","Margin_SS");
        define ("MARGIN_SSS","Margin_SSS");
        define ("MARGIN_COCO","Margin_COCO");
        define ("MARGIN_COFO","Margin_COFO");
        define ("MARGIN_FOFO","Margin_FOFO");
        define ("MARGIN_FOCO","Margin_FOCO");
        $Seller_Margin_Value_Column = [
            RETAILER => MARGIN_RETAILER,
            DISTRIBUTOR => MARGIN_DISTRIBUTOR,
            SUPERSTOCKIEST =>MARGIN_SS,
            SSSTOCKIEST=>MARGIN_SSS,
            COCO =>MARGIN_COCO,
            COFO =>MARGIN_COFO,
            FOFO =>MARGIN_FOFO,
            FOCO =>MARGIN_FOCO
        ];

        $data = array();
        $status = false;
        $post = $request->post();
        // print_r($post);die;
        extract($post);
        $headBusinessId = $request->header('BusinessId');
        if(!isset($headBusinessId) || $headBusinessId == '' || $headBusinessId == 0){
            $headBusinessId = getDBBusinessId();
        }

        ##get seller types from business config master
        $sTypes = '';
        $configSellerTypes = [];
        $stmt = $this->pdo->prepare("Select Config_Value as seller_type, CAST(SUBSTRING_INDEX(SUBSTRING_INDEX(Config_Value_Code, '_', 1), '_', -1) AS UNSIGNED ) as config_seq, SUBSTRING_INDEX(SUBSTRING_INDEX(Config_Value_Code, '_', 2), '_', -1) as config_ref from Business_Config_Master where Business_Id = ? and Config_Attribute='RETAILER_TYPE' and Status='A' order by Config_Value_Code");
        $stmt->execute([$headBusinessId]);
        $configSellerTypes = $stmt->fetchAll(PDO::FETCH_ASSOC);

        if(!empty($configSellerTypes)){
            $st = array_column($configSellerTypes, 'seller_type');
            $sTypes = implode('/',$st);
            # Make seller type array
            $configSellerTypes = array_column($configSellerTypes, null, 'seller_type');
        }

        ## Get Pricing mode (Config_Value) from Business_Config_Master which config attribute is PRICING and Business Id is current business Id. Default pricing mode is DECENTRALIZED
        $queryPricing = $this->pdo->prepare("Select Config_Value from Business_Config_Master
					where Business_Id = ?
					and   Config_Attribute='PRICING'
					and   Status='A'");
        $queryPricing->execute([$headBusinessId]);
        $resultPricing = $queryPricing->fetch(PDO::FETCH_ASSOC);
        $pricingMode ='DECENTRALIZED'; //CENTRALIZED
        if($resultPricing){
            $pricingMode   = $resultPricing['Config_Value'];
        }
        ## legacy article no number numeric validation is is depend on pricing mode,
        # if it is CENTRALIZED then legacy article number is mandatory and
        # if it is DECENTRALIZED then legacy article number is not mandatory
        $legacy_article_no_validations = 'numeric|10';
        if($pricingMode == 'CENTRALIZED'){
            $legacy_article_no_validations = "mandatory-numeric|[>]0";
        }
        ## Decide request parametes required validations
        $dataValidations =
            [
                "article_no" => "numeric|10||[>=]0",
                "legacy_article_no" => $legacy_article_no_validations,
                "merchant_type" => "mandatory-alpha||Business/Store",
                "business_id" => "mandatory-numeric||2/3",
                "store_no" => "numeric|10|0",
                "MRP" => "mandatory-decimal|[>=]0",
                "list_price" => "decimal|[>=]|0.00",
                "version" => "mandatory-numeric||1",
                "status" => "mandatory-alpha|1|A/I/D",
                "margins" => "array",
                "valid_start_date" => "mandatory-date|today|[>=]".strtotime(date('Y-m-d'))
            ];


        # seller type validation
        $marginTradeValidations = [
            "seller_type" => "mandatory-alpha|50|$sTypes",
            "margin_value" => 'numeric||0.00|'
        ];

        $msg = [];

        // date_default_timezone_set("Asia/Kolkata");
        try {

            $filename = __DIR__.'/app_data.log';
            $myfile = fopen($filename, "a");
            $reqtxt = json_encode($articles);
            fwrite($myfile, PHP_EOL);
            fwrite($myfile, date("d-m-Y h:i:sa"));
            fwrite($myfile, "   upsert_article_margin_pricing   Request Data : ");
            fwrite($myfile, $reqtxt);
            $responseData = [];
            $successRows = [];
            $failedRows = [];
            $sk_article_no = '';

            if(!empty($articles)){
                // $logger->info(json_encode($articles));
                ## Get articles version numbers and Sort articles array with version number
                $versionArr = array_column($articles, 'version');
                array_multisort($versionArr, SORT_ASC, $articles);

                foreach($articles as $i=>$articleDetails){
                    $failedRows = [];
                    $responseData = [];

                    if(isset($articleDetails["margins"]) && ($articleDetails["margins"] == NULL)){

                        unset($articleDetails["margins"]);
                    }

                    foreach($articleDetails["margins"] as $marginskey => $marginsdata){
                        if(!in_array($marginsdata['seller_type'],$st))
                        {
                            unset($articleDetails["margins"][$marginskey]);
                        }
                    }


                    $sk_article_no = '';
                    $article_no = 0;
                    $validArticleRecords = array();
                    $caseSize = '';
                    $articledata = $articleDetails;
                    $legacy_article_no = 0;
                    $legacy_article_no = $articleDetails['legacy_article_no'];
                    $version = $articleDetails['version'];
                    $mrpValidate = $articleDetails['MRP']?? 0;
                    $listpriceValidate = $articleDetails['list_price'];
                    $articleValidate = $articleDetails['article_no'];


                    ##validation start############
                    #1. check API validation first
                    $errors = ApiValidation::validateAPIData($dataValidations, $articleDetails);
                    #2. margin array validations

                    $marginsErrors = [];
                    $marginDetails = [];
                    if(isset($articleDetails['margins'])){
                        $marginDetails = $articleDetails['margins'];
                    }

                    if(!empty($marginDetails)){
                        foreach($marginDetails as $tradeName => $tradeDetail){
                            $index = $tradeDetail['seller_type'];
                            $mErrors = ApiValidation::validateAPIData($marginTradeValidations, $tradeDetail);
                            if(!empty($mErrors)) {
                                $marginsErrors[$tradeName]['seller_type'] = $index;
                                $marginsErrors[$tradeName]['reason'] = $mErrors;
                            }
                        }
                    }
                    #validation end###################

                    ##Article margin insert or update
                    # if no validation error
                    // print_r($mrpValidate);die;
                    if(empty($errors) && empty($marginsErrors) && $mrpValidate >= 0 && $listpriceValidate >= 0 && $listpriceValidate <= $mrpValidate && $articleValidate >= 0 ){
                        $article_no = $articleDetails['article_no'] ?? 0;
                        $legacy_article_no = $articleDetails['legacy_article_no'];
                        $rowStatus = $articleDetails['status'];


                        $merchant_type = $articleDetails['merchant_type'];
                        $business_id = $articleDetails['business_id'];
                        $store_no = $articleDetails['store_no'];

                        # Get filter id (business id or store no), it will depend upon merchant_type
                        # Validation of json request legacy article no or article number is depend on pricing mode, if pricing mode is CENTRALIZED then validate with Article_Supplier_Product_Lookup table else Store_Listing table
                        if($merchant_type == 'Business' && $pricingMode == 'CENTRALIZED'){
                            $filterId = $business_id;

                            $validArticleSQL = "SELECT AM.Article_No, AM.Case_Size FROM `Article_Master` AM
                            LEFT JOIN Article_Supplier_Product_Lookup ASPL ON ASPL.Article_No = AM.Article_No
                            WHERE ASPL.Supplier_Article_No = $legacy_article_no group by ASPL.Article_No";

                        } else {
                            $filterId = $store_no;
                            $validArticleSQL = "SELECT AM.Article_No, AM.Case_Size FROM `Article_Master` AM
                            LEFT JOIN Store_Listing SL ON SL.Article_No = AM.Article_No
                            WHERE AM.Article_No = $article_no AND SL.Store_No = ".$filterId;
                        }

                        $validArticle = $this->pdo->prepare($validArticleSQL);
                        $validArticle->execute();
                        // $validArticle->debugDumpParams();
                        $validArticleRecords = $validArticle->fetchAll(PDO::FETCH_ASSOC);

                        if(!empty($validArticleRecords)){
                            $isValid = true;

                            # compare article_no with fetched records (validArticleRecords) article no,
                            # if exists then process else article number not found
                            $currentArticleNoFromDB = array_column($validArticleRecords, 'Article_No');
                            if($article_no > 0 && !in_array($article_no, $currentArticleNoFromDB)){
                                $isValid = false;
                                $failedRows[] = "'".$sk_article_no."':'".$legacy_article_no."'";
                                $responseData['article_no'] = $article_no ;
                                $responseData['legacy_article_no'] = $legacy_article_no;
                                $responseData['version'] = $version;
                                $responseData['case_size'] = $caseSize;
                                $responseData['reason'] = 'Article number not found';
                                $responseData['margins'] = [];
                                $data['articles'][] = $responseData;

                                // TODO  check if continue is needed here  after error has been records for that a.
                            }


                            #If is valid then process futher
                            if($isValid == true){
                                $responseData['reason'] = 'success';
                                $responseMargings = [];
                                foreach($validArticleRecords as $validArticleRecord)
                                {


                                    $sk_article_no = '';
                                    /*
                                    Name :Sheetal Chive
                                    Changed : Commended Below Line And Get Article No From validArticleRecord[]
                                    Reason : Different case Size Should be update for same article  (Which was not updated)
                                    Date : 2-5-22
                                    */

                                    // $responseData['article_no'] = $article_no;


                                    $caseSize = $validArticleRecord['Case_Size'];//get it from db
                                    $article_no =  $validArticleRecord['Article_No'] ;
                                    $responseData['article_no'] = $caseSize == 1 ? $validArticleRecord['Article_No'] : $responseData['article_no']  ;


                                    $responseData['legacy_article_no'] = $legacy_article_no;
                                    $MRP = $articleDetails['MRP'];
                                    $list_price = $articleDetails['list_price'];
                                    $valid_start_date = $articleDetails['valid_start_date'];
                                    $valid_end_date = "2099-12-31";
                                    $version = $articleDetails['version'];

                                    $responseData['version'] = $version;
                                    $responseData['case_size'] = ($responseData['case_size'] ??'').",".$caseSize;
                                    #### CHECK THIS CODE FOR ARTICLE FORLOOP... NEED CHANGE
                                    if( $article_no == 0 || $article_no == ""){
                                        $sk_article_no = $validArticleRecord['Article_No'];//get it from db
                                        $responseData['article_no'] = (int)$sk_article_no;
                                    } else $sk_article_no = $article_no;

                                    #get article all existing margins
                                    $articleAllMargins = $this->getArticleMargins($this->pdo, $sk_article_no, $filterId, $merchant_type);
                                    $articleAllMarginsArr = [];
                                    if(!empty($articleAllMargins)){
                                        foreach($articleAllMargins as $aam){
                                            $index = $aam['Version'];
                                            $articleAllMarginsArr[$index] = $aam;
                                        }
                                    }
                                    #get json request version number article margin array
                                    $articleCurrentMargins = [];
                                    if (array_key_exists($version, $articleAllMarginsArr)){
                                        $articleCurrentMargins = $articleAllMarginsArr[$version];
                                    }

                                    #base price calculations
                                    $commonBasePrice = 0;
                                    if( isset($list_price) && $list_price > 0 ){
                                        $commonBasePrice = $list_price;
                                    } else {
                                        $commonBasePrice = $MRP ?? 0;
                                    }

                                    $ID = 0;
                                    // $margin_retailer = 0;
                                    // $margin_distributor = 0;
                                    // $margin_ss = 0;
                                    // $margin_sss = 0;
                                    // $margin_coco = 0;
                                    // $margin_cofo = 0;
                                    // $margin_fofo = 0;
                                    // $margin_foco = 0;
                                    $dbArticleMargins[] = [];


                                    # if current version article margin exists
                                    if(!empty($articleCurrentMargins)){
                                        $ID = $articleCurrentMargins['ID'];
                                        $dbVersion = $articleCurrentMargins['Version'];
                                        # get default margins for update
                                        $dbArticleMargins[RETAILER] = $articleCurrentMargins['Margin_Retailer'] ?? 0;
                                        $dbArticleMargins[DISTRIBUTOR] = $articleCurrentMargins['Margin_Distributor'] ?? 0;
                                        $dbArticleMargins[SUPERSTOCKIEST] = $articleCurrentMargins['Margin_SS'] ?? 0;
                                        $dbArticleMargins[SSSTOCKIEST] = $articleCurrentMargins['Margin_SSS'] ?? 0;
                                        $dbArticleMargins[COCO] = $articleCurrentMargins['Margin_COCO'] ?? 0;
                                        $dbArticleMargins[COFO] = $articleCurrentMargins['Margin_COFO'] ?? 0;
                                        $dbArticleMargins[FOFO] = $articleCurrentMargins['Margin_FOFO'] ?? 0;
                                        $dbArticleMargins[FOCO] = $articleCurrentMargins['Margin_FOCO'] ?? 0;
                                        $current_valid_start_date = $articleCurrentMargins['Valid_Start_Date'] ?? 0;
                                        $current_valid_end_date = $articleCurrentMargins['Valid_End_Date'] ?? 0;
                                    }

                                    # decide update or insert
                                    $insertMargin = false;
                                    $updateMargin = false;



                                    if(!empty($articleCurrentMargins) && $ID > 0){
                                        $updateMargin = true;
                                    } else {
                                        $insertMargin = true;
                                    }

                                    ## Get Previous and Post version Valid_Start_Date and convert to timestamp for next comparison
                                    $valid_start_date_timestamp = strtotime($valid_start_date);//current version start date timestamp
                                    $preVersionStartDate = '';
                                    $postVersionStartDate = '';
                                    $articleMarginsPreVersion = [];
                                    $articleMarginsPostVersion = [];
                                    //print "<pre>"; print_r($articleAllMarginsArr); exit;
                                    # Get requested version, previous and next version records if available in db
                                    $preIndex=$this->get_array_pre_index($articleAllMarginsArr, $version);
                                    $nextIndex=$this->get_array_next_index($articleAllMarginsArr, $version);
                                    if(!empty($articleAllMarginsArr)){
                                        # Get Previous version array
                                        if($preIndex != null)
                                        {
                                            # Get Previous version start date and convert it to timestamp
                                            $articleMarginsPreVersion = $articleAllMarginsArr[$preIndex];
                                            $preVersionStartDate = strtotime($articleMarginsPreVersion['Valid_Start_Date']);
                                        }

                                        # Get Next version array
                                        if($nextIndex != null){
                                            # Get Next version start date and convert it to timestamp
                                            $articleMarginsPostVersion = $articleAllMarginsArr[$nextIndex];
                                            $postVersionStartDate = strtotime($articleMarginsPostVersion['Valid_Start_Date']);
                                        }
                                    }

                                    $havingPrePostVersions = false;
                                    $havingPreVersion = false;
                                    $havingPostVersion = false;

                                    # validate current version start date
                                    # article margin can have multiple versions
                                    if(!empty($articleAllMarginsArr)){

                                        if($preVersionStartDate != '' && $postVersionStartDate != ''
                                            && $valid_start_date_timestamp > $preVersionStartDate
                                            && $valid_start_date_timestamp < $postVersionStartDate){
                                            # then valid date
                                            # update pre version end date = current version start date - 1 day
                                            # update current version end date = post version start - 1
                                            $havingPrePostVersions = true;
                                            //print "<br>havingPrePostVersions==".$havingPrePostVersions;

                                        } else if($preVersionStartDate != '' && $postVersionStartDate == '' &&
                                            $valid_start_date_timestamp > $preVersionStartDate){
                                            # then valid date
                                            # update preversion end date = current version start date - 1 day
                                            $havingPreVersion = true;
                                            //print "<br>havingPreVersion==".$havingPreVersion;
                                        } else if($postVersionStartDate != '' && $preVersionStartDate == ''
                                            && $valid_start_date_timestamp != '' && ($valid_start_date_timestamp >= strtotime($current_valid_start_date)
                                                && $valid_start_date_timestamp < strtotime($current_valid_end_date))){
                                            # then valid date
                                            # update current version end date = post version start - 1
                                            $havingPostVersion = true;
                                            //print "<br>havingPostVersion==".$havingPostVersion;
                                        }
                                        // Sheetal : added Below if condition
                                        else if($postVersionStartDate != '' && $preVersionStartDate == '' && $valid_start_date != '' ){
                                            # then valid date
                                            # update current version end date = post version start - 1
                                            $havingPostVersion = true;
                                            //print "<br>havingPostVersion==".$havingPostVersion;
                                        }
                                        else if($preVersionStartDate == '' && $postVersionStartDate == ''
                                            && $valid_start_date != ''){
                                            # table has single margin record, update only
                                            //print "<br>single margin record...........";
                                        } else {
                                            //print "<br>invalid date..........".
                                            # invalid date
                                            $status = false;
                                            $responseData['version'] = $version;
                                            $responseData['legacy_article_no'] = $legacy_article_no;
                                            $responseData['case_size'] = $caseSize;
                                            $responseData['reason'] = 'Invalid start date of article no '.$sk_article_no;
                                            //$responseData['margins'] = [];
                                            $data['articles'][] = $responseData;
                                            // $restxt = json_encode($data);
                                            // fwrite($myfile, PHP_EOL);
                                            // fwrite($myfile, date("d-m-Y h:i:sa"));
                                            // fwrite($myfile, "   Response Data : ");
                                            // fwrite($myfile, $restxt);
                                            continue;
                                            //return get_response("Invalid data", $status, $data);
                                        }
                                    }
                                    sleep(3);

                                    #Generate valid sequence of request margin array
                                    $requestedMargins = [];
                                    if(!empty($configSellerTypes) && !empty($marginDetails))
                                    {
                                        foreach($configSellerTypes as $configSellerType){
                                            $seq = (int)$configSellerType['config_seq'];
                                            foreach($marginDetails as $margin){
                                                if($configSellerType['seller_type'] == $margin['seller_type']){
                                                    $current_st = $margin['seller_type'];
                                                    $requestedMargins[$current_st]['seller_type'] = $margin['seller_type'];
                                                    $requestedMargins[$current_st]['config_seq'] = $configSellerType['config_seq'];
                                                    $requestedMargins[$current_st]['config_ref'] = $configSellerType['config_ref'];
                                                    $requestedMargins[$current_st]['margin_value'] = $margin['margin_value'] ?? 0;
                                                }
                                            }
                                        }
                                    }
                                    # add 0 index in array
                                    $zeroIndexArr['MRP']['seller_type'] = 'MRP';
                                    $zeroIndexArr['MRP']['config_seq'] = 0;
                                    $zeroIndexArr['MRP']['config_ref'] = '';
                                    $zeroIndexArr['MRP']['margin_value'] = $commonBasePrice;
                                    ## Final array
                                    $requestedMarginsNew = array_merge($zeroIndexArr, $configSellerTypes, $requestedMargins);

                                    // print_r($requestedMargins);die;
                                    #With requestedMarginsNew array calculate offer price and make calculated array
                                    $calculatedMarginArr = [];
                                    $updateparams = '';
                                    $sellertypes = ['MRP','Retailer','Distributor','COCO','FOFO','COFO','FOCO','Superstockiest','SSstockiest'];



                                    $Created_By = 1001;
                                    //$status = 'A';
                                    $Creation_Date = date('Y-m-d H:i:s');
                                    $updatevalues = [$list_price, $MRP];
                                    // $responseMargings = [];
                                    if(!empty($requestedMarginsNew)){
                                        $responseData['margins'] = [];// TODO check
                                        foreach($requestedMarginsNew as $requestedMarginNew)
                                        {
                                            $requestedMargin = '';
                                            #Get reference base price with requestedMarginNew['config_ref']
                                            $requestedRef = $requestedMarginNew['config_ref'] ?? '';
                                            $requestedSellerType = $requestedMarginNew['seller_type'] ?? '';

                                            # get reference base price from calculated array

                                            if($requestedRef == 'MRP'){
                                                $refBasePrice = $commonBasePrice;
                                            } else if($requestedRef != ''){
                                                $refBasePrice = $this->search_in_associative_arr($calculatedMarginArr, $requestedRef);
                                            } else {
                                                $refBasePrice = $commonBasePrice;
                                            }
                                            if(array_key_exists($requestedSellerType,$dbArticleMargins)){// ,----- $ST has it been activated
                                                if(isset($requestedMarginNew['margin_value']) ){
                                                    $updateparams .= $Seller_Margin_Value_Column[$requestedSellerType].' = ?, ';
                                                    array_push($updatevalues,$requestedMarginNew['margin_value']);
                                                } else {
                                                    $requestedMarginNew['margin_value'] = $dbArticleMargins[$requestedSellerType];
                                                    // $requestedMargin= $dbArticleMargins[$requestedSellerType];
                                                }
                                            }

                                            $requestedMarginNew['margin_value'] = $requestedMarginNew['margin_value'] ?? 0;

                                            $offerPrice = (($refBasePrice/(100+$requestedMarginNew['margin_value']))*100);
                                            // $offerPrice = callGetArticleOfferPrice($this->pdo, $merchant_type, $business_id, $legacy_article_no, $caseSize, $refBasePrice, $requestedMarginNew['margin_value'] );

                                            ####### make array with new values



                                            if($requestedSellerType == 'MRP'){
                                                $requestedMarginNew['offer_value'] = 0;
                                            } else {
                                                $requestedMarginNew['offer_value'] = $offerPrice;
                                            }
                                            $calculatedMarginArr[] = $requestedMarginNew;

                                            foreach($articleDetails["margins"] as $margins){
                                                $responseMarging = [];
                                                if ($margins['seller_type'] == $requestedSellerType){
                                                    $responseMarging["seller_type"]  = $requestedSellerType;
                                                    $responseMarging["case_size"]  = $caseSize;
                                                    $responseMarging["reason"]  = "Success";
                                                    array_push($responseMargings, $responseMarging);

                                                }
                                            }
                                        }
                                    }
                                    #margin details from input json
                                    $sellerTypesMarginArr = [];
                                    if(!empty($calculatedMarginArr)){
                                        foreach($calculatedMarginArr as $calMargin){
                                            $rstype = $calMargin['seller_type'];
                                            if($rstype == 'Superstockiest') $rstype = 'SS';
                                            else if($rstype == 'SSstockiest') $rstype = 'SSS';
                                            if($rstype != "MRP") {
                                                $marginColumn = "Margin_".$rstype;
                                                $offerPriceColumn = "Offer_Price_".$rstype;
                                                $sellerTypesMarginArr[$marginColumn] = $calMargin['margin_value'];
                                                $sellerTypesMarginArr[$offerPriceColumn] = $calMargin['offer_value'];
                                            }
                                        }
                                    }
                                    array_push($updatevalues,$sellerTypesMarginArr['Offer_Price_Retailer'], $sellerTypesMarginArr['Offer_Price_Distributor'], $sellerTypesMarginArr['Offer_Price_SS'], $sellerTypesMarginArr['Offer_Price_SSS'], $sellerTypesMarginArr['Offer_Price_COCO'], $sellerTypesMarginArr['Offer_Price_COFO'], $sellerTypesMarginArr['Offer_Price_FOFO'], $sellerTypesMarginArr['Offer_Price_FOCO'], $rowStatus, date('Y-m-d', strtotime($valid_start_date)), $Creation_Date, $Created_By, $ID);
                                    $Created_By = 1001;
                                    //$status = 'A';
                                    $Creation_Date = date('Y-m-d H:i:s');




                                    if($updateMargin == true)
                                    {
                                        $endDate = '';
                                        if($havingPrePostVersions == true){
                                            # update pre version end date = current version start date - 1 day
                                            if(!empty($articleMarginsPreVersion)){
                                                $this->updateEndDate($this->pdo, $articleMarginsPreVersion['ID'], $valid_start_date);
                                            }
                                        } else if($havingPreVersion == true && !empty($articleMarginsPreVersion)){
                                            # update preversion end date = current version start date - 1 day
                                            $this->updateEndDate($this->pdo, $articleMarginsPreVersion['ID'], $valid_start_date);
                                        } else if($havingPostVersion == true){
                                            # update current version end date = post version start - 1
                                            if(!empty($articleMarginsPostVersion)){
                                                //updateEndDate($this->pdo, $articleMarginsPostVersion['ID'], $valid_start_date);
                                                $post_valid_start_date = $articleMarginsPostVersion['Valid_Start_Date'];
                                                $valid_end_date = date('Y-m-d', strtotime('-1 day', strtotime($post_valid_start_date)));
                                                $endDate = $valid_end_date;
                                            }
                                        } else {
                                            $endDate = $valid_end_date;
                                        }

                                        ## Update current valid end date
                                        if($endDate != ''){
                                            $dtArr = [];
                                            $updateSQLDT = "UPDATE Article_Margins set Valid_End_Date = :validEndDate1 WHERE ID = :id1;";
                                            $updateStmtDT = $this->pdo->prepare($updateSQLDT);
                                            $dtArr[':validEndDate1'] = $endDate;
                                            $dtArr[':id1'] = $ID;
                                            $resDT = $updateStmtDT->execute($dtArr);
                                        }
                                        # Update article Margin
                                        $updateSQL = "UPDATE Article_Margins set List_Price = ?, MRP = ?, ".$updateparams." Offer_Price_Retailer=?, Offer_Price_Distributor=?, Offer_Price_SS=?, Offer_Price_SSS=?, Offer_Price_COCO=?, Offer_Price_COFO=?, Offer_Price_FOFO=?, Offer_Price_FOCO=?, Status=?, Valid_Start_Date = ?, Changed_Date=?, Changed_By=? WHERE ID = ?;";


                                        $articleMarginUpdateStmt1 = $this->pdo->prepare($updateSQL);
                                        $res1 = $articleMarginUpdateStmt1->execute($updatevalues);



                                        /* echo "\n articleMarginUpdateStmt1 = PDO::errorInfo():\n";
                                        print_r($articleMarginUpdateStmt1->errorInfo()); */


                                        $successRows[] = "'".$sk_article_no."(V-$version)':'".$legacy_article_no."'";
                                        // $responseData['reason'] = 'success';
                                        $responseData['margins'] = $responseMargings;

                                        //$data['articles'][] = $responseData;

                                    } else if($insertMargin == true) {

                                        ## get previous version record and update its end date to new start date - 1
                                        if($havingPreVersion == true && !empty($articleMarginsPreVersion)){
                                            # update preversion end date = current version start date - 1 day
                                            $this->updateEndDate($this->pdo, $articleMarginsPreVersion['ID'], $valid_start_date);
                                        }


                                        /*****
                                         * Sheetal Chive
                                         * Change : Added below Conditions
                                         * Reason : To check margin data is filled or empty. if data is empty then put its value as 0.
                                         * Date : 11-03-2022
                                         ******/

                                        $margin_data_retailer  = !empty($sellerTypesMarginArr['Margin_Retailer']) ? $sellerTypesMarginArr['Margin_Retailer'] : 0.00;
                                        $margin_data_Margin_SS = !empty($sellerTypesMarginArr['Margin_SS']) ? $sellerTypesMarginArr['Margin_SS'] : 0.00;
                                        $margin_data_Margin_SSS = !empty($sellerTypesMarginArr['Margin_SSS']) ? $sellerTypesMarginArr['Margin_SSS'] : 0.00;
                                        $margin_data_Margin_Distributor = !empty($sellerTypesMarginArr['Margin_Distributor']) ? $sellerTypesMarginArr['Margin_Distributor'] : 0.00;

                                        $margin_data_Margin_COCO = !empty($sellerTypesMarginArr['Margin_COCO']) ? $sellerTypesMarginArr['Margin_COCO'] : 0.00;
                                        $margin_data_Margin_FOCO= !empty($sellerTypesMarginArr['Margin_FOCO']) ? $sellerTypesMarginArr['Margin_FOCO'] : 0.00;
                                        $margin_data_Margin_FOFO = !empty($sellerTypesMarginArr['Margin_FOFO']) ? $sellerTypesMarginArr['Margin_FOFO'] : 0.00;
                                        $margin_data_Margin_COFO = !empty($sellerTypesMarginArr['Margin_COFO']) ? $sellerTypesMarginArr['Margin_COFO'] : 0.00;

                                        $Offer_price_margin_data_retailer  = !empty($sellerTypesMarginArr['Offer_Price_Retailer']) ? $sellerTypesMarginArr['Offer_Price_Retailer'] : 0.00;
                                        $Offer_price_margin_data_Margin_Distributor = !empty($sellerTypesMarginArr['Offer_Price_Distributor']) ? $sellerTypesMarginArr['Offer_Price_Distributor'] : 0.00;
                                        $Offer_price_margin_data_Margin_SS = !empty($sellerTypesMarginArr['Offer_Price_SS']) ? $sellerTypesMarginArr['Offer_Price_SS'] : 0.00;
                                        $Offer_price_margin_data_Margin_SSS = !empty($sellerTypesMarginArr['Offer_Price_SSS']) ? $sellerTypesMarginArr['Offer_Price_SSS'] : 0.00;

                                        $Offer_price_margin_data_COCO  = !empty($sellerTypesMarginArr['Offer_Price_COCO']) ? $sellerTypesMarginArr['Offer_Price_COCO'] : 0.00;
                                        $Offer_price_margin_data_Margin_FOFO = !empty($sellerTypesMarginArr['Offer_Price_FOFO']) ? $sellerTypesMarginArr['Offer_Price_FOFO'] : 0.00;
                                        $Offer_price_margin_data_Margin_FOCO = !empty($sellerTypesMarginArr['Offer_Price_FOCO']) ? $sellerTypesMarginArr['Offer_Price_FOCO'] : 0.00;
                                        $Offer_price_margin_data_Margin_COFO = !empty($sellerTypesMarginArr['Offer_Price_COFO']) ? $sellerTypesMarginArr['Offer_Price_COFO'] : 0.00;



                                        $insertSQL = "INSERT INTO Article_Margins (ID, Article_No, MRP, List_Price, Valid_Start_Date, Valid_End_Date, Status, Creation_Date, Created_By, Filter_Type, Filter_ID, Margin_Retailer, Margin_SS, Margin_SSS, Margin_Distributor, Margin_COCO, Margin_FOCO, Margin_FOFO, Margin_COFO, Offer_Price_Retailer, Offer_Price_Distributor, Offer_Price_SS, Offer_Price_SSS, Offer_Price_COCO, Offer_Price_FOFO, Offer_Price_FOCO, Offer_Price_COFO, Version) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
                                        ### Insert article margin


                                        $articleMarginInsertStmt = $this->pdo->prepare($insertSQL);

                                        // $articleMarginInsertStmt->execute([null, $sk_article_no, $MRP, $list_price, $valid_start_date, $valid_end_date, $rowStatus, $Creation_Date, $Created_By, $merchant_type, $filterId, $sellerTypesMarginArr['Margin_Retailer'], $sellerTypesMarginArr['Margin_SS'], $sellerTypesMarginArr['Margin_SSS'], $sellerTypesMarginArr['Margin_Distributor'], $sellerTypesMarginArr['Margin_COCO'], $sellerTypesMarginArr['Margin_FOCO'], $sellerTypesMarginArr['Margin_FOFO'], $sellerTypesMarginArr['Margin_COFO'], $sellerTypesMarginArr['Offer_Price_Retailer'], $sellerTypesMarginArr['Offer_Price_Distributor'], $sellerTypesMarginArr['Offer_Price_SS'], $sellerTypesMarginArr['Offer_Price_SSS'], $sellerTypesMarginArr['Offer_Price_COCO'], $sellerTypesMarginArr['Offer_Price_FOFO'], $sellerTypesMarginArr['Offer_Price_FOCO'], $sellerTypesMarginArr['Offer_Price_COFO'], $version]);
                                        $articleMarginInsertStmt->execute([null, $sk_article_no, $MRP, $list_price, $valid_start_date, $valid_end_date, $rowStatus, $Creation_Date, $Created_By, $merchant_type, $filterId, $margin_data_retailer, $margin_data_Margin_SS, $margin_data_Margin_SSS, $margin_data_Margin_Distributor, $margin_data_Margin_COCO, $margin_data_Margin_FOCO, $margin_data_Margin_FOFO, $margin_data_Margin_COFO, $Offer_price_margin_data_retailer, $Offer_price_margin_data_Margin_Distributor, $Offer_price_margin_data_Margin_SS, $Offer_price_margin_data_Margin_SSS, $Offer_price_margin_data_COCO, $Offer_price_margin_data_Margin_FOFO, $Offer_price_margin_data_Margin_FOCO, $Offer_price_margin_data_Margin_COFO, $version]);

                                        // $articleMarginInsertStmt->debugDumpParams();


                                        $successRows[] = "'".$sk_article_no."(V-$version)':'".$legacy_article_no."'";
                                        $responseData['margins'] = $responseMargings;
                                    }

                                    // multiple case size articles price
                                    ## Call Article Price Update procedure if status true
                                    # and todays date fall in valid from to date range
                                    // if($status == true){

                                    #$valid_start_date, $valid_end_date
                                    $start = strtotime($valid_start_date);
                                    $end = strtotime($valid_end_date);
                                    $check = strtotime(date('Y-m-d'));

                                    ## Check that user date is between start & end
                                    $valid_date_range = (($start <= $check ) && ($check <= $end));
                                    if($valid_date_range == true){
                                        # If valid start date is today then only call procedure
                                        $callProcdr = $this->pdo->prepare("CALL Article_Price_Update('$sk_article_no', '$business_id')");
                                        $callProcdr->execute();
                                        // # send notification to all cust of the business
                                        send_api_notification($this->pdo,'upsert_article_margin_pricing');
                                    }
                                    // }
                                }
                                $status = true;
                                // $responseData['article_no'] = $sk_article_no;
                                $responseData['legacy_article_no'] = $legacy_article_no;
                                // $responseData['case_size'] = $caseSize;
                                $responseData['version'] = $version;
                                $data['articles'][] = $responseData;
                            }
                        }
                        else
                        {
                            $failedRows[] = "'".$sk_article_no."':'".$legacy_article_no."'";
                            $responseData['reason'] = 'Article number not found...';
                            $responseData['article_no'] = $article_no ;
                            $responseData['legacy_article_no'] = $legacy_article_no;
                            $responseData['version'] = $version;
                            $responseData['case_size'] = $caseSize;
                            //$responseData['margins'] = [];
                            $data['articles'][] = $responseData;

                            // $jsondata = json_encode($data['articles']);
                            // $restxt = json_encode($data);
                            // fwrite($myfile, PHP_EOL);
                            // fwrite($myfile, date("d-m-Y h:i:sa"));
                            // fwrite($myfile, "   Response Data : ");
                            // fwrite($myfile, $restxt);
                            continue;
                            //$status = false;
                        }

                    } else {
                        $status = false;
                        $responseData['article_no'] = $articleDetails['article_no'];
                        $responseData['legacy_article_no'] = $articleDetails['legacy_article_no'];
                        $responseData['case_size'] = $caseSize;
                        $responseData['version'] = $version;
                        if(!empty($marginsErrors)){
                            $marginError = 'Margins has some error';
                        }
                        // print_r($articleValidate);die;
                        if(!empty($marginsErrors))
                        {
                            $responseData['margins'] = array_values($marginsErrors);

                        }else if($listpriceValidate > $mrpValidate){
                            $responseData['reason'] = 'least price should be greater than MRP';
                            $responseData['Article no :'] = $articleValidate;
                        }

                        else
                        {
                            $responseData['reason'] = $errors;
                        }

                        $data['articles'][] = $responseData;
                        $failedRows[] = "'".$sk_article_no."(V-$version)':'".$articleDetails['legacy_article_no']."'";
                        // $jsondata = json_encode($data['articles']);
                        // $restxt = json_encode($data);
                        // fwrite($myfile, PHP_EOL);
                        // fwrite($myfile, date("d-m-Y h:i:sa"));
                        // fwrite($myfile, "   Response Data : ");
                        // fwrite($myfile, $restxt);
                        continue;
                    }

                    ## Call Article Price Update procedure if status true
                    # and todays date fall in valid from to date range
                    if($status == true){

                        #$valid_start_date, $valid_end_date
                        $start = strtotime($valid_start_date);
                        $end = strtotime($valid_end_date);
                        $check = strtotime(date('Y-m-d'));

                        ## Check that user date is between start & end
                        $valid_date_range = (($start <= $check ) && ($check <= $end));
                        if($valid_date_range == true){
                            # If valid start date is today then only call procedure
                            // $callProcdr = $this->pdo->prepare("CALL Article_Price_Update('$sk_article_no', '$business_id')");
                            // $callProcdr->execute();
                            // # send notification to all cust of the business
                            send_api_notification($this->pdo,'upsert_article_margin_pricing');
                        }
                    }

                    // }// end foreach $articles
                }
                $allErrors = [];
                if(!empty($errors) || !empty($marginsErrors)){
                    ///msg .= 'Invalid data.';
                }

                if(!empty($successRows)){
                    $msg[] = 'Margins data updated successfully for following article numbers '.implode(", ",array_unique($successRows))." ";
                    $status = true;
                } else {
                    $status = false;
                }
                if(!empty($failedRows)) {
                    $msg[] = ' Failed: Margins data not updated for following articles numbers '.implode(", ",$failedRows)." ";
                }
                $msg = implode(",", $msg);
                //return get_response($msg, $status, $data);

            }
            fwrite($myfile, PHP_EOL);
            fwrite($myfile, date("d-m-Y h:i:sa"));
            fwrite($myfile, "   upsert_article_margin_pricing   Response Data : ");
            $restxt = json_encode($response);
            fwrite($myfile, $restxt);
            $restxt = json_encode($msg);
            fwrite($myfile, $restxt);
            $restxt = json_encode($status);
            fwrite($myfile, $restxt);
            $restxt = json_encode($data);
            fwrite($myfile, $restxt);
            fclose($myfile);
            return get_response($msg, $status, $data);
        }
        catch (PDOException $e)
        {
            $status = false;
            $data = $e->getMessage();
            fwrite($myfile, PHP_EOL);
            fwrite($myfile, date("d-m-Y h:i:sa"));
            fwrite($myfile, "   upsert_article_margin_pricing   Response Data : ");
            $restxt = json_encode($data);
            fwrite($myfile, $restxt);
            $restxt = json_encode($response);
            fwrite($myfile, $restxt);
            $restxt = json_encode("Something went wrong");
            fwrite($myfile, $restxt);
            $restxt = json_encode($status);
            fwrite($myfile, $restxt);
            fclose($myfile);
            return get_response("Something went wrong", $status);
        }
    }

    private function upsertArticleData($pdo, $action, $articleDetails, $caseFromDB, $articleIndex){
        $successRecords = [];
        $responseCaseSizes = [];
        $responseMoreImages = [];
        $responseArtLang = '';

        $error = [];
        $article_no         = $articleDetails['article_no'] ?? null;
        $Display_Seq_No         = $articleDetails['display_Seq_No'] ;
        $Display_Seq_Text         = $articleDetails['display_Seq_Text'];
        $Display_Seq_Text = str_replace(" ","",$Display_Seq_Text);
        if(is_numeric($Display_Seq_Text))
        {
            $Display_Seq_Text =  str_pad($Display_Seq_Text,5,"0",STR_PAD_LEFT);
        }
        $legacy_article_no  = $articleDetails['legacy_article_no'] ?? null;
        $article_desc       = $articleDetails['article_desc'] ?? null;
        $article_short_desc = $articleDetails['article_short_desc'] ?? null;
        $gst                = $articleDetails['gst'] ?? null;
        $brand_id           = $articleDetails['brand_id'] ?? null;
        $volume             = $articleDetails['volume'] ?? null;
        $unit               = $articleDetails['unit'] ?? null;
        $EAN                = $articleDetails['EAN'] ?? null;
        $article_status     = $articleDetails['status'] ?? 'A';
        $dietary_code       = $articleDetails['dietary_code'] ?? null;
        $approx_weight      = $articleDetails['approx_weight'] ?? null;
        $article_tag        = $articleDetails['article_tag'] ?? null;
        $HSN                = $articleDetails['HSN'] ?? null;
        $schedule           = $articleDetails['schedule'] ?? null;
        $case_sizes         = $articleDetails['case_sizes'] ?? [];
        $tech_spec          = $articleDetails['tech_spec'] ?? null;
        $composition        = $articleDetails['composition'] ?? null;
        $V_Mfg              = $articleDetails['V_Mfg'] ?? null;
        $V_Mfg_Month_Year   = $articleDetails['V_Mfg_Month_Year'] ?? null;
        $V_Model            = $articleDetails['V_Model'] ?? null;
        $oem_part_no        = $articleDetails['oem_part_no'] ?? null;
        $mrp                = $articleDetails['mrp'] ?? null;
        $business_id        = $articleDetails['business_id'];

        $caseSizesDetails = $case_sizes;
        // print_r($caseSizesDetails);

        $createdBy = CREATED_BY_TMP_USER;
        $all_case_sizes = implode(',',array_keys($caseFromDB));
        $test_array = array();
        if($action == 'insert'){

            $articleCaseSizes = [];
            # Insert case size wise articles
            try{
                # if new article does not have case size 1 , then insert given article with case size 1 as well
                $arr = array_column($caseSizesDetails, 'case_size');

                $has1case = false;
                if(in_array(1, array_column($caseSizesDetails, 'case_size'))){
                    $has1case = true;
                } else if (in_array(1, array_values($caseFromDB))){
                    $has1case = true;
                }

                if($has1case == false){
                    $case1Arr[0]['case_size'] = 1;
                    $case1Arr[0]['status'] = 'A';
                    $caseSizesDetails = array_merge($case1Arr, $caseSizesDetails);
                }

                if(!empty($caseSizesDetails)){

                    foreach($caseSizesDetails as $i => $caseSizesDetail){
                        $responseMI = '';
                        $mysqlErrors = [];
                        $insert_sql = "INSERT INTO Article_Master (Display_Seq_No,Display_Seq_Text,OEM_Part_No, Article_Desc, Article_Short_Desc, GST, Brand_ID, Valume, Unit, EAN, Status, Dietary_Code, Approx_Weight, Article_Tag, HSN, Schedule, Case_Size, Tech_Spec, Composition, V_Mfg, V_Mfg_Month_Year, V_Model, Creation_Date, Creation_Time, Created_By, Change_Date, Change_Time, Changed_By, Changed_Date) values ('$Display_Seq_No','$Display_Seq_Text','$oem_part_no', '$article_desc', '$article_short_desc', '$gst', '$brand_id', '$volume', '$unit', '$EAN', '".$caseSizesDetail['status']."', '$dietary_code', '$approx_weight', '$article_tag', '$HSN', '$schedule', '".$caseSizesDetail["case_size"]."', '$tech_spec', '$composition', '$V_Mfg', '$V_Mfg_Month_Year', '$V_Model', CURRENT_DATE(), CURRENT_TIME, '$createdBy', CURRENT_DATE(), CURRENT_TIME, '$createdBy', NOW())";
                        $article_insert = $pdo->query($insert_sql);
                        // $article_insert->execute();

                        $article_no =  $pdo->lastInsertId();

                        $last_insert_query = "Select * From Article_Master where `Article_No` = '$article_no'";

                        $result = $pdo->query($last_insert_query);
                        $data = $result->fetch(PDO::FETCH_ASSOC);

                        $last_display_seq_text = $data['Display_Seq_Text'];
                        $last_display_seq_no= $data['Display_Seq_No'];


                        $responseCaseSizes['case_size'] = (int)$caseSizesDetail["case_size"];
                        $successRecords[$legacy_article_no][$article_no] = $caseSizesDetail["case_size"];
                        if(empty($all_case_sizes)){
                            array_push($test_array,$article_no);
                        }else{
                            array_push($all_case_sizes,$article_no);
                        }

                    }


                    $pdo->query( "set @rank:='$last_display_seq_no';");
                    if(empty($all_case_sizes)){
                        $test_array = implode(',',$test_array);
                        $update_sql = "update Article_Master set Display_Seq_No = LEAST(Display_Seq_No + 1, 9999)  where `Display_Seq_No` >= '$last_display_seq_no' and `Display_Seq_Text` = '$last_display_seq_text' and concat(Display_Seq_Text, Display_Seq_No)  != 'A9999' and `Article_No` NOT IN ( $test_array);";
                    }
                    $article_insertd = $pdo->prepare($update_sql);
                    $article_insertd->execute();
                    # upload image to all case sizes and pdf
                    if(!empty($articleCaseSizes)){
                        $responseMI = $this->insertCaseSizeWiseFiles($pdo, $articleCaseSizes, $_FILES, $articleDetails, $articleIndex);
                    }
                }

                $responseNewArr['successRecords'] = $successRecords;
                $responseNewArr['responseCS'] = $responseCS;
                $responseNewArr['responseMI'] = $responseMI;
                $responseNewArr['responseAL'] = $responseAL;
            } catch ( PDOException $e) {
                $responseNewArr['processed'] = false;
                $responseNewArr['reason'] = $e->getMessage();
            }

            return $responseNewArr;

        } else {

            $articleCaseSizes = [];

            # update article margin and other tables
            if(!empty($caseSizesDetails)){

                // print_r($caseSizesDetails);die;
                foreach($caseSizesDetails as $i => $caseSizesDetail){
                    $responseCaseSizes['case_size'] = (int)$caseSizesDetail["case_size"];

                    # if request case size matches with datbase case size, then get article an update
                    $article_no = array_search($caseSizesDetail['case_size'], $caseFromDB);
                    // if(){
                    //     $caseSizesDetail['case_size']
                    // }
                    $articleCaseSizes[$article_no] = $responseCaseSizes['case_size']; # need to image upload for all case sizes
                    if($article_no > 0){

                        # update
                        $this->processRelationalTables($pdo, $business_id, $article_no, $_FILES, $articleDetails);

                        $this->upsertArticleCategory($pdo, $business_id, $article_no, $articleDetails);

                        $responseArtLang = $this->insertInArticleLanguage($pdo, $article_no, $articleDetails);

                        $statusUpdate = '';
                        if(isset($caseSizesDetail['status'])){
                            $statusUpdate = "Status = '".$caseSizesDetail['status']."', ";
                        }


                        $update_sql  = "UPDATE Article_Master SET Display_Seq_No = '$Display_Seq_No',Display_Seq_Text = '$Display_Seq_Text', OEM_Part_No = '$oem_part_no', Article_Desc = '$article_desc', Article_Short_Desc = '$article_short_desc', GST = '$gst', Brand_ID = '$brand_id', Valume = '$volume', Unit = '$unit', EAN = '$EAN', $statusUpdate Dietary_Code = '$dietary_code', Approx_Weight = '$approx_weight', Article_Tag = '$article_tag', HSN = '$HSN', Schedule = '$schedule', Case_Size = '".$caseSizesDetail['case_size']."', Tech_Spec = '$tech_spec', Composition = '$composition', V_Mfg = '$V_Mfg', V_Mfg_Month_Year = '$V_Mfg_Month_Year', V_Model = '$V_Model', Change_Date = CURRENT_DATE(), Change_Time = CURRENT_TIME(), Changed_By = '$createdBy', Changed_Date = now() WHERE Article_No = '$article_no'";//MRP = '$mrp',

                        $article_update = $pdo->prepare($update_sql);
                        $query_response = $article_update->execute();
                        // $article_update->debugDumpParams();
                        $last_insert_query = "Select * From Article_Master where `Article_No` = '$article_no'";

                        $result = $pdo->query($last_insert_query);
                        $data = $result->fetch(PDO::FETCH_ASSOC);

                        $last_display_seq_text = $data['Display_Seq_Text'];
                        $last_display_seq_no= $data['Display_Seq_No'];

                        $successRecords[$legacy_article_no][$article_no] = $caseSizesDetail["case_size"];

                        if($article_no > 0){

                            $articleCaseSizes[$article_no] = $responseCaseSizes['case_size']; # need to image upload for all case sizes

                            # Insert new mapping into Article_Supplier_Product_Lookup table
                            $insert_sql2 = "INSERT INTO `Article_Supplier_Product_Lookup` (`Article_No`, `Supplier_No`, `Supplier_Article_No`, `SS_Article_No`, `Creation_Date`, `Created_By`, `Changed_Date`, `Changed_By`) VALUES ('".$article_no."', '0', '".$legacy_article_no."', '', now(), '".$createdBy."', now(), '".$createdBy."');";
                            $lookup_insert = $pdo->prepare($insert_sql2);
                            $lookup_insert->execute();

                            $this->processRelationalTables($pdo, $business_id, $article_no, $_FILES, $articleDetails);
                            $this->upsertArticleCategory($pdo, $business_id, $article_no, $articleDetails);

                            $responseArtLang = $this->insertInArticleLanguage($pdo, $article_no, $articleDetails);
                            //$responseMoreImages = uploadArticleMoreImages($pdo, $article_no, $_FILES, $articleDetails);

                            if(isset($article_no) && isset($legacy_article_no))
                            {
                                $Margin_Data_array = array(
                                    "articles"=>

                                        [[
                                            "legacy_article_no" => $legacy_article_no,
                                            "article_no"        => $article_no,
                                            "merchant_type"    =>  'Business',
                                            "business_id"      => 2 ,
                                            "MRP"              => 0,
                                            "list_price"       => 0,
                                            "store_no"         => 0,
                                            "margins"          =>[["seller_type" => "",
                                                "margin_value"=> ""]],
                                            "version"          => 1,
                                            "status"           => 'A' ,
                                            "valid_start_date" => date('Y-m-d H:i:s'),
                                        ]],
                                    "page"=> 1,
                                    "total_pages"=> 1
                                );

                                $json_Margin_Data_array= '';
                                $json_Margin_Data_array = json_encode($Margin_Data_array);

                                // $url1 = "http://chitale.loc/b2bservices/sf_services/public/index.php/upsert_article_margin_pricing";
                                $url = UPSERT_ARTICLE_MARGIN_PRICING;

                                $headers = apache_request_headers();

                                $headers_data = array('Content-Type: application/json',
                                    "Authorization: $headers[Authorization]",
                                    'AuthType: '.$headers['AuthType'],
                                    'Content-Length: ' . strlen($json_Margin_Data_array));
                                $ch = curl_init();
                                curl_setopt($ch,CURLOPT_URL,$url);
                                curl_setopt($ch, CURLOPT_POST, true);
                                curl_setopt($ch, CURLOPT_POSTFIELDS, $json_Margin_Data_array);
                                curl_setopt($ch, CURLOPT_HTTPHEADER, $headers_data);
                                $headers = apache_request_headers();
                                curl_setopt($ch, CURLOPT_RETURNTRANSFER,true);
                                curl_setopt($ch, CURLOPT_VERBOSE, true);
                                $outputData = curl_exec($ch);

                                $outputData = str_replace("\xEF\xBB\xBF",'',$outputData);
                            }
                            $responseCaseSizes['processed'] = true;
                            $responseCaseSizes['reason'] = 'Success';
                        } else {
                            $failesRecords[$legacy_article_no][$article_no] = $caseSizesDetail["case_size"];
                            $mysqlErrors = $article_insert->errorInfo();
                            $errors = 'Legacy article no '.$legacy_article_no.' with case size '.$caseSizesDetail["case_size"].' has error '.$mysqlErrors['2'];
                            $responseCaseSizes['processed'] = false;
                            $responseCaseSizes['reason'] = $errors;
                        }

                        $responseCS[] = $responseCaseSizes;
                        //$responseMI = $responseMoreImages;
                        $responseAL = $responseArtLang;
                    }

                    $responseCaseSizes['processed'] = true;
                    $responseCaseSizes['reason'] = 'Success';

                    $responseCS[] = $responseCaseSizes;
                    //$responseMI = $responseMoreImages;
                    $responseAL = $responseArtLang;
                }
                if(isset($articleDetails['display_Seq_No'])){
                    // $pdo->query( "set @rank:='$last_display_seq_no';");
                    $update_sql = "update Article_Master set Display_Seq_No = LEAST(Display_Seq_No + 1, 9999)  where `Display_Seq_No` >= '$last_display_seq_no' and `Display_Seq_Text` = '$last_display_seq_text' and concat(Display_Seq_Text, Display_Seq_No)  != 'A9999' and `Article_No` NOT IN ( $all_case_sizes);";
                    // $result =  $pdo->query($update_sql);
                    $result = $pdo->prepare($update_sql);
                    $res1 = $result->execute();
                    // $number_of_rows = $result->rowCount();
                    //  $result->debugDumpParams();
                }
                # upload image to all case sizes and pdf
                if(!empty($articleCaseSizes)){
                    $responseMI = $this->insertCaseSizeWiseFiles($pdo, $articleCaseSizes, $_FILES, $articleDetails, $articleIndex);
                }

            }
            $responseNewArr['successRecords'] = $successRecords;
            $responseNewArr['responseCS'] = $responseCS;
            $responseNewArr['responseMI'] = $responseMI;
            $responseNewArr['responseAL'] = $responseAL;

            return $responseNewArr;
        }
    }

    private function insertCaseSizeWiseFiles($pdo, $articleCaseSizes, $FILES, $articleDetails, $articleIndex){

        $legacy_article_no = $articleDetails['legacy_article_no'];
        ## get article number with case size 1
        # Validate legacy article no
        if($legacy_article_no > 0){
            $where = 'ASPL.Supplier_Article_No = '.$legacy_article_no.' AND AM.Case_Size = 1';
        }
        $case1ArticleSQL = "SELECT AM.Article_No FROM `Article_Master` AM
    LEFT JOIN Article_Supplier_Product_Lookup ASPL ON ASPL.Article_No = AM.Article_No
    WHERE $where ";
        $queryArticle = $pdo->prepare($case1ArticleSQL);
        $queryArticle->execute();
        /****
         * commented by sheetal
         * Reason : as per discussion by daya this line is same as print_r to sql statement
         * Commented at 2 feb 2022
         */
        // $queryArticle->debugDumpParams();
        $case1Article = $queryArticle->fetch(PDO::FETCH_ASSOC);
        $case1ArticleNo = 0;
        if(!empty($case1Article)){
            $case1ArticleNo = $case1Article['Article_No'];
        }


        $image_urls = imageUploadUpsertArticle($articleCaseSizes, $FILES, $case1ArticleNo, $articleIndex);
        $product_pdfs = pdfUploadUpsertArticle($articleCaseSizes, $FILES, $articleDetails, $case1ArticleNo, $articleIndex);
        $responseMoreImages = uploadArticleMoreImages($pdo, $articleCaseSizes, $_FILES, $articleDetails, $case1ArticleNo, $articleIndex);

        foreach($articleCaseSizes as $ano=>$requestedCaseSizes){

            $updateFiles = [];
            if(isset($image_urls[$ano]) && $image_urls[$ano] != '') {
                $updateFiles[] = "Image_Url = '".$image_urls[$ano]."'";
            }
            if(isset($product_pdfs[$ano]) && $product_pdfs[$ano] != '') {
                $updateFiles[] = "Product_Pdf = '".$product_pdfs[$ano]."'";
            }
            $upFiles = implode(',',$updateFiles);
            if(!empty($updateFiles)){
                //$update_sql  = "UPDATE Article_Master SET ".$upFiles." where Article_No = ".$ano;
                $article_update = $pdo->prepare($update_sql);
                $query_response = $article_update->execute();
                /****
                 * commented by sheetal
                 * Reason : as per discussion by daya this line is same as print_r to sql statement
                 * Commented at 2 feb 2022
                 */
                // $article_update->debugDumpParams();
            }
        }
        return $responseMoreImages;
    }

    private function imageUploadUpsertArticle($articleCaseSizes, $FILES, $case1ArticleNo, $articleIndex)
    {
        ## $articleIndex : is index from main articles array to get current article image
        $imgArr = [];
        # get first article no
        $articleKeys = array_keys($articleCaseSizes);
        $article_no = $articleKeys[0];

        $image_url = null;

        /*
        $allowedImgType = ['image/jpeg', 'image/png', 'image/jpg'];
        if(isset($FILES['articles']['type'][$articleIndex]['image_url'])){
            if (!in_array($FILES['articles']['type'][$articleIndex]['image_url'], $allowedImgType)){
            }
        } */

        /*  Image url process */
        if(isset($FILES['articles']['name']) && !empty($FILES['articles']['name'])){
            $images = $FILES['articles']['name'];
            foreach($images as $j=>$img){
                if($j == $articleIndex){
                    $image = $img['image_url'];
                    if($image != '' && $article_no > 0)
                    {
                        $tmpName = $FILES['articles']['tmp_name'][$j]['image_url'];
                        $size = $FILES['articles']['size'][$j]['image_url'];
                        if($size <= 2000000){
                            if(move_uploaded_file($tmpName,WEB_ADD_ARTICLE.$article_no.'.png')){
                                $imgArr[$article_no] = $article_no.'.png';
                                if(!empty($articleCaseSizes)){
                                    foreach($articleCaseSizes as $ano=>$case_size){
                                        if($case_size != 1){
                                            copy(WEB_ADD_ARTICLE.$article_no.'.png'  , WEB_ADD_ARTICLE.$ano.'.png');
                                            $imgArr[$ano] = $ano.'.png';
                                        }
                                    }
                                }
                            } else {
                                $this->copyCase1Image($articleCaseSizes, $case1ArticleNo);
                            }
                        }
                    } else {
                        $this->copyCase1Image($articleCaseSizes, $case1ArticleNo);
                    }
                }
            }
        }

        return $imgArr;
    }

    private function copyCase1Image($articleCaseSizes, $case1ArticleNo){
        if(file_exists(WEB_ADD_ARTICLE.$case1ArticleNo.'.png')){
            ## If issue in upload then copy case size 1 article image to this new size (having same legacy article number)
            foreach($articleCaseSizes as $ano=>$case_size){
                if($case_size != 1){
                    copy(WEB_ADD_ARTICLE.$case1ArticleNo.'.png'  , WEB_ADD_ARTICLE.$ano.'.png');
                    $imgArr[$ano] = $ano.'.png';
                }
            }
        }
    }

    private function uploadArticleMoreImages($pdo, $articleCaseSizes, $FILES, $article, $case1ArticleNo, $articleIndex){
        $createdBy = CREATED_BY_TMP_USER;

        $imgArr = [];
        # get first article no
        $articleKeys = array_keys($articleCaseSizes);
        $article_no = $articleKeys[0];

        ## Upload more images and insert into Article_More_Images
        if(isset($FILES['articles']['name']) && !empty($FILES['articles']['name'])){
            $images = $FILES['articles']['name'];
            if(!empty($images)){
                foreach($images as $j=>$img){
                    if($j == $articleIndex && isset($img['more_images'])){
                        $image = $img['more_images'];
                        if($image != '' && $article_no > 0)
                        {
                            $responseMI = [];
                            $moreImages = $FILES['articles']['name'][$j]['more_images'];
                            $tmpNames = $FILES['articles']['tmp_name'][$j]['more_images'];
                            if(!empty($tmpNames)){
                                foreach($tmpNames as $i=>$tmpNm){

                                    $size = $FILES['articles']['size'][$j]['more_images'][$i]['legacy_image_url'];
                                    $tmpName = $tmpNm['legacy_image_url'];
                                    $oldName = $moreImages[$i]['legacy_image_url'];
                                    $responseMoreImages['legacy_image_url'] = $oldName;//response array

                                    # size less than equal to 2MB
                                    if($size <= 2000000){
                                        if(move_uploaded_file($tmpName,WEB_ADD_ARTICLE.$article_no.'_'.$i.'.png')){
                                            $moreImagesUrl = GET_ARTICLE.$article_no.'_'.$i.'.png';
                                            $moreImgStatus = $article['more_images'][$i]['status'];

                                            if(!empty($articleCaseSizes)){
                                                foreach($articleCaseSizes as $ano=>$case_size){
                                                    $moreImage = $ano.'_'.$i.'.png';
                                                    if($case_size != 1){
                                                        copy(WEB_ADD_ARTICLE.$article_no.'_'.$i.'.png', WEB_ADD_ARTICLE.$ano.'_'.$i.'.png');
                                                    }

                                                    $selectMoreImg = "SELECT Id, Large_Image_Url FROM `Article_More_Images`
                                                    WHERE Article_No = $ano AND Legacy_File_Name = '".$oldName."'";
                                                    $queryMoreImg = $pdo->prepare($selectMoreImg);
                                                    $queryMoreImg->execute();
                                                    /****
                                                     * commented by sheetal
                                                     * Reason : as per discussion by daya this line is same as print_r to sql statement
                                                     * Commented at 2 feb 2022
                                                     */
                                                    // $queryMoreImg->debugDumpParams();
                                                    $resMoreImg = $queryMoreImg->fetch(PDO::FETCH_ASSOC);
                                                    if(!empty($resMoreImg)){
                                                        if($moreImgStatus == 'D'){
                                                            $deleteSql = "DELETE FROM `Article_More_Images` WHERE Article_No = $ano AND Legacy_File_Name = '".$oldName."'";
                                                            $deleteSql = $pdo->prepare($deleteSql);
                                                            $deleteSql->execute();

                                                            $pFilename = WEB_ADD_ARTICLE.$resMoreImg['Large_Image_Url'];
                                                            if (file_exists($pFilename)) {
                                                                unlink($pFilename);
                                                            }
                                                        }
                                                    } else {
                                                        #insert
                                                        $insert_sql = "INSERT INTO Article_More_Images (Article_No, Large_Image_Url, Legacy_File_Name, `Creation_Date`, `Created_By`, `Changed_Date`, `Changed_By`) values ('$ano', '$moreImage', '$oldName', now(), '".$createdBy."', now(), '".$createdBy."')";
                                                        $article_insert = $pdo->prepare($insert_sql);
                                                        $article_insert->execute();
                                                        /****
                                                         * commented by sheetal
                                                         * Reason : as per discussion by daya this line is same as print_r to sql statement
                                                         * Commented at 2 feb 2022
                                                         */
                                                        // $article_insert->debugDumpParams();
                                                    }
                                                }
                                            }

                                        } else {
                                            $this->copyCaseSize1MoreImages($articleCaseSizes, $case1ArticleNo, $i);
                                        }
                                        $responseMoreImages['processed'] = true;
                                        $responseMoreImages['reason'] = 'Success';
                                    } else {
                                        $responseMoreImages['processed'] = false;
                                        $responseMoreImages['reason'] = 'Image size exceeds 2MB';
                                    }
                                    $responseMI[] = $responseMoreImages;
                                }
                            }

                            return $responseMI;
                        } else {
                            ## If issue in upload then copy case size 1 article image to this new size (having same legacy article number)
                            $i = 0;
                            foreach($articleCaseSizes as $ano=>$case_size){
                                if($case_size != 1){
                                    // print "<br>====".WEB_ADD_ARTICLE.$case1ArticleNo.'_0.png'." , ".WEB_ADD_ARTICLE.$ano.'_'.$i.'.png';
                                    //copy(WEB_ADD_ARTICLE.$case1ArticleNo.'_0.png'  , WEB_ADD_ARTICLE.$ano.'_'.$i.'.png');
                                    $i++;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    private function copyCaseSize1MoreImages($articleCaseSizes, $case1ArticleNo, $i){
        ## If issue in upload then copy case size 1 article image to this new size (having same legacy article number)
        if(file_exists(WEB_ADD_ARTICLE.$case1ArticleNo.'_0.png')){
            foreach($articleCaseSizes as $ano=>$case_size){
                if($case_size != 1){
                    // print "<br>====".WEB_ADD_ARTICLE.$case1ArticleNo.'_0.png'." , ".WEB_ADD_ARTICLE.$ano.'_'.$i.'.png';
                    copy(WEB_ADD_ARTICLE.$case1ArticleNo.'_0.png'  , WEB_ADD_ARTICLE.$ano.'_'.$i.'.png');
                }
            }
        }
    }

    private function pdfUploadUpsertArticle($articleCaseSizes, $FILES, $article, $case1ArticleNo, $articleIndex)
    {
        $pdfArr = [];
        # get first article no
        $articleKeys = array_keys($articleCaseSizes);
        $article_no = $articleKeys[0];

        /*  PDF process */
        if(isset($FILES['articles']['name'][$articleIndex]['product_pdf']))
        {
            $pdf = $FILES['articles']['name'][$articleIndex]['product_pdf'];
            $tmpName = $FILES['articles']['tmp_name'][$articleIndex]['product_pdf'];

            if(move_uploaded_file($tmpName,ADD_ARTICLE_PDF.$article_no.'.pdf')){
                $pdfArr[$article_no] = GET_ARTICLE_PDF.$article_no.'.pdf';
                if(!empty($articleCaseSizes)){
                    foreach($articleCaseSizes as $ano=>$case_size){
                        if($case_size != 1){
                            copy(ADD_ARTICLE_PDF.$article_no.'.pdf'  , ADD_ARTICLE_PDF.$ano.'.pdf');
                            $pdfArr[$ano] = GET_ARTICLE_PDF.$ano.'.pdf';
                        }
                    }
                }
            }
        }
        elseif(isset($article['product_pdf']))
        {
            $product_pdf = trim($article['product_pdf']);
            if(!empty($articleCaseSizes)){
                foreach($articleCaseSizes as $ano=>$case_size){
                    $pdfArr[$ano] = $product_pdf;
                }
            }
        }

        return $pdfArr;
    }

    private function processRelationalTables($pdo, $business_id, $article_no, $FILES, $article)
    {
        $createdBy = CREATED_BY_TMP_USER;
        $V_Mfg              = $article['V_Mfg'] ?? null;
        $V_Mfg_Month_Year   = $article['V_Mfg_Month_Year'] ?? null;
        $V_Model            = $article['V_Model'] ?? null;
        $article_status     = $article['status'] ?? 'A';

        /* Article business process */
        $getBusiness = $pdo->prepare("SELECT * FROM Article_Business WHERE Business_ID='$business_id' AND Article_No='$article_no'");
        $getBusiness->execute();
        /****
         * commented by sheetal
         * Reason : as per discussion by daya this line is same as print_r to sql statement
         * Commented at 2 feb 2022
         */
        // $getBusiness->debugDumpParams();
        $businessCount = $getBusiness->fetchAll(PDO::FETCH_ASSOC);

        if(empty($businessCount))
        {
            $insert_sql = "INSERT INTO Article_Business (Article_No,Business_ID,Status, `Creation_Date`, `Created_By`, `Changed_Date`, `Changed_By`) values ('$article_no', '$business_id', '$article_status', now(), '".$createdBy."', now(), '".$createdBy."')";
            $article_insert = $pdo->prepare($insert_sql);
            $article_insert->execute();
            /****
             * commented by sheetal
             * Reason : as per discussion by daya this line is same as print_r to sql statement
             * Commented at 2 feb 2022
             */
            // $article_insert->debugDumpParams();
        }
        else
        {
            $update_sql = "UPDATE Article_Business SET Business_ID = $business_id, Status = '$article_status', Changed_Date = now(), Changed_By = '".$createdBy."' WHERE Article_No = $article_no";
            $article_business = $pdo->prepare($update_sql);
            $query_response = $article_business->execute();
            /****
             * commented by sheetal
             * Reason : as per discussion by daya this line is same as print_r to sql statement
             * Commented at 2 feb 2022
             */
            // $article_business->debugDumpParams();
        }

        /* Article where use process  */
        if($V_Mfg !='' || $V_Mfg_Month_Year != '' ||$V_Model != ''){
            $getWhereUse = $pdo->prepare("SELECT * FROM Article_Where_Use WHERE Article_No='$article_no'");
            $getWhereUse->execute();
            /****
             * commented by sheetal
             * Reason : as per discussion by daya this line is same as print_r to sql statement
             * Commented at 2 feb 2022
             */
            // $getWhereUse->debugDumpParams();
            $whereUseCount = $getWhereUse->fetchAll(PDO::FETCH_ASSOC);

            if(empty($whereUseCount))
            {
                $insert_sql = "INSERT INTO Article_Where_Use (Article_No,V_Mfg,V_Mfg_Month_Year,V_Model, `Creation_Date`, `Created_By`, `Changed_Date`, `Changed_By`) values ('$article_no', '$V_Mfg', '$V_Mfg_Month_Year', '$V_Model', now(), '".$createdBy."', now(), '".$createdBy."')";
                $article_insert = $pdo->prepare($insert_sql);
                $article_insert->execute();
                /****
                 * commented by sheetal
                 * Reason : as per discussion by daya this line is same as print_r to sql statement
                 * Commented at 2 feb 2022
                 */
                // $article_insert->debugDumpParams();
            }
            else
            {
                $update_sql = "UPDATE Article_Where_Use SET V_Mfg = '$V_Mfg', V_Mfg_Month_Year = '$V_Mfg_Month_Year', V_Model = '$V_Model', Changed_Date = now(), Changed_By = '".$createdBy."' WHERE Article_No = $article_no";
                $article_where_use_update = $pdo->prepare($update_sql);
                $query_response = $article_where_use_update->execute();
                /****
                 * commented by sheetal
                 * Reason : as per discussion by daya this line is same as print_r to sql statement
                 * Commented at 2 feb 2022
                 */
                // $article_where_use_update->debugDumpParams();
            }
        }

    }

    private function upsertArticleCategory($pdo, $business_id, $article_no, $article)
    {
        $createdBy = CREATED_BY_TMP_USER;
        $grandParent = $article['product_type'];
        $parent = $article['product'] ?? '';
        $child = $article['sub_product'] ?? '';
        # if parent (product) is not sent but child (sub_product) is sent then use child as parent
        if(($parent == '' || $parent == null) && $child != ''){
            $parent = $child;
            $child = '';
        }

        $catIDs = $this->searchArticleCategory($pdo, $article, $business_id);
        #Check category mapping exists in Article_Category table
        if(!empty($catIDs)) {

            $hasLeaf = false;
            #check catIDs has any leaf node
            if(in_array('leaf', $catIDs)){
                $hasLeaf = true;

            }

            foreach($catIDs as $catID=>$node_type){
                $get_mapping  = $pdo->prepare("SELECT * FROM  Article_Category WHERE Business_ID='$business_id' AND Article_No='$article_no' AND Category_ID = '$catID' ");
                $get_mapping->execute();
                /****
                 * commented by sheetal
                 * Reason : as per discussion by daya this line is same as print_r to sql statement
                 * Commented at 2 feb 2022
                 */
                // $get_mapping->debugDumpParams();
                $get_mapping = $get_mapping->fetch(PDO::FETCH_ASSOC);
                # if not mapped the insert into article_category
                if(empty($get_mapping)){
                    $this->map_article_to_category($pdo, $article_no, $business_id, $catID, $node_type, $hasLeaf, $createdBy);
                }
            }
        }
    }

    private function map_article_to_category($pdo, $article_no, $business_id, $catID, $node_type, $hasLeaf, $createdBy){

        # if catid has any leaf then assign article to all leaf catids and ignore branch cat,
        # if it does not have any leaf cat then assgin article to branch catid

        if( ($hasLeaf == true && $node_type == 'leaf') || ($hasLeaf == false && $node_type == 'branch')){
            $insert_cat_sql = "INSERT INTO Article_Category (Article_No, Business_ID, Category_ID, Status, `Creation_Date`, `Created_By`, `Changed_Date`, `Changed_By`) values ('$article_no', '$business_id', '$catID', 'A', now(), '".$createdBy."', now(), '".$createdBy."')";
            $article_cat_insert = $pdo->prepare($insert_cat_sql);
            $article_cat_insert->execute();
            /****
             * commented by sheetal
             * Reason : as per discussion by daya this line is same as print_r to sql statement
             * Commented at 2 feb 2022
             */
            // $article_cat_insert->debugDumpParams();
        }
    }

    private function insert_categories($pdo, $catId, $catDesc, $business_id){

        $insert_category_sql = "
    CALL Add_Child_To_Leaf_Node('".$catId."', '".$catDesc."', '' ,'".CREATED_BY_TMP_USER."', '".$business_id."', @LASTCATID );
    ";
        $insert_category = $pdo->prepare($insert_category_sql);
        $insert_category->execute();
        /****
         * commented by sheetal
         * Reason : as per discussion by daya this line is same as print_r to sql statement
         * Commented at 2 feb 2022
         */
        // $insert_category->debugDumpParams();
        $out = $insert_category->fetch(PDO::FETCH_ASSOC);

        # get out parameter from procedure (last inserted id)
        $sql2 = $pdo->prepare("select @LASTCATID as lastInsertedId");
        $sql2->execute();
        $out2 = $sql2->fetch(PDO::FETCH_ASSOC);

        if(!empty($out2)){
            return $out2['lastInsertedId'];
        } else return 0;

    }

    private function searchArticleCategory($pdo, $article, $business_id)
    {
        $catID = [];
        $root = $article['brand_shop_name'];
        $grandParent = $article['product_type'];
        $parent = $article['product'] ?? '';
        $child = $article['sub_product'] ?? '';
        # if parent (product) is not sent but child (sub_product) is sent then use child as parent
        if(($parent == '' || $parent == null) && $child != ''){
            $parent = $child;
            $child = '';
        }

        $missing = '';

        if($grandParent != '' && $parent != '' && $child != ''){
            $catID = $this->getCatIdWith3LevelsCatName($pdo, $business_id, $root, $grandParent, $parent, $child);

            if(empty($catID)){
                # if we got result with grandparent and parent then child is missing
                $catID = $this->getCatIdWith2LevelsCatName($pdo, $business_id, $root, $grandParent, $parent);

                if(empty($catID)){
                    # if we didn't got result with grandparent then parent is missing
                    $missing = 'parent';

                    $catID = $this->getCatIdWith1LevelCatName($pdo, $business_id, $root, $grandParent);
                    if(empty($catID)){
                        # if didn't got result then grandparent is missing
                        $missing = 'grandparent';
                    } else {
                        # if we got result with grandparent then parent is missing
                        $missing = 'parent';
                    }
                } else {
                    $missing = 'child';
                }
            }
        }


        if(empty($catID) && $grandParent != '' && $parent != '' && $child == ''){
            $catID = $this->getCatIdWith2LevelsCatName($pdo, $business_id, $root, $grandParent, $parent);

            if(empty($catID)){
                # if we didn't got result with grandparent then parent is missing
                $missing = 'parent';
                $catID = $this->getCatIdWith1LevelCatName($pdo, $business_id, $root, $grandParent);

                if(empty($catID)){
                    # if we didn't got result with grandparent then grandparent is missing
                    $missing = 'grandparent';
                } else {
                    # if we got result with grandparent then parent is missing
                    $missing = 'parent';
                }
            }
        }

        if(empty($catID) && $grandParent != '' && $parent == '' && $child == ''){
            $catID = $this->getCatIdWith1LevelCatName($pdo, $business_id, $root, $grandParent);

            if(empty($catID)){
                # if we didn't got result with grandparent then grandparent is missing
                # means every category is missing
                $missing = 'grandparent';
            }
        }

        $grandParentID = 0;
        $parentID = 0;
        $childID = 0;
        $newCatArr = [];

        /* print "missing=====".$missing;
        print_r($catID); exit; */

        # If all 3 levels categories are missing from database then insert new categories level wise
        if($missing != ''){

            if($missing == 'grandparent'){
                $rootCat = $this->getBusinessRootCategory($pdo, $business_id);
                $grandParentID = $this->insert_categories($pdo, $rootCat, $grandParent, $business_id);
                // print_r($grandParentID);
                if($parent != ''){
                    $parentID = $this->insert_categories($pdo, $grandParentID, $parent, $business_id);

                    // print_r($parentID);
                    if($parentID > 0) $newCatId[$parentID] = 'branch';
                } else {
                    if($grandParentID > 0) $newCatId[$grandParentID] = 'branch';
                }
                # if child /sub_product is not sent in request
                if($child != ''){
                    $childID = $this->insert_categories($pdo, $parentID, $child, $business_id);
                    if($childID > 0) $newCatId[$childID] = 'leaf';
                } else {
                    if($parentID > 0) $newCatId[$parentID] = 'branch';
                }
            } else if($missing == 'parent'){
                if(!empty($catID)){
                    # insert required levels category
                    foreach($catID as $grandParentID=>$node_type){
                        $parentID = $this->insert_categories($pdo, $grandParentID, $parent, $business_id);
                        # if child /sub_product is not sent in request
                        if($child != ''){
                            $childID = $this->insert_categories($pdo, $parentID, $child, $business_id);
                            if($childID > 0) $newCatId[$childID] = 'branch';
                        } else {
                            if($parentID > 0) $newCatId[$parentID] = 'leaf';
                        }
                    }
                }
            } else if($missing == 'child'){
                if(!empty($catID)){
                    # insert required levels category
                    foreach($catID as $parent_id=>$node_type){
                        $childID = $this->insert_categories($pdo, $parent_id, $child, $business_id);
                        if($childID > 0) $newCatId[$childID] = 'leaf';
                    }
                }

            }

            if(!empty($newCatId)){
                $newCatArr = $newCatId;
            }
            return $newCatArr; //inserted categories array
        }

        return $catID; // existing categories array
    }

    function getBusinessRootCategory($pdo, $business_id){
        $sql  = $pdo->prepare("SELECT Category_ID FROM  Nested_Category WHERE Business_ID='$business_id' limit 1");
        $sql->execute();
        /****
         * commented by sheetal
         * Reason : as per discussion by daya this line is same as print_r to sql statement
         * Commented at 2 feb 2022
         */
        // $sql->debugDumpParams();
        $result = $sql->fetch(PDO::FETCH_ASSOC);
        if(!empty($result)){
            return $result['Category_ID'];
        } else return 0;
    }

    private function getCatIdWith1LevelCatName($pdo, $business_id, $root, $cat1Name){

        $cat_type = $this->getBrandStoreNameType($pdo, $root);

        if($cat_type == 'ROOT'){
            #Brand Store Name = ROOT
            $brandStoreSql = " grandParent.lft >= ROOT.lft AND grandParent.rgt <= ROOT.rgt AND  ROOT.Cat_Desc = '".$root."' AND ROOT.Type IN ('ROOT') AND ROOT.depth IN (0)";
        } else {
            # General Category or Brand Store name != ROOT
            $brandStoreSql = "( (grandParent.lft >= ROOT.lft AND grandParent.rgt <= ROOT.rgt AND  ROOT.Type <> 'Brand_Store' AND ROOT.depth = 1 )
        OR  (grandParent.lft >= ROOT.lft AND grandParent.rgt <= ROOT.rgt AND  ROOT.Cat_Desc = '".$root."' AND ROOT.Type IN ('Brand_Store') AND ROOT.depth IN (1) )
       )";
        }

        $sql = "SELECT
    IF ( grandParent.lft + 1 = grandParent.rgt,  'leaf', 'branch') as node_type,
    grandParent.*
        FROM
            (SELECT
                        node.Category_ID,
                        node.Cat_Desc,
                        node.Type,
                        node.lft,
                        node.rgt,
                        (COUNT(parent.Cat_Desc) - 1) AS depth
                    FROM
                        Nested_Category AS node
                        INNER JOIN Nested_Category AS parent ON node.lft BETWEEN  parent.lft AND parent.rgt AND parent.Business_ID = ".$business_id."
                    WHERE
                        node.Business_ID = ".$business_id." AND node.Status = 'A'
                    GROUP BY Category_ID
                    ORDER BY node.lft
            )  AS ROOT ,

            (SELECT
                        node.Category_ID,
                        node.Cat_Desc,
                        node.Type,
                        node.lft,
                        node.rgt,
                        (COUNT(parent.Cat_Desc) - 1) AS depth
                    FROM
                        Nested_Category AS node
                        INNER JOIN Nested_Category AS parent ON node.lft BETWEEN parent.lft AND parent.rgt AND parent.Business_ID = ".$business_id."
                    WHERE
                        node.Business_ID = ".$business_id." AND node.Status = 'A'
                    GROUP BY Category_ID
                    ORDER BY node.lft
            ) AS grandParent

        WHERE
            $brandStoreSql

            AND grandParent.Cat_Desc = '".$cat1Name."'
            ORDER BY node_type Desc ;";

        $getCategory = $pdo->prepare($sql);
        $getCategory->execute();
        /****
         * commented by sheetal
         * Reason : as per discussion by daya this line is same as print_r to sql statement
         * Commented at 2 feb 2022
         */
        //$getCategory->debugDumpParams();
        $categories = $getCategory->fetchAll(PDO::FETCH_ASSOC);
        $catArr = array_column($categories, 'node_type', 'Category_ID');
        return $catArr;
    }

    private function getCatIdWith2LevelsCatName($pdo, $business_id, $root, $cat1Name, $cat2Name){

        $cat_type = $this->getBrandStoreNameType($pdo, $root);

        if($cat_type == 'ROOT'){
            #Brand Store Name = ROOT
            $brandStoreSql = " grandParent.lft >= ROOT.lft AND grandParent.rgt <= ROOT.rgt AND  ROOT.Cat_Desc = '".$root."' AND ROOT.Type IN ('ROOT') AND ROOT.depth IN (0)";
        } else {
            # General Category or Brand Store name != ROOT
            $brandStoreSql = "( (grandParent.lft >= ROOT.lft AND grandParent.rgt <= ROOT.rgt AND  ROOT.Type <> 'Brand_Store' AND ROOT.depth = 1 )
        OR  (grandParent.lft >= ROOT.lft AND grandParent.rgt <= ROOT.rgt AND  ROOT.Cat_Desc = '".$root."' AND ROOT.Type IN ('Brand_Store') AND ROOT.depth IN (1) )
       )";
        }

        $sql = "SELECT
    IF ( Parent.lft + 1 = Parent.rgt,  'leaf', 'branch') as node_type,
    Parent.*
        FROM
            (SELECT
                        node.Category_ID,
                        node.Cat_Desc,
                        node.Type,
                        node.lft,
                        node.rgt,
                        (COUNT(parent.Cat_Desc) - 1) AS depth
                    FROM
                        Nested_Category AS node
                        INNER JOIN Nested_Category AS parent ON node.lft BETWEEN  parent.lft AND parent.rgt AND parent.Business_ID = ".$business_id."
                    WHERE
                        node.Business_ID = ".$business_id." AND node.Status = 'A'
                    GROUP BY Category_ID
                    ORDER BY node.lft
            )  AS ROOT ,
            Nested_Category AS Parent,

            (SELECT
                        node.Category_ID,
                        node.Cat_Desc,
                        node.Type,
                        node.lft,
                        node.rgt,
                        (COUNT(parent.Cat_Desc) - 1) AS depth
                    FROM
                        Nested_Category AS node
                        INNER JOIN Nested_Category AS parent ON node.lft BETWEEN parent.lft AND parent.rgt AND parent.Business_ID = ".$business_id."
                    WHERE
                        node.Business_ID = ".$business_id." AND node.Status = 'A'
                    GROUP BY Category_ID
                    ORDER BY node.lft
            ) AS grandParent

        WHERE
            $brandStoreSql

            AND (Parent.lft > grandParent.lft AND Parent.rgt < grandParent.rgt AND grandParent.Cat_Desc = '".$cat1Name."' AND grandParent.Type = 'Category'   AND grandParent.depth > 0)
            AND Parent.Cat_Desc = '".$cat2Name."'
            AND Parent.Status = 'A'
            ORDER BY node_type Desc ;";

        $getCategory = $pdo->prepare($sql);
        $getCategory->execute();
        /****
         * commented by sheetal
         * Reason : as per discussion by daya this line is same as print_r to sql statement
         * Commented at 2 feb 2022
         */
        //$getCategory->debugDumpParams();

        $categories = $getCategory->fetchAll(PDO::FETCH_ASSOC);
        $catArr = array_column($categories, 'node_type', 'Category_ID');
        return $catArr;
    }

    private function getBrandStoreNameType($pdo, $root){
        $sql = "Select Type from Nested_Category where Cat_Desc = '".$root."'";
        $getCatType = $pdo->prepare($sql);
        $getCatType->execute();
        /****
         * commented by sheetal
         * Reason : as per discussion by daya this line is same as print_r to sql statement
         * Commented at 2 feb 2022
         */
        // $getCatType->debugDumpParams();
        $type = $getCatType->fetch(PDO::FETCH_ASSOC);
        if(!empty($type)){
            return $type['Type'];
        } else return '';
    }

    private function getCatIdWith3LevelsCatName($pdo, $business_id, $root, $cat1Name, $cat2Name, $cat3Name){

        $cat_type =  $this->getBrandStoreNameType($pdo, $root);

        if($cat_type == 'ROOT'){
            #Brand Store Name = ROOT
            $brandStoreSql = " grandParent.lft >= ROOT.lft AND grandParent.rgt <= ROOT.rgt AND  ROOT.Cat_Desc = '".$root."' AND ROOT.Type IN ('ROOT') AND ROOT.depth IN (0)";
        } else {
            # General Category or Brand Store name != ROOT
            $brandStoreSql = "( (grandParent.lft >= ROOT.lft AND grandParent.rgt <= ROOT.rgt AND  ROOT.Type <> 'Brand_Store' AND ROOT.depth = 1 )
        OR  (grandParent.lft >= ROOT.lft AND grandParent.rgt <= ROOT.rgt AND  ROOT.Cat_Desc = '".$root."' AND ROOT.Type IN ('Brand_Store') AND ROOT.depth IN (1) )
       )";
        }

        $catID = 0;
        $sql = "SELECT
    IF ( Child.lft + 1 = Child.rgt,  'leaf', 'branch') as node_type,
    Child.*
        FROM
            (SELECT
                        node.Category_ID,
                        node.Cat_Desc,
                        node.Type,
                        node.lft,
                        node.rgt,
                        (COUNT(parent.Cat_Desc) - 1) AS depth
                    FROM
                        Nested_Category AS node
                        INNER JOIN Nested_Category AS parent ON node.lft BETWEEN  parent.lft AND parent.rgt AND parent.Business_ID = ".$business_id."
                    WHERE
                        node.Business_ID = ".$business_id."  AND node.Status = 'A'

                    GROUP BY Category_ID
                    ORDER BY node.lft
            )  AS ROOT ,
            Nested_Category AS Parent,
            Nested_Category AS Child,

            (SELECT
                        node.Category_ID,
                        node.Cat_Desc,
                        node.Type,
                        node.lft,
                        node.rgt,
                        (COUNT(parent.Cat_Desc) - 1) AS depth
                    FROM
                        Nested_Category AS node
                        INNER JOIN Nested_Category AS parent ON node.lft BETWEEN parent.lft AND parent.rgt AND parent.Business_ID = ".$business_id."
                    WHERE
                        node.Business_ID = ".$business_id." AND node.Status = 'A'
                    GROUP BY Category_ID
                    ORDER BY node.lft
            ) AS grandParent

        WHERE
            $brandStoreSql
            AND (Parent.lft > grandParent.lft AND Parent.rgt < grandParent.rgt AND grandParent.Cat_Desc = '".$cat1Name."' AND grandParent.Type = 'Category'   AND grandParent.depth > 0)
            AND  (Child.lft > Parent.lft AND Child.rgt < Parent.rgt AND Parent.Cat_Desc = '".$cat2Name."')
            AND Child.Cat_Desc = '".$cat3Name."'
            AND Parent.Status = 'A' AND Child.Status = 'A'
            ORDER BY node_type Desc ;";

        $getCategory = $pdo->prepare($sql);
        $getCategory->execute();
        /****
         * commented by sheetal
         * Reason : as per discussion by daya this line is same as print_r to sql statement
         * Commented at 2 feb 2022
         */
        //$getCategory->debugDumpParams();
        $categories = $getCategory->fetchAll(PDO::FETCH_ASSOC);
        $catArr = array_column($categories, 'node_type', 'Category_ID');
        return $catArr;
    }

### Article languages process
    private function insertInArticleLanguage($pdo, $article_no, $article){
        $createdBy = CREATED_BY_TMP_USER;

        $responseAL = [];
        if(isset($article['article_languages']) && count($article['article_languages'])>0)
        {
            foreach ($article['article_languages'] as $key => $language)
            {
                $lang_code  = trim($language['lang_code']);
                $article_desc_lang = trim($language['article_desc_lang']);
                $responseArtLang['lang_code'] = $lang_code;//response array

                try
                {
                    $get_langauge  = $pdo->prepare("SELECT * FROM  Article_Language WHERE Lang_Code='$lang_code' AND Article_No='$article_no'");
                    $get_langauge->execute();
                    /****
                     * commented by sheetal
                     * Reason : as per discussion by daya this line is same as print_r to sql statement
                     * Commented at 2 feb 2022
                     */
                    // $get_langauge->debugDumpParams();
                    $lang_count = $get_langauge->fetchAll(PDO::FETCH_ASSOC);
                    if(empty($lang_count))
                    {
                        $insert_sql = "INSERT INTO Article_Language (Article_No, Lang_Code, Article_Desc_Lang) VALUES (".$article_no.", '".$lang_code."', '".$article_desc_lang."')";

                        $article_insert = $pdo->prepare($insert_sql);
                        $article_insert->execute();
                        /****
                         * commented by sheetal
                         * Reason : as per discussion by daya this line is same as print_r to sql statement
                         * Commented at 2 feb 2022
                         */
                        // $article_insert->debugDumpParams();
                    }
                    else
                    {
                        $update_sql = "UPDATE Article_Language SET Article_Desc_Lang = '$article_desc_lang' WHERE Article_No = $article_no AND Lang_Code = '$lang_code'";

                        $lang_update    = $pdo->prepare($update_sql);
                        $query_response = $lang_update->execute();
                        /****
                         * commented by sheetal
                         * Reason : as per discussion by daya this line is same as print_r to sql statement
                         * Commented at 2 feb 2022
                         */
                        // $lang_update->debugDumpParams();
                    }
                    $responseArtLang['processed'] = true;
                    $responseArtLang['reason'] = 'Success';

                }
                catch ( PDOException $e)
                {
                    $responseArtLang['processed'] = false;
                    $responseArtLang['reason'] = $e->getMessage();
                    $errorsLang[$key][$lang_code] = $lang_code;
                    $errorsLang[$key]['reason']   = $e->getMessage();
                }
                $responseAL[] = $responseArtLang;


                //continue;
            }//end foreach
            return $responseAL;
            //exit;
        } else return false;
    }
}
