<?php

namespace App\Http\Controllers;

class ApiValidation
{
    /***
     * validateAPIData
     * validate API fields
     * @param : type (array) $dataValidations
     * @param : type (array) $data
     * $response = array of error validations
     * $data format : "mandatory-date|today|[>=]".strtotime(date('Y-m-d'))
    mandatory-date : mandatory is passed if field is required and date is type of validation
    mandatory-numeric : mandatory and number/integer is allowed
    mandatory-decimal : mandatory and decimal number/float is allowed
    mandatory-alpha : mandatory and only alphabated is allowed
    mandatory-array : mandatory and array is allowed


    Format 2: "mandatory-ValidationType|max length|default value can be slash separated|[condition]"
    E.g.  "mandatory-alpha or date or numeric or decimal or alpha or array|100|Business/Store

     ***/
    function validateAPIData(array $dataValidations, array $data){
        $errors = [];
        $mandatory = 0;
        if(!empty($dataValidations)){
            foreach($dataValidations as $field => $validations){
                $errorMsg = '';
                $paramName = str_replace("*","",$field);//Get parameter name remove star
                $paramValue = '';

                // if(isset($data[$paramName])) $paramValue = $data[$paramName];//get parameter value with paramName
                if(isset($data[$paramName])){
                    $paramValue = $data[$paramName];
                }
                $isEmptyValue = false;
                if (($paramValue == null || $paramValue == '') && $paramValue !== 0){
                    $isEmptyValue = true;
                }

                #check if fields has mandatory- in it, if yes then its mandatory
                $mandatory = 0;
                if(strpos($validations, "mandatory-") !== false){
                    $mandatory = 1;
                    if($isEmptyValue){
                        $errorMsg .= $field.' field is required. ';
                    }
                }

                #get validation type, max length and default value if given
                $dv = "";
                $validationType = '';
                $validations = str_replace("mandatory-", "", $validations);
                $validationArr = explode("|",$validations);
                if(!empty($validationArr)){
                    $validationType = $validationArr[0];//validation type

                    # max length
                    if(isset($validationArr[1]) && $validationType != "utf16text") {
                        $maxLength = $validationArr[1]; //max length
                        if(isset($maxLength) && $maxLength > 0 && $isEmptyValue == false) {
                            # default multiple value
                            if(strlen($paramValue) > $maxLength){
                                $errorMsg .= $paramName.' length should be less than equal to '.$maxLength.". ";
                            }
                        }
                    }

                    # default value
                    if(isset($validationArr[2])) {
                        $defaultValue = $validationArr[2]; //its default value
                        if(isset($defaultValue) && $defaultValue != '') {
                            # default multiple value
                            // print_r($validationArr[2]);
                            if(strpos($defaultValue, "/") !== false){
                                $defaultCompareValues = explode('/', strtolower($defaultValue)); // slash for or condition
                                if(!in_array(strtolower($paramValue), $defaultCompareValues)){
                                    $errorMsg .= $paramName.' should be '.$defaultValue.". ";
                                }
                            } else {
                                $dv = $paramName." set '".$defaultValue."' as default value. ";
                            }
                        }
                    }
                    # condition

                    if(isset($validationArr[3])) { // condition
                        $condition = $validationArr[3]; //its default value

                    }
                }

                if($validationType == 'array'){
                    if(!empty($paramValue) && !is_array($paramValue) ){
                        $errorMsg .= $field.' should be an array. ';
                    }
                }

                if($validationType == "numeric"){

                    // print_r(is_double($temp));die;
                    if(!$isEmptyValue && isset($condition) && $condition != '') {
                        $errorMsg .= $this->validateConditions($paramValue, $condition, $validationType,$paramName);
                    }
                    if(!$isEmptyValue && !is_float($paramValue) && !is_numeric($paramValue) ){//Y-m-d
                        $errorMsg .= $field.' is not valid number. ';
                    }


                }



                if($validationType == "decimal"){
                    // print_r($validationType);
                    // print_r("\n");
                    // print_r($paramValue);
                    // print_r("\n");
                    $validationArr = explode("|",$validations);

                    $condition = $validationArr[1]; //its default value
                    if(!$isEmptyValue && isset($condition) && $condition != '') {
                        $errorMsg .= $this->validateConditions($paramValue, $condition, $validationType,$paramName);
                    }
                    if(!$isEmptyValue && !is_float($paramValue) && !is_numeric($paramValue)){
                        $errorMsg .= $field.' is not number. ';
                    }

                }

                if($validationType == "alpha"){
                    if(!$isEmptyValue && !ctype_alpha($paramValue)){
                        $errorMsg .= $field.' is not string. ';
                    }
                }

                if($validationType == "alnum"){
                    $caseInsensetive = '/i';
                    if(!$isEmptyValue && isset($condition) && $condition != '') {
                        $errorMsg .= $this->validateConditions($paramValue, $condition, $validationType,$paramName);
                    }

                    if(!$isEmptyValue && !preg_match('/^[a-z0-9 .\-]+$'.$caseInsensetive, $paramValue)){
                        $errorMsg .= $field.' is not alpha numeric. ';
                    }
                }

                if($validationType == "alnumslash"){
                    if(!$isEmptyValue && !preg_match('/^[a-z0-9:\/.\-]+$/i', $paramValue)){
                        $errorMsg .= $field.' is not alpha numeric. ';
                    }
                }

                if($validationType == "alnumslashspace"){
                    if(!$isEmptyValue && !preg_match('/^[a-z0-9 :\/.\-]+$/i', $paramValue)){
                        $errorMsg .= $field.' is not valid. ';
                    }
                }

                if($validationType == "text"){
                    if(!$isEmptyValue && !preg_match('/^[a-z0-9,\'\" :\/\%\—\\.\-\?!@\$=;\*&\(\)\[\]_]+$/i', $paramValue)){
                        $errorMsg .= $field.' has invalid characters, allowed characters are a-z0-9,\'" :/.-—%?!@$=;*&()[]_  ';
                    }
                }

                if($validationType == "utf16text"){
                    # remaining
                }

                if($validationType == "date"){
                    if(!$isEmptyValue && isset($condition) && $condition != '') {
                        $errorMsg .= $this->validateConditions(strtotime($paramValue), $condition, $validationType,$paramName);
                    }

                    if(!$isEmptyValue && !$this->isRealDate($paramValue)){//Y-m-d
                        $errorMsg .= $field.' is not valid date. ';
                    }
                }


                #If mandatory then show default value message
                if($mandatory == 1 && $errorMsg != ''){
                    $errorMsg.=$dv;
                }

                if($errorMsg != ''){
                    //$errors[$field][] = $errorMsg;
                    $errors[] = $errorMsg;
                }


            }
            /* print_r($errors);
            exit; */
        }
        $errors = implode(" ", $errors);
        return $errors;
    }

    function isRealDate($date) {
        if (false === strtotime($date)) {
            return false;
        }
        /**
         * Commented below line by Priyanka Kumbhar
         * Commented @ 20-04-2022
         * Reason : Facing following error "a non well formed numeric value encountered"
         */
        // list($year, $month, $day) = explode('-', $date);
        $date=strtotime($date);
        $month=date("m",$date);
        $year=date("Y",$date);
        $day=date("d",$date);
        return checkdate($month, $day, $year);
    }

    function validateConditions($paramValue, $condition, $validationType,$paramName){
        preg_match("/\[[^\]]*\]/", $condition, $matches);

        $operator = str_replace(array( '[', ']' ), '', $matches['0']);
        $otherParam = (int)str_replace($matches['0'], '', $condition);
        $errorMsg = '';
        $param1 = $paramValue;
        $param2 = $otherParam;

        if($validationType == 'date'){
            $param1 = date('Y-m-d', $paramValue);
            $param2 = date('Y-m-d', $otherParam);
        }

        if($validationType == 'numeric'){
            $param1 = $paramName;
            $param2 = 0;
        }

        if($validationType == 'decimal'){
            $param1 = $paramName;
            $param2 = 0;
        }
        if($validationType == 'alnum'){
            $param1 = $paramName;
            $param2 = 0;
        }
        // print_r($operator);
        switch ($operator) {
            case '<':
                if($paramValue < $otherParam) {
                    //nothing
                } else $errorMsg .= $param1." should be less than ".$param2.". ";
                break;
            case '>':
                if($paramValue > $otherParam) {
                    //nothing
                } else $errorMsg .= $param1." should be greater than ".$param2.". ";
                break;
            case '>=':
                if($paramValue >= $otherParam) {
                    //nothing
                } else $errorMsg .= $param1." should be greater than equal to ".$param2.". ";
                break;
            case '<=':
                if($paramValue <= $otherParam) {
                    //nothing
                } else $errorMsg .= $param1." should be less than equal to ".$param2.". ";
                break;
            default:
                break;
        }

        return $errorMsg;
    }





    #print sql errors in try catch block
    function handle_sql_errors($query, $error_message)
    {
        echo '<pre>';
        print_r($query);
        echo '</pre>';
        echo $error_message;
        die;
    }
}
