<?php

function get_order_details($pdo, $order_no)
{
    $param_arry = array();
    $param_arry[':order_no'] = $order_no;

    try {
        $orderhdrstmt = "SELECT Order_No, COALESCE(Ext_invoice_No ,'') Ext_Invoice_No, Order_Amount, Net_Amount, Store_No, Cust_No, FSO_No FROM Order_Hdr where Order_No = :order_no  LIMIT 1;";
        $selodrstmt = $pdo->prepare($orderhdrstmt);
        $selodrstmt->execute($param_arry);
        $order_rows = $selodrstmt->fetch(PDO::FETCH_ASSOC);
        return ($order_rows);

    } catch (PDOException $e) {
        $msg = "Error retrieving order details for order number $order_no " . $e->getMessage();
        return (array());
    }
} // get_order_details

function get_order_item_details($pdo, $order_no)
{
    $param_arry = array();
    $param_arry[':order_no'] = $order_no;

    try {
        $orderitemstmt = "SELECT Order_No, Article_No, Order_Quantity, Order_Mrp, Order_Offer_Price, Order_Amount from Order_Itm where Order_No = :order_no;";
        $selorigstmt = $pdo->prepare($orderitemstmt);
        $selorigstmt->execute($param_arry);
        $rows = $selorigstmt->fetchAll(PDO::FETCH_ASSOC);
        return ($rows);

    } catch (PDOException $e) {
        $msg = "Error retrieving order items order number $order_no " . $e->getMessage();
        return (array());
    }
}

function get_amount_store_transaction($pdo, $orderno, $invoiceno)
{
    try {
        $storebal_param_arry = array();
        $storebal_param_arry[':orderno'] = $orderno;
        $storebal_param_arry[':invoiceno'] = $invoiceno;

        $storetranstmt = $pdo->prepare("SELECT * FROM Store_Transaction
        WHERE Order_No = :orderno AND Invoice_No = :invoiceno
        ORDER BY Transaction_ID DESC Limit 1;");
    } catch (PDOException $e) {
        $msg = "Error creating statement for store transactions for order " . $orderno . " " . $invoiceno . ".";
        echo " $msg " . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    try {
        $storetranstmt->execute($storebal_param_arry);
        $store_bals = $storetranstmt->fetch(PDO::FETCH_ASSOC);
        $msg = "Retrieved existing and outstanding balance for order " . $orderno . " and buyer " . $invoiceno . ".";
        return ($store_bals);
    } catch (PDOException $e) {
        $msg = "Error getting existing and outstanding balance for order " . $orderno . " and buyer " . $invoiceno . ".";
        echo $msg . $e->getMessage();
        return (array());

    }
}

function get_store_balance($pdo, $storeno, $custno)
{

    try {
        $storebal_param_arry = array();
        $storebal_param_arry[':storeno'] = $storeno;
        $storebal_param_arry[':custno'] = $custno;

        $storetranstmt = $pdo->prepare("SELECT Balance_Amt, Outstanding_Amt, Outstanding_Balance_Amt FROM Store_Transaction
            WHERE Store_No = :storeno AND Cust_No = :custno
            ORDER BY Transaction_ID DESC Limit 1;");
    } catch (PDOException $e) {
        $msg = "Error creating statement for store transactions for store " . $storeno . " " . $custno . ".";
        echo " $msg " . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    try {
        $storetranstmt->execute($storebal_param_arry);
        $store_bals = $storetranstmt->fetch(PDO::FETCH_ASSOC);
        $msg = "Retrieved existing and outstanding balance for store " . $storeno . " and buyer " . $custno . ".";
        return ($store_bals);
    } catch (PDOException $e) {
        $msg = "Error getting existing and outstanding balance for store " . $storeno . " and buyer " . $custno . ".";
        echo $msg . $e->getMessage();
        return (array());

    }

} // get_store_balance

function get_invoice_hdr($pdo, $order_no, $invoice_number)
{
    $param_arry = array();
    $param_arry[':order_no'] = $order_no;
    $param_arry[':invoice_number'] = $invoice_number;

    try {
        $invoicehdrstmt = "SELECT Invoice_Hdr_Id from Invoice_Hdr where Order_No = :order_no and Invoice_No = :invoice_number;";
        $selinvoicetmt = $pdo->prepare($invoicehdrstmt);
        $selinvoicetmt->execute($param_arry);
        $rows = $selinvoicetmt->fetchAll(PDO::FETCH_ASSOC);
        return ($rows);

    } catch (PDOException $e) {
        $msg = "Error retrieving order items order number $order_no " . $e->getMessage();
        return (array());
    }
}

function get_invoice_tax_hdr($pdo, $order_no, $invoice_number)
{
    $param_arry = array();
    $param_arry[':order_no'] = $order_no;
    $param_arry[':invoice_number'] = $invoice_number;

    try {
        $invoicetaxhdrstmt = "SELECT Invoice_Tax_Hdr_Id from Invoice_Tax_Hdr where Order_No = :order_no and Invoice_No = :invoice_number;";
        $selinvoicetaxtmt = $pdo->prepare($invoicetaxhdrstmt);
        $selinvoicetaxtmt->execute($param_arry);
        $rows = $selinvoicetaxtmt->fetchAll(PDO::FETCH_ASSOC);
        return ($rows);

    } catch (PDOException $e) {
        $msg = "Error retrieving order items order number $order_no " . $e->getMessage();
        return (array());
    }
}

function update_invoice_tax_header($pdo, $invoice_tax_hdr_order,$order_no)
{
    // print_r($invoice_tax_hdr_order);
    // die();
    $creationDateTime = date('Y-m-d H:i:s');
    $invoicetaxhdrstmt = $pdo->prepare("UPDATE Invoice_Tax_Hdr
                            SET Order_No = :order_no,
                                Creation_date = :creation_date,
                                Created_By = :created_by
                            where Invoice_Tax_Hdr_Id = :invoice_tax_hdr_id");

    foreach ($invoice_tax_hdr_order as $key => $val) {
        $invoice_hdr_tax_param_arry = array();
        $invoice_hdr_tax_param_arry[':order_no'] = $val['Order_No'];
        $invoice_hdr_tax_param_arry[':invoice_tax_hdr_id'] = $val['Invoice_Tax_Hdr_Id'];
        $invoice_hdr_tax_param_arry[':creation_date'] = $creationDateTime;
        $invoice_hdr_tax_param_arry[':created_by'] = 10000;

        try {
            $invoicetaxhdrstmt->execute($invoice_hdr_tax_param_arry);
            // $invoicetaxhdrstmt->debugDumpParams();
            $msg = "Invoice tax header updated for  " . $order_no;
            $retval = true;
        } catch (PDOException $e) {
            $msg = "Error updating invoice tax header for order " . $order_no;
            echo $msg . $e->getMessage();
            $retval = false;
            return ($retval);
        }
    }
    return ($retval);
}

function get_invoice_tax_itm($pdo, $order_no, $invoice_number)
{
    $param_arry = array();
    $param_arry[':order_no'] = $order_no;
    $param_arry[':invoice_number'] = $invoice_number;

    try {
        $invoicetaxitmstmt = "SELECT Invoice_Tax_Itm_Id from Invoice_Tax_Itm where Order_No = :order_no and Invoice_No = :invoice_number;";
        $selinvoiceitmtmt = $pdo->prepare($invoicetaxitmstmt);
        $selinvoiceitmtmt->execute($param_arry);
        $rows = $selinvoiceitmtmt->fetchAll(PDO::FETCH_ASSOC);
        return ($rows);

    } catch (PDOException $e) {
        $msg = "Error retrieving invoice items order number $order_no " . $e->getMessage();
        return (array());
    }
}

function update_invoice_tax_item($pdo, $invoice_tax_itm_order, $order_no)
{
    // print_r($invoice_tax_itm_order);
    // die();
    $creationDateTime = date('Y-m-d H:i:s');
    $invoiceitmhdrstmt = $pdo->prepare("UPDATE Invoice_Tax_Itm
                            SET Order_No = :order_no,
                                Creation_date = :creation_date,
                                Created_By = :created_by
                            where Invoice_Tax_Itm_Id = :invoice_tax_itm_id");

    foreach ($invoice_tax_itm_order as $key => $val) {
        $invoice_hdr_itm_param_arry = array();
        $invoice_hdr_itm_param_arry[':order_no'] = $val['Order_No'];
        $invoice_hdr_itm_param_arry[':invoice_tax_itm_id'] = $val['Invoice_Tax_Itm_Id'];
        $invoice_hdr_itm_param_arry[':creation_date'] = $creationDateTime;
        $invoice_hdr_itm_param_arry[':created_by'] = 10000;

        try {
            $invoiceitmhdrstmt->execute($invoice_hdr_itm_param_arry);
            // $invoiceitmhdrstmt->debugDumpParams();
            // die();
            $msg = "Invoice tax itm updated for  " . $order_no;
            $retval = true;
        } catch (PDOException $e) {
            $msg = "Error updating invoice tax itm for order " . $order_no;
            echo $msg . $e->getMessage();
            $retval = false;
            return ($retval);
        }
    }
    return ($retval);
}

function update_order_items($pdo, $order_items)
{
    $orderitemstmt = '';
    $param_array = '';

    try {

        $orderitemstmt = $pdo->prepare("UPDATE Order_Itm
                            SET Pick_Quantity = :pick_quantity
                                , Pick_Amount = ( :pick_qty * Order_Offer_Price)
                                , Net_Amount =  ( :pick_quanty * Order_Offer_Price)
                                , Item_Weight = ((  Item_Weight / Order_Quantity ) * :pick_quanti )
                                , Saving_Amount = :saving_amount
                                , Change_Date = now()
                                , Change_Time = now()
                                , Changed_By = 10000
                            where Order_No = :order_no and Article_No = :article_number;");

    } catch (PDOException $e) {
        $msg = "Error preparing update order items for order" . $order_items[$key]['order_no'] . " article " . $order_items[$key]['article_number'];
        echo " $msg " . $e->getMessage();
        $retval = false;
        throw $e;
    }

    try {

        foreach ($order_items as $key => $val) {

            $param_array = array();
            $param_array[':order_no'] = $order_items[$key]['order_no'];
            $param_array[':article_number'] = $order_items[$key]['article_number'];
            $param_array[':pick_quantity'] = $order_items[$key]['pick_quantity'];
            $param_array[':pick_qty'] = $order_items[$key]['pick_quantity'];
            $param_array[':pick_quanti'] = $order_items[$key]['pick_quantity'];
            $param_array[':pick_quanty'] = $order_items[$key]['pick_quantity'];
            $param_array[':saving_amount'] = $order_items[$key]['saving_amount'];
            $orderitemstmt->execute($param_array);
            $msg = "The order items are updated for " . $order_items[$key]['order_no'] . " ";
            $retval = true;
        }

    } catch (PDOException $e) {
        $msg = "Error updating order items for order " . $order_items[0]['order_no'];
        echo $msg . $e->getMessage();
        $retval = false;
        throw $e;
    }

    return ($retval);

}

function update_invoice_order_hdr($pdo, $order_hdr)
{
    $orderhdrstmt = '';

    $param_array[':order_no'] = $order_hdr['order_number'];
    $param_array[':orderno'] = $order_hdr['order_number'];
    $param_array[':ext_invoice_no'] = $order_hdr['ext_invoice_no'];
    $param_array[':order_status'] = $order_hdr['order_status'];

    try {
        $orderhdrstmt = $pdo->prepare("UPDATE Order_Hdr oh inner join
        (select  sum(order_item.perunit_wt * order_item.Pick_Quantity) order_wt ,
                sum(order_item.perunit_offer_price * order_item.Pick_Quantity ) order_amt,
                sum(order_item.Item_Discount) header_discount
        from
        ( select Article_No, Pick_Quantity, (  Item_Weight / Pick_Quantity ) perunit_wt, Order_Offer_Price perunit_offer_price, Item_Discount from Order_Itm where Order_No = :orderno ) order_item
        ) x
        set oh.Order_Amount = x.order_amt
        , oh.Total_Weight = x.order_wt
        , oh.Header_Discount = x.header_discount
        , oh.Net_Amount = x.order_amt - x.header_discount
        , oh.Saving_Amount = x.order_amt - ( x.order_amt - x.header_discount )
        , oh.Order_Status = :order_status
        , oh.Change_Date = now()
        , oh.Change_Time = now()
        , oh.Changed_By = 10000
        where oh.Order_No = :order_no and
              oh.Ext_Invoice_No = :ext_invoice_no;");

    } catch (PDOException $e) {
        $msg = "Error preparing update order header order" . $order_hdr['order_number'];
        echo " $msg " . $e->getMessage();
        $retval = false;
        throw $e;
    }

    try {
        $orderhdrstmt->execute($param_array);
        $msg = "The order header is updated for " . $order_hdr['order_number'] . " ";
        $retval = true;

    } catch (PDOException $e) {
        $msg = "Error updating order header for order " . $order_hdr['order_number'];
        echo $msg . $e->getMessage();
        $retval = false;
        throw $e;
    }

    return ($retval);

}

function create_store_transaction($pdo, $store_transaction)
{
    // insert store_transaction entry
    // echo"<pre>";
    // print_r($store_transaction);
    // echo"<br>";
    $creationDateTime = date('Y-m-d H:i:s');
    $store_txn_param_arry = array();
    $store_txn_param_arry[':transaction_type'] = $store_transaction['transaction_type'];
    $store_txn_param_arry[':store_no'] = $store_transaction['store_no'];
    $store_txn_param_arry[':cust_no'] = $store_transaction['cust_no'];
    $store_txn_param_arry[':fso_no'] = $store_transaction['fso_no'];
    $store_txn_param_arry[':order_no'] = $store_transaction['order_no'];
    $store_txn_param_arry[':invoice_no'] = $store_transaction['invoice_number'];
    $store_txn_param_arry[':transaction_date'] = $creationDateTime;
    // $store_transaction['invoice_date'];

    $store_txn_param_arry[':deposit_amount'] = $store_transaction['deposit_amount'];
    $store_txn_param_arry[':withdraw_amount'] = $store_transaction['withdraw_amount'];
    $store_txn_param_arry[':balance_amount'] = $store_transaction['balance_amt'];
    $store_txn_param_arry[':outstanding_amount'] = $store_transaction['outstanding_amt'];
    $store_txn_param_arry[':outstanding_balance_amount'] = $store_transaction['outstanding_balance_amt'];

    $store_txn_param_arry[':creation_date'] = $creationDateTime;
    $store_txn_param_arry[':created_by'] = 10000;

    $store_txn_param_arry[':change_date'] = $creationDateTime;
    $store_txn_param_arry[':changed_by'] = 10000;

    $store_txn_param_arry[':remark'] = $store_transaction['remark']; // "Order Ready Ext Bill Entry"

    // print_r($store_txn_param_arry);

    try {

        $storetxnstmt = $pdo->prepare("INSERT INTO `Store_Transaction`( Transaction_Type, Store_No, Cust_No, FSO_No, Order_No, Invoice_No, Deposit_Amt, Withdraw_Amt, Balance_Amt, Outstanding_Amt, Outstanding_Balance_Amt, Transaction_Date, Creation_Date, Created_By, Changed_Date, Changed_By, Remark)
        values ( :transaction_type, :store_no, :cust_no, :fso_no, :order_no, :invoice_no, :deposit_amount, :withdraw_amount,:balance_amount,:outstanding_amount, :outstanding_balance_amount, :transaction_date, :creation_date, :created_by, :change_date, :changed_by, :remark)");

    } catch (PDOException $e) {
        $msg = "Error preparing store transaction for order  " . $store_transaction['order_no'];
        echo " $msg " . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    // print_r($storetxnstmt);

    try {
        $storetxnstmt->execute($store_txn_param_arry);
        // $storetxnstmt->debugDumpParams();
        // echo"<br>";
        // die();
        $msg = "Store transaction entry created for order " . $store_transaction['order_no'];
        $retval = true;

    } catch (PDOException $e) {
        $msg = "Error creating store transaction entry for order " . $store_transaction['order_no'];
        echo $msg . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    return ($retval);

}

function create_store_transaction_from_order($pdo, $store_transaction)
{
    // insert store_transaction entry for updated order amount after reversing the original order amount entry.

    $creationDateTime = date('Y-m-d H:i:s');
    $store_txn_param_arry = array();
    $store_txn_param_arry[':transaction_type'] = $store_transaction['transaction_type'];
    $store_txn_param_arry[':store_no'] = $store_transaction['store_no'];
    $store_txn_param_arry[':cust_no'] = $store_transaction['cust_no'];
    $store_txn_param_arry[':fso_no'] = $store_transaction['fso_no'];
    $store_txn_param_arry[':order_no'] = $store_transaction['order_no'];
    $store_txn_param_arry[':order_number'] = $store_transaction['order_no'];
    $store_txn_param_arry[':invoice_no'] = $store_transaction['invoice_number'];
    $store_txn_param_arry[':transaction_date'] = $store_transaction['invoice_date'];

    $store_txn_param_arry[':deposit_amount'] = $store_transaction['deposit_amount'];
    // $store_txn_param_arry[':withdraw_amount'] = $store_transaction['withdraw_amount'];
    $store_txn_param_arry[':balance_amount'] = $store_transaction['balance_amt'];
    $store_txn_param_arry[':outstanding_amount'] = $store_transaction['outstanding_amt'];
    $store_txn_param_arry[':outstanding_balance_amount'] = $store_transaction['outstanding_balance_amt'];

    $store_txn_param_arry[':creation_date'] = $creationDateTime;
    $store_txn_param_arry[':created_by'] = 10000;

    $store_txn_param_arry[':change_date'] = $creationDateTime;
    $store_txn_param_arry[':changed_by'] = 10000;

    $store_txn_param_arry[':remark'] = $store_transaction['remark']; // "Order Ready Ext Bill Entry"

    // print_r($store_txn_param_arry);

    try {

        $storetxnstmt = $pdo->prepare("INSERT INTO `Store_Transaction`( Transaction_Type, Store_No, Cust_No, FSO_No, Order_No, Invoice_No, Deposit_Amt, Withdraw_Amt, Balance_Amt, Outstanding_Amt, Outstanding_Balance_Amt, Transaction_Date, Creation_Date, Created_By, Changed_Date, Changed_By, Remark)
        select :transaction_type, :store_no, :cust_no, :fso_no, :order_no, :invoice_no, :deposit_amount, Order_Amount,:balance_amount,:outstanding_amount, :outstanding_balance_amount, :transaction_date, :creation_date, :created_by, :change_date, :changed_by, :remark
        from Order_Hdr
        where Order_No = :order_number");

    } catch (PDOException $e) {
        $msg = "Error preparing store transaction for order  " . $store_transaction['order_no'];
        echo " $msg " . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    // print_r($storetxnstmt);

    try {
        $storetxnstmt->execute($store_txn_param_arry);
        $msg = "Store transaction entry created for order " . $store_transaction['order_no'];
        $retval = true;

    } catch (PDOException $e) {
        $msg = "Error creating store transaction entry for order " . $store_transaction['order_no'];
        echo $msg . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    return ($retval);

}

function create_store_transaction_detail($pdo, $store_transaction_detail)
{
    // insert store_transaction_detail entry

    $creationDateTime = date('Y-m-d H:i:s');
    $store_txn_detail_param_arry = array();

    $store_txn_detail_param_arry[':store_no'] = $store_transaction_detail['store_no'];
    $store_txn_detail_param_arry[':cust_no'] = $store_transaction_detail['cust_no'];

    // print_r($store_txn_detail_param_arry);

    try {
        $storetxndetstmt = $pdo->prepare("INSERT INTO `Store_Transaction_Detail`( Transaction_ID, Deposit_Mode, Ref_No, Bank_Name, Branch_Name, Creation_Date, Created_By, Changed_Date, Changed_By)
            SELECT Transaction_Id, 3, 'NA', 'NA', 'NA', Creation_Date, Created_By, Changed_Date, Changed_By from Store_Transaction where Store_No = :store_no and Cust_No = :cust_no order by Transaction_Id desc LIMIT 1;");

    } catch (PDOException $e) {
        $msg = "Error preparing store transaction detail for order  " . $store_transaction_detail['order_no'];
        echo " $msg " . $e->getMessage();
        $retval = false;
        return ($retval);
    }
    // print_r($storetxndetstmt);

    try {
        $storetxndetstmt->execute($store_txn_detail_param_arry);
        $msg = "Store transaction detail entry created for order " . $store_transaction_detail['order_no'];
        $retval = true;

    } catch (PDOException $e) {
        $msg = "Error creating store transaction detail entry for order " . $store_transaction_detail['order_no'];
        echo $msg . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    return ($retval);

}

function insert_invoice_order_history($pdo, $order_no, $memo)
{

    // insert_order_history

    $creationDate = date('Y-m-d');
    $creationTime = date('H:i:s');

    $history_param_arry = array();
    $history_param_arry[':order_id'] = $order_no;
    $history_param_arry[':memo'] = $memo;
    $history_param_arry[':history_date'] = $creationDate;
    $history_param_arry[':history_time'] = $creationTime;
    $history_param_arry[':status'] = 'A';
    $history_param_arry[':creation_date'] = $creationDate;
    $history_param_arry[':creation_time'] = $creationTime;
    $history_param_arry[':change_date'] = $creationDate;
    $history_param_arry[':change_time'] = $creationTime;
    $history_param_arry[':login_type'] = "Seller";
    $history_param_arry[':user_no'] = 10000;

    try {
        $orderhistorystmt = $pdo->prepare("INSERT INTO `Order_History`(`Order_ID`, `Memo`, `History_Date`, `History_Time`, `Status`, `Creation_Date`, `Creation_Time`, `Created_By`, `Change_Date`, `Change_Time`, `Changed_By`,`Login_Type`)
        VALUES (  :order_id, :memo, :history_date, :history_time, :status, :creation_date, :creation_time, :user_no, :change_date, :change_time, :user_no,:login_type)");

    } catch (PDOException $e) {
        $msg = "Error preparing order history sql statement for order " . $order_hdr['order_number'];
        echo " $msg " . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    try {
        $orderhistorystmt->execute($history_param_arry);
        $msg = "The order history is created for " . $order_no;
        $retval = true;

    } catch (PDOException $e) {
        $msg = "Error creating order history for order " . $order_no;
        // echo $msg . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    return ($retval);

}

function create_invoice_header($pdo, $invoice_hdr)
{
    // insert invoice_header entry

    $creationDateTime = date('Y-m-d H:i:s');

    $invoice_hdr_param_arry = array();
    $invoice_hdr_param_arry[':order_no'] = $invoice_hdr['order_no'];
    $invoice_hdr_param_arry[':invoice_no'] = $invoice_hdr['invoice_number'];
    $invoice_hdr_param_arry[':invoice_date'] = $invoice_hdr['invoice_date'];
    $invoice_hdr_param_arry[':invoice_type'] = $invoice_hdr['invoice_type'];
    $invoice_hdr_param_arry[':tax_type'] = $invoice_hdr['tax_type'];
    $invoice_hdr_param_arry[':tot_inv_qty'] = $invoice_hdr['total_invoice_qty'];
    $invoice_hdr_param_arry[':tot_inv_amount'] = $invoice_hdr['total_invoice_amount'];
    $invoice_hdr_param_arry[':tot_base_amount'] = $invoice_hdr['total_base_amount'];
    $invoice_hdr_param_arry[':tot_cgst_amount'] = $invoice_hdr['total_cgst_amount'];
    $invoice_hdr_param_arry[':tot_sgst_amount'] = $invoice_hdr['total_sgst_amount'];
    $invoice_hdr_param_arry[':tot_igst_amount'] = $invoice_hdr['total_igst_amount'];
    $invoice_hdr_param_arry[':tot_gst_amount'] = $invoice_hdr['total_gst_amount'];
    $invoice_hdr_param_arry[':tot_cess_amount'] = $invoice_hdr['total_cess_amount'];
    $invoice_hdr_param_arry[':creation_date'] = $creationDateTime;
    $invoice_hdr_param_arry[':created_by'] = 10000;
    $invoice_hdr_param_arry[':Changed_Date'] = $creationDateTime;
    $invoice_hdr_param_arry[':Changed_By'] = 10000;


    // print_r($invoice_hdr_param_arry);

    try {
        $invoicehdrstmt = $pdo->prepare("INSERT INTO `Invoice_Hdr`(Order_No,Invoice_No,  Invoice_Date, Invoice_Type, Tax_Type, Tot_Inv_Amt, Tot_Inv_Qty, Tot_Base_Amt, Tot_CGST_Amt, Tot_SGST_Amt, Tot_IGST_Amt, Tot_GST_Amt, Tot_CESS_Amt, Created_By, Creation_Date, Changed_By, Changed_Date)
        values ( :order_no, :invoice_no, :invoice_date,:invoice_type,:tax_type, :tot_inv_amount, :tot_inv_qty, :tot_base_amount,:tot_cgst_amount, :tot_sgst_amount,:tot_igst_amount,:tot_gst_amount,  :tot_cess_amount,:created_by,:creation_date, :Changed_By, :Changed_Date)");

    } catch (PDOException $e) {
        $msg = "Error creating invoice header record " . $invoice_hdr['order_no'];
        echo " $msg " . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    // print_r($invoicehdrstmt);

    try {
        $invoicehdrstmt->execute($invoice_hdr_param_arry);
        $msg = "Invoice header entry created for " . $invoice_hdr['order_no'];
        $retval = true;

    } catch (PDOException $e) {
        $msg = "Error creating invoice header entry for order " . $invoice_hdr['order_no'];
        echo $msg . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    return ($retval);

}

function create_invoice_tax_header($pdo, $invoice_tax_hdr)
{

    $creationDateTime = date('Y-m-d H:i:s');

    $order_no = $invoice_tax_hdr[0]['order_no'];

    $invoicetaxhdrstmt = $pdo->prepare("INSERT INTO Invoice_Tax_Hdr( GST_Tax_Id, CESS_Tax_Id, Invoice_No, Order_No, Tot_Inv_Amt, Tot_Inv_Qty, Tot_Base_Amt, Tot_CGST_Amt, Tot_SGST_Amt, Tot_IGST_Amt, Tot_GST_Amt, Tot_CESS_Amt, Created_By, Creation_date)
    values ( :gst_tax_id, :cess_tax_id, :invoice_no, :order_no, :tot_inv_amount, :tot_inv_qty, :tot_base_amount,:tot_cgst_amount, :tot_sgst_amount,:tot_igst_amount,:tot_gst_amount,  :tot_cess_amount,:created_by,:creation_date)");

    foreach ($invoice_tax_hdr as $key => $val) {
        $invoice_hdr_tax_param_arry = array();
        $invoice_hdr_tax_param_arry[':gst_tax_id'] = $key;
        $invoice_hdr_tax_param_arry[':cess_tax_id'] = 1; // constant for now as there isnt clarity on whether this applies.
        $invoice_hdr_tax_param_arry[':order_no'] = $invoice_tax_hdr[$key]['order_no'];
        $invoice_hdr_tax_param_arry[':invoice_no'] = $invoice_tax_hdr[$key]['invoice_number'];
        $invoice_hdr_tax_param_arry[':tot_inv_qty'] = $invoice_tax_hdr[$key]['total_invoice_qty'];
        $invoice_hdr_tax_param_arry[':tot_inv_amount'] = $invoice_tax_hdr[$key]['total_invoice_amount'];
        $invoice_hdr_tax_param_arry[':tot_base_amount'] = $invoice_tax_hdr[$key]['total_base_amount'];
        $invoice_hdr_tax_param_arry[':tot_cgst_amount'] = $invoice_tax_hdr[$key]['total_cgst_amount'];
        $invoice_hdr_tax_param_arry[':tot_sgst_amount'] = $invoice_tax_hdr[$key]['total_sgst_amount'];
        $invoice_hdr_tax_param_arry[':tot_igst_amount'] = $invoice_tax_hdr[$key]['total_igst_amount'];
        $invoice_hdr_tax_param_arry[':tot_gst_amount'] = $invoice_tax_hdr[$key]['total_gst_amount'];
        $invoice_hdr_tax_param_arry[':tot_cess_amount'] = $invoice_tax_hdr[$key]['total_cess_amount'];
        $invoice_hdr_tax_param_arry[':creation_date'] = $creationDateTime;
        $invoice_hdr_tax_param_arry[':created_by'] = 10000;

        //  print_r($invoicetaxhdrstmt);

        //  echo " Key is " . $key;
        //  print_r($invoice_hdr_tax_param_arry);

        try {
            $invoicetaxhdrstmt->execute($invoice_hdr_tax_param_arry);
            $msg = "Invoice tax header created for  " . $order_no;
            $retval = true;
        } catch (PDOException $e) {
            $msg = "Error creating invoice tax header for order " . $order_no;
            echo $msg . $e->getMessage();
            $retval = false;
            return ($retval);
        }
    }
    return ($retval);

}

function create_invoice_tax_items($pdo, $invoice_items)
{

    $creationDateTime = date('Y-m-d H:i:s');
    $order_no = $invoice_items[0]['order_no'];

    $invoicetaxitemstmt = $pdo->prepare("INSERT INTO Invoice_Tax_Itm( GST_Tax_Id, GST_Tax_Detail_Id, CESS_Tax_Id, CESS_Tax_Detail_Id, Article_No, Invoice_No, Order_No, Tot_Inv_Amt, Tot_Inv_Qty, Tot_Base_Amt, Tot_CGST_Amt, Tot_SGST_Amt, Tot_IGST_Amt, Tot_GST_Amt, Tot_CESS_Amt, Created_By, Creation_date)
    values ( :gst_tax_id, :gst_tax_detail_id, :cess_tax_id, :cess_tax_detail_id, :article_no, :invoice_no, :order_no, :tot_inv_amount, :tot_inv_qty, :tot_base_amount,:tot_cgst_amount, :tot_sgst_amount,:tot_igst_amount,:tot_gst_amount,  :tot_cess_amount,:created_by,:creation_date)");

    foreach ($invoice_items as $key => $val) {
        $invoice_tax_items_param_arry = array();
        $invoice_tax_items_param_arry[':gst_tax_id'] = $invoice_items[$key]['gst_tax_id'];
        $invoice_tax_items_param_arry[':gst_tax_detail_id'] = $invoice_items[$key]['gst_tax_detail_id'];
        $invoice_tax_items_param_arry[':cess_tax_id'] = 1; // constant for now as there isnt clarity on whether this applies.
        $invoice_tax_items_param_arry[':cess_tax_detail_id'] = 1; // constant for now as there isnt clarity on whether this applies.
        $invoice_tax_items_param_arry[':article_no'] = $invoice_items[$key]['sk_article_no'];
        $invoice_tax_items_param_arry[':order_no'] = $invoice_items[$key]['order_no'];
        $invoice_tax_items_param_arry[':invoice_no'] = $invoice_items[$key]['invoice_number'];
        $invoice_tax_items_param_arry[':tot_inv_qty'] = $invoice_items[$key]['article_invoice_quantity'];
        $invoice_tax_items_param_arry[':tot_inv_amount'] = $invoice_items[$key]['article_invoice_amount'];
        $invoice_tax_items_param_arry[':tot_base_amount'] = $invoice_items[$key]['article_base_amount'];
        $invoice_tax_items_param_arry[':tot_cgst_amount'] = $invoice_items[$key]['article_cgst_amount'];
        $invoice_tax_items_param_arry[':tot_sgst_amount'] = $invoice_items[$key]['article_sgst_amount'];
        $invoice_tax_items_param_arry[':tot_igst_amount'] = $invoice_items[$key]['article_igst_amount'];
        $invoice_tax_items_param_arry[':tot_gst_amount'] = $invoice_items[$key]['article_gst_amount'];
        $invoice_tax_items_param_arry[':tot_cess_amount'] = $invoice_items[$key]['article_cess_amount'];
        $invoice_tax_items_param_arry[':creation_date'] = $creationDateTime;
        $invoice_tax_items_param_arry[':created_by'] = 10000;

        try {
            $invoicetaxitemstmt->execute($invoice_tax_items_param_arry);
            $msg = "Invoice tax item created for  " . $order_no;
            $retval = true;
        } catch (PDOException $e) {
            $msg = "Error creating invoice tax items for order " . $order_no;
            echo $msg . $e->getMessage();
            $retval = false;
            return ($retval);
        }
    }
    return ($retval);

}

function insert_order_history($pdo, $order_hdr)
{

    // insert_order_history

    $creationDate = date('Y-m-d');
    $creationTime = date('H:i:s');

    $history_param_arry = array();
    $history_param_arry[':order_id'] = $order_hdr['order_number'];
    $history_param_arry[':memo'] = "Order Ready To Deliver";
    $history_param_arry[':history_date'] = $creationDate;
    $history_param_arry[':history_time'] = $creationTime;
    $history_param_arry[':status'] = 'A';
    $history_param_arry[':creation_date'] = $creationDate;
    $history_param_arry[':creation_time'] = $creationTime;
    $history_param_arry[':change_date'] = $creationDate;
    $history_param_arry[':change_time'] = $creationTime;
    $history_param_arry[':login_type'] = "Seller";
    $history_param_arry[':user_no'] = 10000;

    try {
        $orderhistorystmt = $pdo->prepare("INSERT INTO `Order_History`(`Order_ID`, `Memo`, `History_Date`, `History_Time`, `Status`, `Creation_Date`, `Creation_Time`, `Created_By`, `Change_Date`, `Change_Time`, `Changed_By`,`Login_Type`)
        VALUES (  :order_id, :memo, :history_date, :history_time, :status, :creation_date, :creation_time, :user_no, :change_date, :change_time, :user_no,:login_type)");

    } catch (PDOException $e) {
        $msg = "Error preparing order history sql statement for order " . $order_hdr['order_number'];
        echo " $msg " . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    try {
        $orderhistorystmt->execute($history_param_arry);
        $msg = "The order history is created for " . $order_hdr['order_number'];
        $retval = true;

    } catch (PDOException $e) {
        $msg = "Error creating order history for order " . $order_hdr['order_number'];
        // echo $msg . $e->getMessage();
        $retval = false;
        return ($retval);
    }

    return ($retval);

}

function update_order_hdr($pdo, $order_hdr)
{

    // update order header

    $orderhdrstmt = '';

    $param_array[':order_no'] = $order_hdr['order_number'];
    $param_array[':orderno'] = $order_hdr['order_number'];
    $param_array[':ext_invoice_no'] = $order_hdr['ext_invoice_no'];
    $param_array[':order_status'] = $order_hdr['order_status'];

    //   $param_array[':order_amount'] = $order_hdr['order_amount'];
    //   $param_array[':net_amount'] = $order_hdr['net_amount'];
    //   $param_array[':saving_amount'] = $order_hdr['saving_amount'];

    try {
        // $orderhdrstmt = $pdo->prepare("UPDATE Order_Hdr
        //                 SET Order_Status = :order_status
        //                     , Ext_Invoice_No = :ext_invoice_no
        //                     , Order_Amount = :order_amount
        //                     , Net_Amount = :net_amount
        //                     , Saving_Amount = :saving_amount
        //                     , Change_Date = now()
        //                     , Change_Time = now()
        //                     , Changed_By = 10000
        //                 where Order_No = :order_no;");

        $orderhdrstmt = $pdo->prepare("UPDATE Order_Hdr oh inner join
        (select  sum(order_item.perunit_wt * order_item.Pick_Quantity) order_wt ,
                sum(order_item.perunit_offer_price * order_item.Pick_Quantity ) order_amt,
                sum(order_item.Item_Discount) header_discount
        from
        ( select Article_No, Pick_Quantity, (  Item_Weight / Pick_Quantity ) perunit_wt, Order_Offer_Price perunit_offer_price, Item_Discount from Order_Itm where Order_No = :orderno ) order_item
        ) x
        set oh.Order_Amount = x.order_amt
        , oh.Total_Weight = x.order_wt
        , oh.Header_Discount = x.header_discount
        , oh.Net_Amount = x.order_amt - x.header_discount
        , oh.Saving_Amount = x.order_amt - ( x.order_amt - x.header_discount )
        , oh.Ext_Invoice_No = :ext_invoice_no
        , oh.Order_Status = :order_status
        , oh.Change_Date = now()
        , oh.Change_Time = now()
        , oh.Changed_By = 10000
        where oh.Order_No = :order_no ;");

    } catch (PDOException $e) {
        $msg = "Error preparing update order header order" . $order_hdr['order_number'];
        echo " $msg " . $e->getMessage();
        $retval = false;
        throw $e;
    }

    try {
        $orderhdrstmt->execute($param_array);
        $msg = "The order header is updated for " . $order_hdr['order_number'] . " ";
        $retval = true;

    } catch (PDOException $e) {
        $msg = "Error updating order header for order " . $order_hdr['order_number'];
        echo $msg . $e->getMessage();
        $retval = false;
        throw $e;
    }

    return ($retval);

}
